package TAPEAgent;
#  $Name:  $ 
#  $Id: TAPEAgent.pm,v 1.12.2.2 2001/04/06 22:12:51 ccadieux Exp $

use strict;
use base 'Agent';

sub isSelectable {"Imaginary Tape"} # displayed by the GUI in front of a checkbox that allows
                                    # admin to turn this Agent off.
sub revision {'$Revision: 1.1 $'}

sub new { # standard to all Agent packages.
	my($self) = Agent->new();
	bless ($self, 'TAPEAgent');
	return $self;
}

sub RUN {
	my($agent) = @_;
	if (!$agent->category_selected) {
		Debug->print2("Skipping " . ref($agent) . ".");
		return;
	}

	my($device, $dc, $wwn, $err, $report);

	my($pdm) = $agent->get_pdm(); # get handle to the PDM
	my($renv) = System->get_renv(); # get environment data-struct
	foreach $device ( $agent->deviceList() ) { # RUN for each tape in rasagent config.
		$dc++;
		Debug->print1("-> TAPEAgent: Reading device $device->{name}");
		($wwn, $err, $report) = $agent->readTape($device) ;
		if ($wwn) {
			Catalog->deviceWrite( $wwn, ['tape', $device->{name},$wwn, $wwn, $device->{ip},
			$device->{ipno},undef, undef ]);
		} else {
			# if accessing the tape fails, get the wwn from the last successful access to the tape.
			$wwn = Catalog->deviceRead('tape', Catalog::Name , $device->{name}, Catalog::Key);
		}
		if (!$wwn) { # still no wwn, give up.
			Debug->err(TEXT => "Cannot identify this Tape, no wwn could be extracted from the tokens");
			next;
		}
		$agent->addIdentification($report); # add device location to the report.
		my $id = { accessName => $device->{ip},
		deviceName => $wwn,
		active => "Y",
		logFile => "",
		name => $device->{name},
		category => Report::CAT_TAPE, # valid category
		ip => "",
		};
		if ($err) { # got a wwn, but error on this run.
			$pdm->saveReport(
			Report->new($id, $report , undef, Report::STATUS_CANNOT_CONNECT));
		} else { # everything is cool.
			$pdm->saveReport(Report->new($id, $report , undef)); # normal status.
		}
	}
	#
	# extract tokens for a specific tape device.
	sub readTape {
		my($agent, $device) = @_;
		my($renv) = System->get_renv();
		my($http_to) = $renv->{'timeout.http'} || 30;
		my($ping_to) = $renv->{'timeout.ping'} || 10;
		if (!Util->testIp($device->{ip}, $ping_to)) {
			Debug->print(PING => "failed on $device->{ip} (to=$ping_to)");
			return (undef, "TAPEAgent: Ping failed on $device->{ip}", undef);
		}
		my $tok = Util::Http->get("http: //$device->{ip}:81/tokens.html", $http_to);
		my @tokens = split(/\n/, $tok);
		my %Report;
		# parse and create categories for the tokens (info, fan, etc..)
		foreach my $line (@tokens) {
			next if (substr($line,0,1) ne ";");
			my @a = split(/ *= */, substr($line,1));
			if (substr($a[0],0,3) eq "fan") {
				$Report{"senseor.fan." . substr($a[0],3)} = $a[1];
			} elsif (index("name,wwn,ip,state", $a[0]) >= 0) {
				$Report{"id.$a[0]"} = $a[1];
			}
			$Report{$a[0]} = $a[1];
		}
		my $wwn = $Report{wwn};

		return ($wwn, undef, \%Report);
	}
}

sub REPORT {
	my($class, $rep) = @_;
	my($name) = $rep->name(); # rep is the Report object,
	my($report) = $rep->value();# $rep->value is the actual hash created previously.

	foreach my $k (keys %$report) {
		print "<tr><td>$k <td> $report->{$k}</td>";
	}
	print "</table>";
}
1;
