package A3500FCAgent;

#  Works with A1000 and A3500fc

#  $Name:  $ 
#  $Id: A3500FCAgent.pm,v 1.16 2001/08/27 17:40:57 ccadieux Exp $
use Agent;
use PDM;
use Carp;
use Catalog;
use vars qw($AUTOLOAD);
use Report;
use strict;
use Debug;
use base 'Agent';
use vars qw( %RAID);

sub isSelectable {"A1000/A3500"}

sub revision {'$Revision: 1.16 $'}

sub new {
  my($self) = Agent->new();

  bless ($self, 'A3500FCAgent');
  return $self;
}
sub DESTROY {}

#
#  healthck -a
#  Health Check Summary Information
#  da106_004:                Failed Drive Channel at Drive [5,0];[5,8]
#  da106_004:        
#  arraymon(1M)    drivutil(1M)  fwutil(1M)     lad(1M)
#  logutil (1M)    nvutil(1M)    parityck(1M)   raidutil(1M)
#  rdacutil(1M)    rmevent(4)    rmparams(4)    rmscript(1M)
#  storutil(1M)
#

sub getPortKey {
  my($class, $wwn, $port) = @_;
  my($VAR1);
  my $ci = Catalog->deviceRead('a3500fc', Catalog::Key, $wwn);
  if (!$ci) {
     Debug->print2("A35:getPortKey: Cannot find key for $wwn:$port");
     return undef;
  }
  if ($port == 0) {
     return $ci->[Catalog::Port1];
  } else {
     return $ci->[Catalog::Port2];
  }
}


sub RUN {
  my($agent, $ras_flag) = @_;
  my( $dev_name, $com, %SYS, $DIR);
  my( @devices, @all_dif);
  my($sys_xml, $state_xml, $raid, @a, $dev, $log_err, $lines);
  my($pdm) = $agent->{pdm};
  my($renv) = System->get_renv();

  if (!$agent->category_selected) {
     Debug->print2("Skipping " . ref($agent)  . ".");
     return;
  }
  $DB::single = 1;
  $DIR = $agent->find_raid;
  if (!$DIR && !System->get_testMode) {
     Debug->err(CANNOT_RUN => "raid", "Cannot find RAID commands");
     return;
  }
  Timelapse->start(ref($agent));

  my($logfile)   = $agent->{renv}->{logfile};
  if ($logfile) {
    ($log_err, $lines) = $agent->read_log_file($logfile,'NEW');
    Debug->err('CANNOT_READ', $logfile) if ($log_err);
  }
  
  my($log_lines, $x) ;
  for ($x=0; $x <= $#$lines; $x++) {
      $log_lines .= $agent->read_thresholds(\$x, $lines);
  }

  my($dc) = 0;
  my($category);

  foreach $dev ( $agent->deviceList() ) {
     next if (!Util->isMineToMonitor($dev));
     my $wwn;
     if ($dev->{type} eq "a3500fc") {
        $category = Report::CAT_A3500FC;
        $wwn  = "a3-" . $dev->{name};

     } elsif ($dev->{type} eq "a1000") {
        $wwn  = "a1-" . $dev->{name};
        $category = Report::CAT_A1000;
     } else {
       next;
     }

     $dc++;
     $dev_name = $dev->{name};
     Debug->print1("-> $dev->{type}: Reading device $wwn");
     if (!$wwn) {
        Debug->print2("No ctrl_id for $dev_name, skipping");
        next;
     }
     my($sys_report, $ctrl1, $ctrl2) = $agent->sonoma($dev_name, $DIR);
     my($snA) = $sys_report->{'device_info.a.Vendor_ID'};
     my($snB) = $sys_report->{'device_info.b.Vendor_ID'};
     Catalog->deviceWrite($wwn, 
            [$dev->{type}, $dev->{name}, $ctrl1, $ctrl2, undef, undef,
             "$snA.$ctrl1", "$snB.$ctrl2" ]);

     my($id) = {
                accessName=> $dev_name,
                deviceName=> $wwn,
                active    => $dev->{active},
                name      => $dev_name,
                display   => $wwn,
                category  => $category,
                ip        => "" ,
               };
     my($pdm) = $agent->{pdm};
     $sys_report->{'id.name'} = $dev_name;
     $sys_report->{"id.ipno"} = System->get_ipno();

     if ($sys_report) {
       $sys_report->{'id.device'}  = $dev_name;
       $sys_report->{'id.ctrl_id'} = $wwn;
       $pdm->saveReport(Report->new($id, $sys_report, $log_lines ) );
     } else {
       $pdm->saveReport(Report->new($id, 
		$sys_report,$log_lines, Report::STATUS_CANNOT_CONNECT ));
     }
     $log_lines = ""; # use only once
  }

  
  Debug->print2("  No devices found") if (!$dc);
  Timelapse->stop(ref($agent));
}

sub read_thresholds {
  my($agent,$x, $array) = @_;

  my($line) = $array->[$$x];
  my($out);

  if ($line =~ /controller.*A3500.*recommended firmware version/) {
     $out .= $agent->log_inc('CTRL_FIRM', $line);
  }

  return $out;
}

sub log_inc {
  my($agent, $type, $mline) = @_;
  my($data, $thr, $th, $val, $t1, $today, $hrs);

  my(@m) = split(/ +/, $mline);
  my($date) = "$m[0] $m[1] $m[2]";
  my($info) = "$m[3] $m[4] $m[5] $m[6]";
  $info =~ s/"/ /g;
  my($pdm) = $agent->{pdm};
  my($level, $cnt, $desc, $mins) = $pdm->threshold()->test('a3500',$type, 0,1);

  my($tx) = "Received $cnt $desc messages";
  $tx .= " in $mins mins" if ($mins);
  if ($level eq "W") {
     return "WARN\t$date\t$tx\t$info\n";
  } elsif ($level eq "E") {
     return "ERR\t$date\t$tx\t$info\n";
  }
}




sub sonoma {
  my($agent, $dev_name, $DIR) = @_;
  my(%SYS, $in, $found, $l, $id, $value, $x, $ix, $s, $l1, @b, $k, $xx);
  my($lun_cnt) = 0;
  my($TO) = $agent->{renv}{'timeout.rm6'};
  my(@SER);

   my($err,$health) = Util->run_command( "$DIR/healthck $dev_name", "lad.txt", $TO);
   my($hinfo);

   for ($x=1; $x < $#$health; $x++) {
      if ($health->[$x] =~ /$dev_name: +(.*)/) {
         $SYS{"device_info.healthck"} .= $1 . ",";
      }
   }

   my($err,$com) = Util->run_command( "$DIR/lad", "lad.txt", $TO);

   my(%X);
   for ($x=0; $x <= $#$com; $x++) {
      my(@a) = split(/\s+/, $com->[$x]);
      $X{$a[0]} = $a[1];
   }

# RDACUTIL (only once per enclosure)

   my($err,$com) = Util->run_command(
       "$DIR/rdacutil -i $dev_name", "rdacutil.txt", $TO);
   $in = "";
   $s = $com->[1];
   $ix = index($s, ":");
   $SYS{'device_info.mode'} = Util->trim(substr($s,$ix+1));
   my(@devices) ;

   for ($x=2; $x <= $#$com; $x++) {
      $l = $com->[$x];
      if ($l =~ / +(.*) +controller ([^\(]+) \((\w+)\) +units: (.*)/) {
        $SYS{"controller.count"}++;
        $SYS{"controller.$2.state"} = Util->rtrim($1);
        $SYS{"controller.$2.units"}   = Util->trim($4);
        $SYS{"controller.$2.device"}  = $3;
        $SYS{"controller.$2.serial"}  = $X{$3};
        push(@devices, $3);
        push(@SER,  $X{$3});
#        $SYS{"controller.$2.id"}      = $2;
      }
   }

   for ($xx=0; $xx <= $#devices; $xx++) {
     my($devName) = $devices[$xx];
     my($devAB) = ($xx==0) ? "a":"b";

#  RAIDUTIL  (run on each device)
     my($err,$com) = Util->run_command( 
       "$DIR/raidutil -c $devName -B -i", "raidutil.txt", $TO);

     if ($err || "$com->[0] $com->[1]" =~ /No RAID devices found/) {
        Debug->print2("Error: $err / $com->[0]");
        return undef;
     }
     $in = ""; $found = 0;

     foreach $l (@$com) {
        if ($l =~ /LUNs found/) {      
            $in = "lun"; $found = 1;
            $l =~ /found on (.+)\./;
            my($vol) = $1;
            $SYS{'id.vol'} = $vol;

        } elsif ($l =~ /Battery age is (.*)/) {
            $SYS{"device_info.$devAB.battery"} = $1;
            $in = "info";
        } elsif ($l =~ /succeeded!/) {
            $in = "";
        } elsif ($in eq "lun") {
            $l =~ /LUN (\d+) +RAID (\d+) +(\d.*)/;
            $SYS{"lun.$1.raid"} = $2;
            $SYS{"lun.$1.size"} = $3;
            $lun_cnt = $1 if ($1 > $lun_cnt);
        } elsif ($in eq "info") {
            if (length($l) > 0) {
               $id = Util->rtrim(substr($l,0,17));
               $value = Util->rtrim(substr($l,18));
               $id =~ s/ /_/g;
               $SYS{"device_info.$devAB.$id"} = $value;
            }
        }
     }
     $SYS{"lun.count"} = $lun_cnt+1;
     

# DRIVUTIL -L (per enclusure)
    if ($xx == 0) {  # only on the first one
       my($err,$com) = Util->run_command(
       "$DIR/drivutil -l $devName", "drivutil_l.txt", $TO);

       for ($x=5; $x <= $#$com; $x++) {
         $l = $com->[$x];
         $l =~ /  (\d+) +(\d+) +(\w+) +(\d+) +(\d+) +(.*)/;
         $SYS{"lun.$1.group"}      = $2;
         $SYS{"lun.$1.device"}     = $3;
         $SYS{"lun.$1.raid_level"} = $4;
         $SYS{"lun.$1.capacity"}   = $5;
         $SYS{"lun.$1.status"}   = Util->rtrim($6);
#         $SYS{"lun.$1.id"}         = $1;
       }

# DRIVUTIL -i (per enclusure)
#Location  Capacity   Status         Vendor  Product          Firmware     Serial
#            (MB)                              ID             Version      Number
#[1,0]     34732      Optimal        SEAGATE ST136403LSUN36G  034A         LT0418

       my($err,$com) = Util->run_command(
       "$DIR/drivutil -i $dev_name", "drivutil_i.txt", $TO);

       my($no) = 0;
       for ($x=6; $x <= $#$com; $x++) {
         $l = $com->[$x];
         next if (substr($l,0,1) ne "[");
         $SYS{"disk.$no.group"}    = Util->rtrim(substr($l,0,9));
         $SYS{"disk.$no.capacity"} = Util->rtrim(substr($l,10,9));
         $SYS{"disk.$no.status"}   = Util->rtrim(substr($l,21,11));
#         $SYS{"disk.$no.id"}       = $no;
         $l1 = substr($l,36);
         @b = split(/ +/, $l1);
         $SYS{"disk.$no.vendor"}   = $b[0];
         $SYS{"disk.$no.product_id"}= $b[1];
         $SYS{"disk.$no.firmware_version"} = $b[2];
         $SYS{"disk.$no.serial"}    = $b[3];
         $no++;
       }
       $SYS{"disk.count"}   = $no;
     
# DRIVUTIL -D (per enclusure)
       my($err,$com) = Util->run_command(
       "$DIR/drivutil -d $dev_name", "drivutil_d.txt", $TO);

       foreach $l (@$com) {
         if ($l =~ /(Hot Spare)/ || $l =~ /(Group +\d+):/) {
           $k = $1; $k =~ s/ +/_/;
           $SYS{"drive_grouping.$k"} = Util->trim(substr($l,11));
         }
       }
     }
   }

   $agent->addIdentification(\%SYS);
  
   Debug->dump('a3500Config', \%SYS);
   return (\%SYS, $SER[0], $SER[1]);
          
}

sub find_raid {
  my($agent) = @_;
  my($DIR);

  if (!System->get_testMode() ) {
    if (-d "/usr/lib/osa/bin") {
       $DIR = "/usr/lib/osa/bin";
    } elsif (-d "/usr/sbin/osa") {
       $DIR = "/usr/sbin/osa";
    } else {
       Debug->err('NO_RAID_MGR', ref($agent). " on " . System->hostname());
       return undef;
    }
  }
  return $DIR;
}


#
#   $cnt = 0;
#   no tableCnt: separate report.
#   with tableCnt: many device in same table.
#   $agent->REPORT($host, $report, {tableCnt => \$cnt};

sub REPORT {
  my($class, $host, $r, $arg) = @_;
  my($out); 

  my($name) = $r->{_id}{name};
  print "<table border=0 cellpadding=3 width=100% bgcolor=#666699>
          <tr><td><font color=white><b>A3500FC: $name</table>";
  my($tableCnt) = $arg->{tableCnt};
  if (!$tableCnt || $$tableCnt == 0) {
    $out .= "<table border=1 cellspacing=0 cellpadding=1 width=100% bgcolor=white>";
  }
  $out .= "
  <tr bgcolor=#E0E0E0>
    <th><font color=black>Type</th>
    <th><font color=black>Identification</th>
    <th><font color=black>Component</th>
    <th><font color=black>Status</th>
    <th><font color=black>Ports</th>
    <th><font color=black>Revision</th>
   ";

#  my($mon) = &mon('a3k',$name);
  my($v) = $r->{_value};

  $out .= "<tr><td colspan=6>&nbsp;<b>$r->{_id}{category} on $host &nbsp;" .
          "<b>Name: $r->{_id}{name}&nbsp; [$r->{_id}{deviceName}]&nbsp;</td>\n";

  my($d, $x, $comp, $b, $status, $serial, $ports, $rev);
  my($cnt)= 0;
  foreach $x ('a','b') {
      $comp = "controller.$x";
      $status = $v->{"$comp.state"};
      $b = ($v->{"$comp.state"} eq "Active")? "":"<font color=red>";
      $serial = $v->{"$comp.device"};
      $out .= 
           "<tr><td><center>Controller".
           "<td>$serial".
           "<td>$x".
           "</td><td>$b$status</td><td>&nbsp;</td><td>&nbsp;</td>\n";

      $cnt++;
  }
  for ($x=0; $x <= 9; $x++ ){
      $comp = "disk.$x";
      $status = $v->{"$comp.status"};
      $b = ($v->{"$comp.status"} eq "Optimal")? "":"<font color=red>";
      $serial = $v->{"$comp.product_id"} . "/ " . $v->{"$comp.serial"};
      $ports = $v->{"$comp.group"};
      $rev = $v->{"$comp.firmware_version"};
      $out .=  
           "<tr><td><center>Disk".
           "<td>$serial".
           "<td>$x".
           "</td><td>$b$status</td><td>$ports</td><td>$rev</td>\n";
      $cnt++;
  }
  if (!$tableCnt) {
    $out .= "</table>";
  }
  return $out;
}

1;

