package T3Agent;
#  $Name:  $ 
#  $Id: T3Agent.pm,v 1.51 2001/08/24 21:34:19 ccadieux Exp $

use Agent;
use Debug;
use Carp;
use Util;
use Catalog;
use vars qw($AUTOLOAD);
use Debug;
use Report;
use Util::Http;
use strict;
use base 'Agent';

use Net::Telnet;
use Data::Dumper;

sub isSelectable {"T3"}
sub revision {'$Revision: 1.51 $'}

use vars qw(%N2IP);

sub new {
  my($self) = Agent->new();

  bless ($self, 'T3Agent');
  return $self;
}
sub DESTROY {}

sub getPortKey {
  my($class, $key, $port) = @_;
  my($VAR1);

  my $ci = Catalog->deviceRead('t3', Catalog::Key, $key);

  if (!$ci) {
     Debug->print2("getPortKey: Cannot find key for $key:$port");
     return undef;
  }

  if ($port == 0) {
     return $ci->[Catalog::Port1];
  } else {
     return $ci->[Catalog::Port2];
  }
}

sub RUN {
  my($agent,$ras_flag) = @_;
  my($log_lines,  $out,$device, $connect_errs);
  my($report ,  $id, $lines, $log_err, $type);
  my($onedayold, $wwn, $portWWN, $port2WWN);
  if (!$agent->category_selected) {
     Debug->print2("Skipping " . ref($agent)  . ".");
     return;
  }
  $DB::single = 1;
  my($pdm) = $agent->{pdm};
 
  Timelapse->start("T3MESSAGEAgent");

  my($renv)    = $agent->{renv};
  my($logfile) = $renv->{t300logfile};

  my $ISP;
  if ($logfile && -r $logfile) {
     ($log_err, $lines) = $agent->read_log_file($logfile,'NEW');  
     Debug->err('CANNOT_READ', $logfile) if ($log_err);

     foreach my $line (@$lines) {
         $log_lines .= &find_alert($line, $device, \$ISP);
     }
     if ($ISP) {
        $log_lines = "Found $ISP ISP2100[] messages.\n" . $log_lines;
     }
     my($name) = $renv->{hostname} || System->hostname();
     if ($log_lines) {
       $id = {accessName  => $name,
                deviceName  =>  System->hostid(),
                active      => "Y",
                logFile     => $logfile,
                display     => "$name (id=" . System->hostid() . ")",
                name        => $name, 
                category    => Report::CAT_T3MESSAGE,
                ip          => "",
           };
       $report->{"id.name"} = $name;

       $pdm->saveReport( 
               Report->new($id, $report , $log_lines, Report::STATUS_OK));
     }
  }
  Timelapse->stop("T3MESSAGEAgent");

  Timelapse->start(ref($agent));

  my($sdt);
  my($dc) = 0;
  my $syslog_info  = System->grep("/etc/syslog.conf", "\.t3");
  my (%IB);
  foreach $device ( $agent->deviceList() ) {
     next if (substr($device->{datahost},0,1) eq "*");
     my $h   = $device->{datahost} || System->hostname();
     $IB{$h} = 1;
  }
  foreach my $dh (keys %IB) {
     $IB{$dh} = Util->getFCWWNs($dh);
  }
  $onedayold = 1 if $pdm->isXdays('sent_t300_stats', 1);

  foreach $device ( $agent->deviceList() ) {

     $dc++;
     $id  = $device->{_name};
     if (($sdt = Util->sdt($device)) ) {
        Debug->print('SDT', $sdt);
        next;
     }
     Debug->print1("-> Processing T3:$device->{name}/$device->{ip}");
  
     if (!$device->{ip}) {
        Debug->err( 'IP_NOT_FOUND',"T3 at $device->{name}" );
        next;
     }

     ($portWWN, $port2WWN, $wwn, $connect_errs, $report) = 
                                         $agent->read_purple($device, $onedayold);
     $wwn = lc($wwn);

     my($ctrl1) = $report->{'controller.u1ctr.fruVendor'} . "." . 
                  $report->{'controller.u1ctr.fruModel'} . "." .
                  $report->{'controller.u1ctr.fruSerialNo'};

     my($ctrl2) = $report->{'controller.u2ctr.fruVendor'} . "." . 
                  $report->{'controller.u2ctr.fruModel'} . "." .
                  $report->{'controller.u2ctr.fruSerialNo'};

     if ($portWWN) {
        Catalog->deviceWrite( $wwn, ['t3', $device->{name}, lc($portWWN),
        lc($port2WWN), $device->{ip}, $device->{ipno}, $ctrl1, $ctrl2 ]);
     } else {
        $wwn     = Catalog->deviceRead('t3', Catalog::Name , $device->{name}, Catalog::Key);
        $portWWN = Catalog->deviceRead('t3', Catalog::Name , $device->{name}, Catalog::WWN1);
     }

     if (!$wwn) {
       Debug->err(TEXT => "Cannot identify this T3, skipping ");
       next;
     }

     my $t1 = $device->{ipno} || $device->{ip};
     $id =  { accessName  => $device->{ip},
              deviceName  => $wwn,
              active      => $device->{active},
              logFile     => $logfile,
              name        => $device->{name},
              display     => "$device->{name} (ip=$t1)",
              category    => Report::CAT_T3,
              ip          => $device->{ip},
            };

     my $dh = $device->{datahost} || System->hostname();

     $report->{"info.IB_host"} = $dh;

     my $nodeWWN = $portWWN;
     my $node2WWN = $port2WWN;
     substr($nodeWWN, 7,1) = "0"  if (length($nodeWWN) > 7);
     substr($node2WWN, 7,1) = "0" if (length($node2WWN) > 7);

     if (substr($device->{datahost},0,1) ne "*") {
       if ( ($nodeWWN  && index(lc($IB{$dh}), lc($nodeWWN )) >= 0) ||
            ($node2WWN && index(lc($IB{$dh}), lc($node2WWN)) >= 0) ) {
          $report->{"info.IB_status"} = Report::STATUS_OK;
       } else {
          $report->{"info.IB_status"} = Report::STATUS_CANNOT_CONNECT;
       }
     }

     $report->{"id.name"}        = $device->{name};
     $report->{"id.ip"}          = $device->{ip};
     $report->{"id.ipno"}        = $device->{ipno};
     $report->{"info.syslog"}    = $syslog_info;
     $report->{"info.OOB_date"} = Util->get_today();

     if ($connect_errs || !$wwn) {
        $report->{"info.OOB_connection"} = $connect_errs;
        $pdm->saveReport(
          Report->new($id, $report , "", Report::STATUS_CANNOT_CONNECT));

     } else {
        $report->{"info.OOB_connection"} = "OK";
        $pdm->saveReport(Report->new($id, $report , ""));
     }
  }
  Debug->print2("  No devices found") if (!$dc);
  Timelapse->stop(ref($agent));
}


############################################################
#                    SUBROUTINES
############################################################


%N2IP = ();

sub find_alert {
  my($line, $device, $ISP) = @_;
  my( $level, $name, $no1, $no2, $rest);
  my($m, $year, $date, $name, $comp);
  my($ipno);

  my(@a) = split(/\s+/, $line);
   
   if ($line =~ /: W: (.*)/) {
        $level = 2;

   } elsif ($line =~ /: E: (.*)/) {
        $level = 1;

   } elsif ($line =~ /: WARNING: / ) {
        $level = 1;

   } elsif ($line =~ /ISP2\d\d\d/ ) {
        $$ISP = $$ISP + 1;
        $level = 0;

   } elsif ($line =~ /: [NI]: u\dd\d/ ) {
        $level = 2;

   } elsif ($line =~ /: [N]: u\dpcu\d.*Battery .*OK/ ) {
        $level = 2;

   } elsif ($line =~ /: [N]: u\dpcu\d.*PCU\d hold time/) {
        $level = 2;

   }
   if ($level) {
     return "$line\n";
   } else {
     return "";
   }
}




#######################################################
#      READ/PARSE HTTP TOKENS
#######################################################
# ;u1ctr,fruCtlrPartnerId,u2ctr;

sub read_purple {
  my($agent, $device,  $onedayold) = @_;
  my($fru_date, $mode);
  my($connect_err, $stats);
  my($config, $config_errors);
  my($l, $config2, $config_rep, $stats_rep);
  my($sysprop, $elemprop0, $elemprop1, $key , $portWWN, $port2WWN);

  my($renv)   = $agent->{renv};
  my($ping_to) = $renv->{'timeout.ping'} || 10;
  my($http_to) = $renv->{'timeout.http'} || 30;
  my($id)     = $device->{_name};
  my($ip)     = $device->{ip};
  my($unit)   = $device->{name};
  my($home)   = System->get_home();

  if (System->get_testMode() ) {
    $elemprop0 = Util->readf("$home/Test/t300.elemprop");
  } else {
    if (!Util->testIp($device->{ip},$ping_to)) {
       Debug->print(PING => "failed on $device->{ip} (to=$ping_to)");
       return (undef,undef,undef, "T3Agent: Ping failed on $device->{ip}", undef);
    }
       
    $elemprop0 = Util::Http->get(
       "http://guest\@$device->{ip}/elemprop.htm?unitIndex=0",$http_to);
    $elemprop1 = Util::Http->get(
       "http://guest\@$device->{ip}/elemprop.htm?unitIndex=1",$http_to);
  }
  if ($elemprop0) {
     ($portWWN, $port2WWN, $key, $config_rep) = $agent->parse_elem($elemprop0, $elemprop1, $device);
  } else {
     Debug->err('CANNOT_HTTP', $device->{ip});
     return (undef, undef, undef, "T3Agent: Empty elemprop $device->{ip}", undef);
  }

  if ($onedayold) {
    if (System->get_testMode() ) {
       $sysprop = Util->readf("$home/Test/t300.sysprop");
    } else {
       $sysprop = Util::Http->get(
             "http://guest\@$device->{ip}/sysprop.htm",$http_to);
    }
    if ($sysprop) {
       $stats_rep = $agent->parse_sys($sysprop);
       my($cache) = PDM->getCacheHandle('t300_stats');
       $cache->{$ip} = $stats_rep;
    } else {
       Debug->err('CANNOT_HTTP', $device->{ip});
       $connect_err = "T300Agent: Empty Sysprop $device->{ip}";
    }
  } else {
    my($cache) = PDM->getCacheHandle('t300_stats');
    $stats_rep = $cache->{$ip};
  }
  foreach my $k (keys %$stats_rep) {
      $config_rep->{$k} = $stats_rep->{$k};
  }
    
  return ($portWWN, $port2WWN, $key, $connect_err, $config_rep);
}

sub parse_elem {
  my($agent, $sys, $partner, $device) = @_;
  my(%V, $l);
  my($in, $disk_cnt, $ctrl_cnt, $type);
  my(@f) = split(/\n/, $sys);
  my(@f2) = split(/\n/, $partner);

  if ($#f2 > 30) {  # add the partner info to the main array.
    $in = 0;
    foreach $l (@f2) {
      if (lc($l) =~ /-- unit info/) {
         $in = 1;
      } elsif ($in) {
         push(@f, $l);
      }
    }
  }

  $in = 0; $disk_cnt = 0; $ctrl_cnt = 0;
  foreach $l (@f) {
     if (lc($l) =~ /-- unit info/) {
        $in = 1;
     } elsif ($in) {
        my(@b) = split(/,/, $l);
        if (substr($l,0,1)  eq ";" && $#b > 1)  {
          chop($b[2]) if (substr($b[2],-1) eq ";");
          $b[2] =~ s/ +/ /g;
          $b[2] =~ s/\c[//g;
          $b[0] = substr($b[0],1);
          $type = "";
          if (substr($b[0],0,2) eq "un") {
              $type = "unit";
          }elsif (substr($b[0],2,1) eq "d") {
              $type = "disk";
              $disk_cnt = substr($b[0],3,1) if (substr($b[0],3,1) > $disk_cnt);
          }elsif (substr($b[0],2,1) eq "c") {
              $type = "controller";
              $ctrl_cnt = substr($b[0],1,1) if (substr($b[0],1,1) > $ctrl_cnt);
          }elsif (substr($b[0],2,1) eq "l") {
              $type = "loopcard";
          }elsif (substr($b[0],2,2) eq "pc") {
              $type = "power";
          }elsif (substr($b[0],2,1) eq "p") {
              $type = "port";
          }elsif (substr($b[0],2,1) eq "m") {
              $type = "midplane";
          }elsif (substr($b[0],2,1) eq "v") {
              $type = (substr($b[0],6,1) eq "-")? "volume_disk":"volume";
          }         
# l=;u1ctr,fruId,u1ctr; $b[0]=u1ctr, $b[1]=fruId, $b[2] = u1ctr
          
          
          $V{"$type.$b[0].$b[1]"} = Util->rtrim($b[2]);
        }
     } 
  }
  $V{"disk.count"} = $disk_cnt;
  $V{"controller.count"} = $ctrl_cnt;
  $V{"volume.u1vol1.volWWN"} =~ s/00000000000000/0/g;
  my($portWWN, $port2WWN);

  $portWWN  = $V{"port.u1p1.portWWN"};
  $port2WWN = $V{"port.u2p1.portWWN"};

#  substr($portWWN,-9,1) = "0" if (length($portWWN) > 10);
#  substr($port2WWN,-9,1) = "0" if (length($port2WWN) > 10);

  my($key);
  if ( $V{"midplane.u1mpn.fruVendor"} ) {
    $key = $V{"midplane.u1mpn.fruVendor"} . "." .  
           $V{"midplane.u1mpn.fruModel"} . "." .
           $V{"midplane.u1mpn.fruSerialNo"};
    $key =~ s/[^\w\-\.]/_/g;
  } else {
    $key = $device->{ipno};   # $portWWN;
  }
  $key = lc($key);

  $V{"id.wwn"} = $key;

  $V{"loopcard.header"} = "";
  $V{"midplane.header"} = "";
  $V{"power.header"} = "";
  $V{"port.header"} = "";
  $V{"unit.header"} = "";

# fix the missing serialNo
  foreach my $k ('u1pcu1','u1pcu2','u2pcu1','u2pcu2') {
    if ($V{"power.$k.fruSerialNo"} =~ /refer to/) {
       $V{"power.$k.fruSerialNo"} = "$key.$k";
    }
  }

  Debug->dump('T3Config'  , \%V);
  $agent->addIdentification(\%V);

  return ($portWWN,$port2WWN, $key, \%V);

}

sub parse_sys {
  my($agent, $sys) = @_;
  my(%V);
  my( $l, @b);

  my(@f) = split(/\n/, $sys);

  my($in) = 0;
  foreach $l (@f) {
     if (lc($l) =~ /-- system info/) {
        $in = 1;
     } elsif ($in) {
        @b = split(/,/, $l);
        if ($#b > 1) {
          chop($b[2]);
          $b[2] =~ s/ +/ /g;
          $b[2] =~ s/\c[//g;
          $V{"system.$b[1]"} = Util->rtrim($b[2]);
        }
     } 
  }

  Debug->dump('T3System', \%V);
  return \%V;
}
  

sub REPORT {
  my($class, $host, $r, $arg) = @_;

  my($name) = $r->{_id}{name};
  my($comm);
  my($out);
  my($tableCnt) = $arg->{tableCnt};
  my($host0) = $host || System->hostname();
  $comm = "<br><font color=red>Management-Path Lost!</font>" if ($r->{_status} eq "CC");

  my($v) = $r->{_value};
  $out .= "
  <table border=1 cellspacing=0 width=100% bgcolor=white>
   <tr><td colspan=4 bgcolor=#666699>
     <table border=0><tr><td>
     <font color=white><b>&nbsp;T3 $r->{_id}{name} ($host)</td>
     </table></td>
   <tr><td bgcolor=#F0F0F0 align=right><b>Name:</td><td width=25%>&nbsp;$r->{_id}{name}</td>
       <td bgcolor=#F0F0F0 align=right><b>Monitored on:</td><td>&nbsp;$host0</td>

   <tr><td bgcolor=#F0F0F0 align=right><b>WWN:</td><td>&nbsp;$r->{_id}{deviceName}</td>
       <td bgcolor=#F0F0F0 align=right><b>Comm:</td><td>&nbsp;$comm</td>
   <tr><td bgcolor=#F0F0F0 align=right><b>IP:</td><td>&nbsp;$v->{'system.sysIpAddr'}</td>
       <td bgcolor=#F0F0F0 align=right><b>&nbsp;</td><td>&nbsp;</td>
  </table>
  <table border=1 cellspacing=0 width=100% bgcolor=white>

  <tr bgcolor=#E0E0E0>
    <th><font color=black>Type</th>
    <th><font color=black>Identification</th>
    <th><font color=black>Component</th>
    <th><font color=black>Status</th>
    <th><font color=black>Revision</th>
  ";

#  my($mon) = &mon('t3',$name);

  my($cnt, $u, $d, $comp, $status, $id, $rev, $type, $bat, $fans);
  $cnt = 0;
  for ($u=1; $u <= 2; $u++) {
     $out .= "<tr><td colspan=5 ><b>&nbsp;Unit $u</td>";
     last if (!$v->{"disk.u${u}d1.fruId"});
     for ($d=1; $d < 10; $d++) {
        $comp="disk.u${u}d$d";
        $status = $v->{"$comp.fruStatus"} . "-" . $v->{"$comp.fruState"};
        $id = lc($v->{"$comp.fruVendor"}) . "/ " . $v->{"$comp.fruModel"} . 
              "/ " .  $v->{"$comp.fruSerialNo"};
        $rev = $v->{"$comp.fruRevision"};
        $out .=  
            "<tr><td><center>Disk".
            "<td>$id".
            "<td>u${u}d$d".
            "</td><td>$status</td><td>$rev</td>\n";

        $cnt++;
     }
     $id =  $v->{"controller.u${u}ctr.fruVendor"} . "/ " .
            $v->{"controller.u${u}ctr.fruModel"} . "/ " .
           $v->{"controller.u${u}ctr.fruSerialNo"};

     $status = $v->{"controller.u${u}ctr.fruStatus"} . "-" . 
               $v->{"controller.u${u}ctr.fruState"};
     $rev = $v->{"controller.u${u}ctr.fruRevision"};
     $out .= "\n<tr><td><center>Cntrl<td>$id<td>u$u.ctrl<td>$status<td>$rev</td>";

     $id = $v->{"midplane.u${u}mpn.fruSerialNo"};
     $status = $v->{"midplane.u${u}mpn.fruStatus"} . "-" . $v->{"midplane.u${u}mpn.fruState"};
     $rev = $v->{"midplane.u${u}mpn.fruRevision"};
     $out .= "\n<tr><td><center>Midplane<td>&nbsp;<td>u$u.mpn<td>$status<td>$rev&nbsp;</td>";

   for ($d=1; $d<= 2; $d++) {
        $comp = "loopcard.u${u}l$d";
        $status = $v->{"$comp.fruStatus"} . "-" . $v->{"$comp.fruState"};
        $id = $v->{"$comp.fruSerialNo"};
        $rev = $v->{"$comp.fruRevision"};
        $out .= "<tr><td><center>Loop<td>$id<td>u${u}l$d<td>$status</td><td>$rev</td>\n";
     }

     for ($d=1; $d<= 1; $d++) {
        $comp = "port.u${u}p$d";
        $status = $v->{"$comp.portStatus"};
        $id  = $v->{"$comp.portWWN"};
        $rev = $v->{"$comp.fruRevision"};
        $type = $v->{"$comp.portType"};
        $out .= "<tr><td><center>Port<td>$id<td>u${u}p$d</td><td>$status</td><td>$rev&nbsp;</td>\n";
     }

     for ($d=1; $d<= 2; $d++) {
        $comp = "power.u${u}pcu$d";
        $status = $v->{"$comp.fruStatus"} . "-" . $v->{"$comp.fruState"};
        $id = lc($v->{"$comp.fruVendor"}) . "- " . $v->{"$comp.fruModel"};
        $bat = $v->{"$comp.fruPowerBatState"};
        $fans = "fan1:" . $v->{"$comp.fruPowerFan1State"} . ", fan2:" . $v->{"$comp.fruPowerFan2State"};
        $rev = $v->{"$comp.fruRevision"};
        $out .= "<tr><td><center>Power<td>$id<td>u${u}pcu$d<td>$status</td><td>$rev&nbsp;</td>\n";
        $out .= "<tr><td colspan=1>&nbsp;<td colspan=4>Battery:$bat, $fans</td>\n";
     }
  }
  if (!$tableCnt) {
    $out .= "</table>";
  }
  return $out;

}


# save = 1 : use cache for all
# save = 0 : use cache for discovery stuff
# save = 2 : don't use cache at all
# DISK LINKFAIL LOSSSYNC LOSSSIG PROTOERR INVTXWORD INVCRC
# --------------------------------------------------------
# u1d1  1        13       0       0        145       0       
#

sub FCCounters {
  my($class ,$arg) = @_;
  my(%ENC, $disks, $in, $l, $node, %WWN, @HBA, $num, $x, %ALPA, %MPX);

  my( $cache1, $cache2);
  if ($arg->{cache} == 1) {
     $cache1 = $cache2 = 1;
  } elsif ($arg->{cache} == 2) {
     $cache1 = $cache2 = 0;
  } else {
     $cache1 = 1; $cache2 = 0;
  }
  my($confDevs) = $arg->{devs}; 
  my($report) = 1 if ($arg->{report});
  my($encs)   = $arg->{encs};

  my($lux) = "/usr/sbin/luxadm";
  my($err,$probe);
  
#
# Find ENCLOSURES AND PATHS with PROBE
#
  $err = $class->readT3IB($cache1, \%ENC, \@HBA, \%MPX);

  if (defined($encs)) {
    %ENC = ();
    my(@a) = split(/,/, $encs);
    foreach my $enc (@a) {
        $ENC{$enc}  = 1;
    }
  }
  
  if (1) {

  foreach my $wwn (keys %ENC) {
    $WWN{$wwn}{enc} = $wwn ; # $ENC{$wwn};
    if ($confDevs) {
       foreach my $d (@$confDevs) {
          if ($d->{type} eq "t3" && 
               lc(substr($d->{wwn},-4)) eq lc(substr($wwn,-4))) {
               $WWN{$wwn}{enc} = $d->{name};
          }
       }
    } 
  }
  }

#
# FOR EACH HBA, GET COUNTERS 
#
  my($map, $err2, $rdls, @c, $hba);
  for ($x=0; $x <= $#HBA; $x++ ) {
     $hba = $HBA[$x];
     Debug->print2("FC-T3 : reading hba $x ($hba)");
     my($err,$map) = FC->run_command("$lux -e dump_map","", $hba, $cache2);
     my($last) = substr($hba,-1);
     my($err2,$rdls) = FC->run_command("$lux -e rdls",$last, $hba, $cache2);
     $in =0;
     my($fabric) = 0;
     foreach $l (@$map) {
        next if ($l =~ /^ *$/);
        if ($l =~ /^Pos/) {
            $in = 1;
            $fabric = 1 if ($l =~ /Port_ID/);
        } elsif ($in) {
          my($n,$alpa,$id,$hard,$port,$node,$type,$desc);
          if ($fabric) {
            ($n,$alpa,$hard,$port,$node,$type,$desc) = split(/\s+/, $l,7) ;
          } else {
            ($n,$alpa,$id,$hard,$port,$node,$type,$desc) = split(/\s+/, $l,8) ;
          }
          $alpa = sprintf("%02s", $alpa);
          $ALPA{$x}{$alpa}{port} = $port; 
          $ALPA{$x}{$alpa}{node} = $node;  # wwn
          $ALPA{$x}{$alpa}{type} = $type;
          $ALPA{$x}{$alpa}{desc} = $desc;
        }
     }
     $in =0; 
     foreach $l (@$rdls) {
        next if ($l =~ /^ *$/);
        if ($l =~ /^al_pa/) {
           $in = 1;
        } elsif ($l =~ /^NOTE:/) {
           $in = 0;
        } elsif ($in) {
           my($alpa,$lnk,$sync,$signal,$seq,$word,$crc) = split(/\s+/, $l);
           $alpa = sprintf("%02s", $alpa);
           $ALPA{$x}{$alpa}{link}   = $lnk;
           $ALPA{$x}{$alpa}{sync}   = $sync;
           $ALPA{$x}{$alpa}{signal} = $signal;
           $ALPA{$x}{$alpa}{seq}    = $seq;
           $ALPA{$x}{$alpa}{word}   = $word;
           $ALPA{$x}{$alpa}{crc}    = $crc;
        }
     }
     
  }
  my($out, @X, %SUMM, $hba , $form0, $form, $form2, $al, $hba0);
  my(%X, %H1);
  my(%ses_cnt) = 0;
  if (!$report) {
    for ($x=0; $x <= $#HBA; $x++ ) {
       my($hb) = $HBA[$x];
       $hb = substr($hb, 0, -7) if (substr($hb, -7) eq ":devctl");
       $H1{$x} = $hb;
    }
  } 
  for ($x=0; $x <= $#HBA; $x++ ) {
     $hba = $hba0 = $HBA[$x];
     $hba =~ s/\/devices\///;
     if ($report) {
       @X = ();
       $out .= "\nHBA ($x): $hba \n";
       $form0 = "%-5.5s %-10.10s %-10.10s %-4.4s %-16.16s %-12.12s %-6.6s";
       $form  = "$form0 %6s %6s %6s %6s %6s %6s\n";
       $form2 = "$form0 %6d %6d %6d %6d %6d %6d\n";
       $out .= sprintf($form, "Path",
              "Enclosure","Disk","Alpa","Wwn","Logical","Desc","Link","Signal","Seq","Crc",
              "Sync","Word");
       $out .= "-" x 111 . "\n";
     }
     my($y) = $ALPA{$x};

     my($al2, $wwn, $enc, $disk, $device);
     my($hbadone)= 0;
     foreach $al (reverse sort keys %$y) {  # read each element on this hba(ctrl/hba)
         $al2 = $y->{$al};
         $wwn  = $al2->{node};
         if ($WWN{$wwn}{enc}) {
             $disk = "port.0";
             $enc = $wwn;
         } elsif ($al2->{desc} =~ /Host Bus/)  {
             # $hbadone = 1;
             # $enc = "hba";
             $disk = "hba";
         } else {
           next;
         }
         my($path) = $x;
         if ($report) {
           my($enc0) = $enc;
           $enc0 = substr($enc0,-10) if (length($enc0) > 10) ;
           push(@X, sprintf($form2, $path, $enc0, $disk, $al, $wwn, $device, $al2->{desc},
                        $al2->{link}, $al2->{signal}, $al2->{seq}, $al2->{crc},$al2->{sync},$al2->{word}));
         } else {
           my($key) = "$path|$enc|$disk|t3";
           $X{$key} = "$al2->{link}\t$al2->{signal}\t$al2->{seq}\t$al2->{crc}\t$al2->{sync}\t$al2->{word}";
         }
     }
     if ($report) {
       foreach $l (sort @X) {
         $out .= $l;
       }
     }
  }
  my($err);

  if (!$report) {
    my($out) = { data => \%X,  hba => \%H1 };
    return ($out);
  } else {
    return $out;
  }
}

#  1         2         3       4         5       6
# LINKFAIL LOSSSYNC LOSSSIG PROTOERR INVTXWORD INVCRC
# link, sig, seq, crc, sync, word
#  1     3    4    6    2     5
#  ($sys_err, $hash, .command_error) = T3Agent->FC...

sub FCCountersOOB {
  my($class, $dev) = @_;
  my(@L, @l, %X, %W);
  my($err, $key);
  my($unit, $path, $l, $disk, $enc, $sim);

  if (!$dev->{telnet}) {
    return ("No telnet password",  {});
  } elsif ($dev->{type} ne "t3") {
    return ("$dev->{name} not a T3", {});
  }
  
  my($ip) = $dev->{ip};
  my($pass) = $dev->{telnet};

  my($t) = new Net::Telnet (
             errmode => "return",
             Timeout   => 30,
             Prompt    => '/<\d+\>/',
             );
  if (!defined($t->open($ip))) {
      return ("Cannot open $ip", {});
  }
  $t->login("root", $pass);
  if ($t->errmsg()) {
     return ($t->errmsg(), {});
  }

 if (0) {
  @l = $t->cmd("port list");
  foreach my $port (@l) {
     my(@b) = split(/\s+/, $port);
     $W{substr($b[0],0,2)} = $b[5];
  }
 }
  foreach $path ('0','1') {
    foreach $unit ('1','2') {
       @l = $t->cmd(".disk linkstat u${unit}d1-9 path $path");
       foreach $l (@l) {
         if ($l =~ /Failed/) {
            $err .= $l;
         } elsif ($l =~ /^u\dd\d/) {
           my(@a) = split(/\s+/, $l);
           $disk = $a[0];
           $key = "$path|$dev->{ipno}|$disk|t3";
           $X{$key} =  "$a[1]\t$a[3]\t$a[4]\t$a[6]\t$a[2]\t$a[5]";
         }
       }
    }
  }

  foreach $sim ('0','1', '2') {
      @l = $t->cmd(".sim -f num $sim linkstat");
      foreach $l (@l) {
        if ($l =~ /Error/) {
            $err .= "sim$sim: $l";
        } elsif ($l =~ /^\d+\s+\d+/) {
          my(@a) = split(/\s+/, $l);
          $key = "|$dev->{ip}|sim$sim|t3";
          $X{$key} =  "$a[0]\t$a[2]\t$a[3]\t$a[5]\t$a[1]\t$a[4]";
        }
     }
  }

  $t->close;
  return (undef, \%X, $err);
}


sub inbandProbe {
  my($class) = @_;

  my($lux) = "/usr/sbin/luxadm";

  my($err,$probe) = Util->run_command("$lux probe -p", "luxprobe.txt", 40);
  my($in, $l, @list, $disks);
  my($hbalist, $t3, $ix);
  for ($ix=0; $ix <= $#$probe; $ix++) {
      $l = $probe->[$ix];
      if ($l =~ /Found/) {
        $in = 1;
      } elsif ($in) {
        if ($l =~ / Node WWN:(.+)\s+Device Type:Disk/) { # t3 or internal..
          my($wwn) = Util->trim($1);
          ($err, $disks) = Util->run_command("$lux display $wwn","lux",40);
          my($rep) = "@$disks";
          if ($rep =~ /T300/) {
             push(@list, $wwn);
          }
        }
      }
  }
  return @list;
}

#
# T3 InBand
# flag == 1 : only look for mpxio path 
# like  Logical Path:/dev/rdsk/c14t60020F2000003EE53AAF7A09000DA257d0s2

sub readT3IB {
  my($agent, $cache1, $ENC, $HBA, $MPX, $flag) = @_;
  my($err, $probe, $disks, $mpx_wwn, $mpx_class);

  my($lux) = "/usr/sbin/luxadm";
  ($err,$probe) = FC->run_command("$lux probe","","-p", $cache1);

  return $err if ($err);
  my($in, $l);
  my($hbalist, $t3, $ix, $h2, $h3);
  for ($ix=0; $ix <= $#$probe; $ix++) {
      $l = $probe->[$ix];
      if ($l =~ /Found/) {
        $in = 1;
      } elsif ($in) {
        if ($l =~ / Node WWN:(.+)\s+Device Type:Disk/) { # t3 or internal..
          if ($flag) {
             next if ($probe->[$ix+3] !~ /scsi_v/);
          }
          my($wwn) = Util->trim($1); 
          ($err, $disks) = FC->run_command("$lux display $wwn","","", $cache1);
          my($rep) = "@$disks";
          if ($rep =~ /T300/) {
             $rep =~ /Serial Num:\s+(\d+)/;
             my($name) = $1;
             $ENC->{$wwn} = $name;

             while ($probe->[$ix+1] =~ /Logical Path:(.*)/) {
               my($hba) = Util->trim($probe->[$ix+3]);
               if ($hba !~ /devices\/scsi_/) {
                    my($ii) = rindex($hba, "/");
                    $hba = substr($hba, 0 , $ii);
                    if (index($hbalist, "$hba,") < 0) {
                      push(@$HBA, $hba);
                      $hbalist .= "$hba,";
                    }
               } else {
                   $h3 = $h2 = $mpx_wwn = $mpx_class = "";
                   foreach my $ll (@$disks) {
                      if ($ll =~ /^\s+Controller\s+(.*)/) {
                         $h3 = $1;
                         $h2 = $1 . ":devctl";
                         if (index($hbalist, "$h2,") < 0) {
                            push(@$HBA, $h2);
                            $hbalist .= "$h2,";
                         }
                      } elsif ($ll =~ /Device Address\s+([0-9a-f]+)/) {
                         $mpx_wwn = $1;
                      } elsif ($ll =~ /Class\s+(.*)/) {
                         $mpx_class = $1;
                      } elsif ($ll =~ /State\s+(.*)/) {
                         $MPX->{$mpx_wwn} = "$1\t$h3" if ($mpx_class eq "primary");
                      }
                   }
           
               }
               $ix += 3;
             }
          }
        }
     }
  }
  return ;
}


1;
