package CIM::Property;
#  $Name:  $ 
#  $Id: Property.pm,v 1.2 2000/11/12 20:03:54 ccadieux Exp $

use Data::Dumper;
use strict;
use base 'CIM::Base';

#
#  property = [name, value, type, keyflag, quals ].
#  qualifiers = [name, value,type, name2, value2, type ...]
#

sub new {
  my($class, $arg) = @_;

  my($self) = $arg;

  bless($self, "CIM::Property");
  return $self;
}

sub addQualifier {
   my($p, $q) = @_;

   if ($p->[4]) {
     push(@{$p->[4]}, @$q);
   } else {
     $p->[4] = $q;
   }
}

sub isReference {
  my($p) = @_;
  return ($p->type eq "reference");
}

sub name {
  my($key) = @_;
  return $key->[0];
}

sub value {
  my($key) = @_;
  return $key->[1];
}

sub type {
  my($key) = @_;
  return $key->[2];
}

sub qualifiers {
  my($key) = @_;
  return $key->[4];
}

sub isKey {
  my($key) = @_;
  return $key->[3];
}



sub toSerial {
  my($key) = @_;

  $Data::Dumper::Indent = 0;
  my($o) = Dumper($key);
  return substr($o, 7);
}

sub fromSerial {
  my($class, $s) = @_;
  my($o) = '';
  $s = '$o=' . $s;
  eval $s;
  return undef if ($@);
  return $o;
}

sub toXML {
  my($p) = @_;
  my($n , $v, $x, $tag, $o, $val);

  if (ref($p->[1]) eq "ARRAY" ) {
     $tag = "PROPERTY.ARRAY";
     $o .= "<$tag NAME=\"$p->[0]\" TYPE=\"$p->[2]\">";
  } elsif (ref($p->[1])) {
     $tag = "PROPERTY.REFERENCE";
     $o .= "<$tag NAME=\"$p->[0]\">";
  } else {
     $tag = "PROPERTY";
     $o .= "<$tag NAME=\"$p->[0]\" TYPE=\"$p->[2]\">";
  }

  if ($p->[3]) { # key
     $o .= "<QUALIFIER NAME=\"KEY\" TYPE=\"boolean\"><VALUE>TRUE</VALUE></QUALIFIER>";
  }
  my($q) = $p->[4];
  if ($#$q >= 0) { # one extra quals
     my($quals) = $p->[4];
     $o .= "<QUALIFIER NAME=\"$quals->[0]\" TYPE=\"$quals->[2]\"><VALUE>$quals->[1]</VALUE></QUALIFIER>";
  }
  if (ref($p->[1]) eq "ARRAY" ) { # array
     $o .= "<VALUE.ARRAY>";
     foreach $val (@{$p->[1]}) {
       $o .= "<VALUE>$val</VALUE>";
     }
     $o .= "</VALUE.ARRAY>";
  } elsif (ref($p->[1])) {
     $o .= "<VALUE.REFERENCE>" . $p->[1]->toXML . "</VALUE.REFERENCE>";
  } else {
     $o .= "<VALUE>$p->[1]</VALUE>";
  }
  $o .= "</$tag>";
  return $o;
}



sub toString {
  my($p) = @_;
  my($v, $k, $x);

  if (ref($p->[1]) eq "ARRAY") {
    $v = "['". join("','", @{$p->[1]}) . "']";
  } elsif (ref($p->[1])) {
    $v = $p->[1]->toString;
  } else {
    $v = $p->[1];
    $v =~ s/'/\\'/g;
    $v = "'$v'";
  }
  my($qual);
  my($q) = $p->[4];
  if ($#$q >= 0) {
    $qual = "Q[";
    $qual .= join(",", @{$p->[4]});
    $qual .= "]";
  } 
  $k = ($p->[3])? "K":"0";
  return "[$p->[0], $v, $p->[2], $k $qual]";
}

1;


# 1;
#


__END__

=head1 NAME

property.pm - Class that contains one property of an instance.


=head3 SYNOPSIS

 use Property;

 $prop = Property->new({ ... });



=head3 DESCRIPTION

This module creates Properties for usage in Instance.



=head3 CONSTRUCTOR

=over 4

=item new( [<name>, <value>, <type>, <keyflag>, <quals> ]);

 Example:
   $pro = Property->new([ serialno => "12312", "integer", 1 ]);

=back 4


=head3 METHODS

=over 4

=item addQualifier

Adds a new qualifier to an existing property.

 $pro->addQualifier([  .. ]);


=item isReference();

is this property a reference.

=item name();

returns the name of the property.

=item value();

returns the value of the property.

=item type();

returns the type of the property.

=item qualifiers();

returns an ARRAYREF to the qualifiers.

=item iKey();

true if the property is a key.

=item toSerial();

Serialize.

=item fromSerial();

Unserialize

=item toXML();

XML

=item toString();

String for debugging.



=back 4

=head3 SEE ALSO




=head3 AUTHOR

 Christian Cadieux (ccadieux@central.sun)



=head3 COPYRIGHT

Copyright (c) 2000 Sun Microsystems

=cut
