package CIM::Tree;
#  $Name:  $ 
#  $Id: Tree.pm,v 1.2 2000/11/12 20:03:54 ccadieux Exp $

use strict;
use CIM::Instance;
use Carp;
#
# $tree = { $inst1_key => [$inst1, [$to_inst2, $using_ass2], [$to_inst3, $using_ass3]...],
#           $inst2_key => [$inst2, [$to_inst5, $using_ass5], [$to_inst6, $using_ass6]...],
#         };
use vars qw(%ITYPE);


sub new {
  my($class, $tree) = @_;
  
  my($self) = $tree;  
  bless($self, "CIM::Tree");
  return $self;
}


#  $subtree = $tree->next($ins1);
#
sub next{
  my($tree, $node) = @_;

  my($v) = (ref($node)) ? $node->key->shortString: $node;
  return $tree->{$v};
}

sub toString {
  my($tree) = @_;
  my($x);

  foreach my $key (keys %$tree) {
      my($t) = $tree->{$key};
      print "==> "  . $t->[0]->className() . "\n";
      for ($x=1; $x <= $#$t; $x++) {
          my($node) = $t->[$x];
          
          print "  goes to " . $node->[0]->name() . "(".
                 $node->[0]->type() . ") using " . $node->[1]->name() . "\n";         
      }
  }

}

#  print $tree->toString($ev, {properties => 1, html => 1, assoc => 0, skip => "NWS_StorageVolume"});

sub visit {
  my($tree, $node, $arg) = @_;
  my(%done);
  $arg->{assoc} = 1 if (!exists($arg->{assoc}));

  croak("first argument must be an 'Instance'") if (ref($node) ne "CIM::Instance");
  return &_visit($tree, $node, 0, \%done , $arg, "*");
}
  
%ITYPE = (A => 'Assoc', D => 'Storage', H => 'Host', C => 'Component',
          E => 'Event', S => 'Software', O => 'Other');
#
sub _visit {
  my($tree, $node, $level, $done, $arg, $using) = @_;
  my($out, $x, $els, $nodeKey) ;
  croak("CIMTree::visit need a starting node!") if (!$node);
  $els = $tree->next($node); 
  my($itype) = $node->type();
  my($k) = $node->key->values;

  if ($arg->{html}) {
    $out .= "    " x $level  . "<b>" . $node->className . "</b> ($k)\n";
  } else {
    $out .= "    " x $level  . $node->className . " (key=$k)\n";
  }

  $nodeKey = $node->key->shortString;
  $done->{$using . $nodeKey} = 1;
  if ($arg->{properties}) {
    $out .= $node->toString($level) . "\n";
  }
  my($cnt) = 0;
  for ($x=1; $x <= $#$els; $x++) {
    my($k2, $k3);
    if (ref($els->[$x][0]) eq "CIM::Instance") {
      next if ($arg->{skip} && index($arg->{skip}, $els->[$x][0]->className) >= 0);
      $k2 = $els->[$x][0]->key->shortString;
      my($cn) = $els->[$x][1]->className;
      if (! ($done->{$cn.$k2} || $done->{"*$k2"}) ) {
        if ($node->type ne "A") {
           $out .= "\n" if ($cnt);
           $cnt++;
           $out .= "    " x ($level+1) . "$cnt-using $cn to: \n" if ($arg->{assoc});
        }
        $done->{$cn.$k2} = 1;
        $done->{$cn.$nodeKey} = 1;
        $out .= &_visit($tree, $els->[$x][0], $level+1, $done, $arg, $cn);
      }
    } else { # property
      $k2 = $els->[$x][0]->name;
      $k3 = $els->[$x][0]->value;
      $k3 = $k3->toString if (ref($k3));
      $out .= "\n" if ($cnt);
      $cnt++;
      $out .= "    " x ($level+1) . "$cnt-using " . $els->[$x][1]->className ." to: \n";
      $out .= "    " x ($level+1) . "$k2 $k3\n";
    }
  }
  return $out;
}
1;



__END__

=head1 NAME

CIM::Tree.pm - Class that contains a Data structure good for traversing a set of CIM instances.


=head3 SYNOPSIS

 use CIM::Tree;

 $tree = CIM::Tree->new({ ... });
 $tree->print();
 $tree->next($node);
 
  Example:
   $tree = { $inst1_key => [$inst1, [$to_inst2, $using_ass2], [$to_inst3, $using_ass3]...],
             $inst2_key => [$inst2, [$to_inst5, $using_ass5], [$to_inst6, $using_ass6]...],
            };



=head3 DESCRIPTION

Data Structure for visiting CIM instance list.



=head3 CONSTRUCTOR

=over 4

=item new( $hashref );

=back 4


=head3 METHODS

=over 4

=item next($node);

return the next instances accessible from a node.

=item toString($arg);

Display the tree

   Example:
     print $tree->toString({html => 1, properties => 1});

=item print();

Print the nodes.


=back 4

=head3 SEE ALSO




=head3 AUTHOR

 Christian Cadieux (ccadieux@central.sun)



=head3 COPYRIGHT

Copyright (c) 2000 Sun Microsystems

=cut



1;
  
