package Catalog;

use Util;
use strict;
use System;
use Data::Dumper;

use vars qw($CAT $PULLED_MASTER $NEWCATALOG);

sub push {
  my($class, $master) = @_;

  return if (!$master);
#  return if (!$NEWCATALOG);

  my($renv) = System->get_renv();
  my($F) = System->get_home() . "/DATA/catalog/" . $renv->{hostname};

  my($url) = Util->makeUrl($master, "?PUT=saveF&file=catalog/$renv->{hostname}");

  open(O, $F);
  my(@a)  = <O>; close(O);
  return if ($#a < 0);
  Debug->print2("  Catalog : push catalog to master");
  my($err, $ans)  = Util::Http->post($url, join("", @a), 20);
  if ($ans !~ /OK/) {
     Debug->err(HTTP_ERROR => "Failed to push catalog info: $err");
  }

}

#  read remote catalog, store in /catalog/file , then re-read all local cat.

#  readRemote("nscc-test" , "nscc-test");  # read one nscc-test catalog
#  readRemote("nscc-test" );               # read all nscc-test catalogs

sub readRemote {
  my($class , $host, $catalog) = @_;

  my($renv) = System->get_renv();
  if ($host eq $renv->{hostname}) { # local
     return 1;
  }

  my $url =  Util->makeUrl($host, "?GET=FILER&file=catalog/$catalog");
  my $data = Util::Http->get($url, 20);
  if (!$data) {
      Debug->print2("Cannot read catalog from $host");
      return undef;
  }

  my($F) = System->get_home() . "/DATA/catalog/$host";
  open(O, ">$F");
  print O $data;
  close(O);
  $class->write();  # in case you are building $CAT already and don't want to loose new stuff.
  $class->read();

  return 1;
}

#
#  read all local catalogs
#
sub read {
  my($class) = @_;

  my($VAR1, @b);
  my($renv) = System->get_renv();
  my($F) = System->get_home() . "/DATA/catalog/";
  $CAT = {}; 
  my($local) = $renv->{hostname};
  opendir(O, $F);
  my @cat0 = readdir(O); closedir(O);
  my @cats;
  my $p1 = 0;    # put local catalog at end of list;
  foreach my $f (@cat0) {
    if ($f eq $local) {
       $p1 = 1;
    } else {
       push(@cats, $f) 
    }
  }
  push(@cats, $local) if ($p1);
  
  foreach my $f (@cats) {
     next if (substr($f,0,1) eq ".");
     open(O, "$F/$f");
     my(@a) = <O>; close(O);
     my($flag) = ($local eq $f) ? 1:0;
     foreach my $line (@a) {
        chop($line);
        @b = split(/\t/, $line);
        $CAT->{$b[0]} = [$b[1],$b[2],$b[3],$b[4],$b[5],$b[6],$b[7],$b[8],$b[9],$b[10],$flag];
     }
  }
}


# 
# find the age of the master catalog present on a slave
#
sub catalogAge {
  my($class, $host) = @_;  # host is the master

  my($renv) = System->get_renv();

  my $master_catalog = System->get_home() . "/DATA/catalog/$host";
  my $age = (stat($master_catalog))[9];

  return $age;
}

sub newCatalog {
  $NEWCATALOG = 1;
}

sub clearCatalog {
  my($class, $host) = @_;

  my($renv) = System->get_renv();
  $NEWCATALOG = 1;
  unlink System->get_home() . "/DATA/catalog/$host";
}
  


sub write {
  my($class) = @_;
  $DB::single = 1;
  return if (!$CAT);
  my($renv) = System->get_renv();
  my($F) = System->get_home() . "/DATA/catalog/" . $renv->{hostname};
  open(O, ">$F");

  foreach my $k (keys %$CAT) {
    my $v = $CAT->{$k};
    next if (!$v->[10]);   # only write local info in catalog
    my $o = "$k\t";
    $o .= join("\t", @$v) . "\n";
    print O $o;
  }
  close(O);
}


#  n2k  : name -> key (cim and instrumentation file name
#  n2w  : name -> wwn
#  n2i  : name -> ip
#  i2o  : IP_name -> IP_no
#

#          0      1     2     3     4        5      6          r7         8   9
# key => [ type, name, wwn1, wwn2, ip_name, ip_no, port1-key, port2-key, Key, Host, local-flag ]
#  wwn1 : a5k wwn or u1p1 of t3
#  wwn2 : u2p1 of t3

sub Type { 0 }
sub Name { 1 }
sub WWN1 { 2 }
sub WWN2 { 3 }
sub IP   { 4 }
sub IPno { 5 }
sub Port1{ 6 }     # t3
sub Port2{ 7 }     # t3
sub Key  { 8 }     
sub Host { 9 }     

sub WWN  {10 }


sub deviceWrite {
  my($class, $key, $values) = @_;

  if (!$CAT) {
     $class->read();
  }
  my($renv) = System->get_renv();
  $CAT->{$key} = $values;
  $CAT->{$key}[&Key]  = $key;
  $CAT->{$key}[&Host] = $renv->{hostname};
  $CAT->{$key}[10]    = 1;
  return $key;
}

sub deviceRead {
  my($class, $category, $type, $key, $return) = @_;
  if (!$CAT) {
     $class->read();
  }
  my $ret = $class->deviceRead_($category, $type, $key, $return);
  if (!defined($ret)) {
     my($master) = Util->findMaster;
     if ($PULLED_MASTER || !$master) {
#        Debug->print2("Catalog:deviceRead: cannot find $category:$key");
        return undef;
     }
     Debug->print2("Pulling master catalog");
     $class->readRemote($master);   # get a complete catalog.
     $PULLED_MASTER = 1;            # set once per rasagent run
     $ret = $class->deviceRead_($category, $type, $key, $return);
     if (!defined($ret)) {  
#        Debug->err(TEXT => "Catalog:deviceRead: cannot find $category:$key");
        return undef;
     }
  }

  if ($type eq &WWN) {
     return ($ret->[0], $ret->[1]);
  } 
  return $ret;
}



sub deviceRead_ {
  my($class, $category, $type, $key, $return) = @_;

  foreach my $k (keys %$CAT) {
     next if ($category && ($CAT->{$k}[0] ne $category));
     if ($type eq &WWN) {
        my($port) = 0;
        if (lc($CAT->{$k}[&WWN1]) eq lc($key)) {
           $port = 1;
        } elsif (lc($CAT->{$k}[&WWN2]) eq lc($key)) {
           $port = 2;
        }
        if ($port) {
           my($x) = $CAT->{$k};
           my(@xx);
           if ($return) {
             @xx = ($x->[$return], $port) ;
           } else {
             @xx = ( $x, $port);
           }
           return \@xx;
        }

     } elsif ($CAT->{$k}[$type] eq $key) {
        my($x) = $CAT->{$k};
        if ($return) {
           return $x->[$return];
        } else {
          return $x;
        }
     }
  }
  return undef;
}
  
  
  
sub deviceAccess0 {
  my($class, $type, $access, $cat, $key) = @_;
  
  if (!$CAT) {
     $class->read();
  }

  if ($key && $access) {
    my($type2) = substr($type, 2,1) . "2" . substr($type,0,1);

    $CAT->{"$type:$access"} = [$cat, $key];
    $CAT->{"$type2:$key"} = [$cat, $access];
    return $key;
  }
  if ($access) {  # give name or ip, get wwn
    my($p) =  $CAT->{"$type:$access"};

    if (wantarray) {
       return ($p->[0], $p->[1]);
    } else {
       return $p->[1];
    }
  }
  return undef;
}

1;




