package FCConfig::Python;
use TO;
use Data::Dumper;
use Agent::SWITCHAgent;

$SAMPLE = {
   'switch:100000c0dd008467' => {
                      'ip' => '172.20.67.165',
                 'primary' => '172.20.67.195',
       'zone.hd.0.members' => '1 2 3 4 5 6 7 8 '
   },
   'switch:100000c0dd008849' => {
                      'ip' => '172.20.67.195',
                 'cascade' => '172.20.67.165',
       'zone.sl.1.members' => '11 13 ',
       'zone.hd.0.members' => '1 2 3 4 5 6 7 8 9 10 ',
       'zone.hd.1.members' => '11 12 13 14 15 16 '
   }
};


@TEXT = ([],
 ['G-1', 'General: Must have at least one switch and one storage.'],
 ['Z-1', 'Zoning: Only 8 and 16 port switches are supported.' ],
 ['Z-2', 'Zoning: Non overlapping zones: a port can only be in one zone.'],
 ['Z-3', 'Zoning: A zone can have a minimum of one port.'],
 ['Z-4', 'Zoning: Ports from SL or NS zones must also be in separate Hard zones.'],

 ['C-1', 'Cascading: Hub to Switch Connectivity is not supported.'],
 ['C-2', 'Cascading: Switch-to-switch is supported for distance only, max distance is 10kM.', 'Manual'],
 ['C-3', 'Cascading: Only T3 can be connected to a cascaded switch.'],
 ['C-4', 'Cascading: Only 2 switches can be cascaded together.'],
 ['C-6', 'Cascading: A zone with remote storage cannot have storage connected to the local switch.'],
 ['C-7', 'Cascading: A zone with remote storage cannot have remote Host.'],
 ['S-1', 'Switch: Minimum switch revision is 461.'],

 ['H-1', 'Host: All hosts in a zone must be running at least Solaris 8.'],
 ['H-3', 'Host: Both hosts and a switch zone can have Crystal+ and Amber cards as initiators.','Manual'],
 ['H-4', 'Host: Platform supported are E3x00-E6x00, E10k, E250, E220, E450, E420.','Manual'],

 ['D-1', 'Device: Device supported are 3500FC, A5200, T3(Purple-1) and FC-Tapes.','Manual'],
 ['D-2', 'Device: Only one type of device per zone.'],
 ['D-3', 'Device: A3500fc and A5200 must be in SL-zone.'],

 ['T3-1', 'T3: Device in Name Server Zone, Host on F-Port, T3 on TL-Port.'],
 ['T3-2', 'T3: Max Storage per zone: 8.'],
 ['T3-3', 'T3: Max initiators per zone: 2.'],
 ['T3-4', 'T3: Max initiator per LUN: 2.', 'Manual'],
 ['T3-5', 'T3: T300 Release 1.16 or greater'],

 ['A3-2', 'A3500FC: Max Storage per zone: 4.'],
 ['A3-3', 'A3500FC: Max initiator per SL-zone: 2.'],

 ['A5-1', 'A5200: Split-loop is not supported.'],
 ['A5-2', 'A5200: Max Storage per zone: 3.'],
 ['A5-3', 'A5200: Max initiator per SL-zone: 2.'],
 ['A5-4', 'A5200: Max initiators per Storage Array: 4.', 'Manual'],
 
);

sub compare {
  my($s1, $s2) = @_;
  my(%P);
  foreach my $k (keys %$s1) {
    if (substr($k, 0, 5) eq "zone.") {
        my($type, $zone_no) = split(/\./, substr($k, 5, -8));
        my $ps = $s1->{$k};
        my @pss = split(/ /, substr($ps,0,-1));
        foreach $port (@pss) {
            if ($P{$port}{$type} && $P{$port}{$type} ne $zone_no) {
                return "switch $s1->{ip}: port $port is in zone ".
                       "$type ($$P{$port}{$type} and $zone_no)";
            }
            $P{$port}{$type} = $zone_no;
        }
     }
  }
  %P = ();
  foreach my $k (keys %$s2) {
    if (substr($k, 0, 5) eq "zone.") {
        my $ps = $s2->{$k};
        my @pss = split(/ /, substr($ps,0,-1));
        foreach $port (@pss) {
            if ($P{$port}{$type} && $P{$port}{$type} ne $no) {
                return "switch $s1->{ip}: port $port is in zone ".
                       "$type ($$P{$port}{$type} and $no)";
            }
            $P{$port}{$type} = $no;
        }
     }
  }
  return undef;
}



sub get_zones {
  my (%ZONES);
#  return $SAMPLE;
  my($renv, $devs, $hosts, $notifs) = PDM::ConfigFile->read;
  my %S;
  foreach my $d (@$devs) {
     if ($d->{type} eq "switch") {
        my $s_ip = $d->{ip};
        if ($d->{primary}) {
           $s_ip = "-f $d->{fc} $d->{primary}";
        }
        my %V;
        my $wwn = SWITCHAgent->getWWN(System->get_home() . "/bin", $s_ip, 30);
        SWITCHAgent->getZones(System->get_home() . "/bin", $s_ip, 30, \%V);
        my $ss = {};
       SWITCHAgent->getStatus(System->get_home() . "/bin", $s_ip, 30, $ss);
        $S{$s_ip} = $ss;
        if ($wwn)  {
           $V{ip} = $d->{ipno};
           if ($d->{primary}) {
              $V{primary} = $d->{primary};
              foreach $z0 (keys %ZONES) {
                 $z = $ZONES{$z0};
                 if ($z->{ip} eq $d->{primary}) {
                      $ZONES{$z0}{cascade} = $d->{ipno};
                 }
              }
           }
           $ZONES{"switch:$wwn"} = \%V;
        } else {
           print "<font color=red><b>Cannot ping $s_ip</font><br>\n";
        }
     }
  }
  return (\%ZONES, \%S);
}

sub zones_details {
  my($q) = @_;

  print "<body bgcolor=white>
   <table border=0 cellpadding=2 width=100% bgcolor=#666699>
    <tr><td><b><font color=white>Extracting Zone information</table>\n";
  ($ZONES, $PORTS) = &get_zones();
  $Data::Dumper::Indent = 1;
  print "<pre>" . substr(Dumper($ZONES), 7,-3) . "\n";
  print substr(Dumper($PORTS),9 -3);
  print "<center><b><form><input type=submit value=Close onclick=window.close()></form>";

}

#
# S : Storage
# H : Hosts
# 
sub found_storage {
  my($to, $type, $key, $ports) = @_;
  
  my $st = $to->nodeByName($key);
  return undef if (!$st);
  my $st_ports = $st->{port};

  my @ports = split(/ /, substr($ports,0,-1));
  
  foreach my $p (@ports) {
     if ($type eq "S") {
       if ($st_ports->[$p] && substr($st_ports->[$p],0,4) ne "host" &&
           substr($st_ports->[$p],0,6) ne "switch") {
          return 1;
       }
     }
     if ($type eq "H" && $st_ports->[$p] =~ /host:/) {
         return 1;
     }
  }
  return undef;
}

sub getDevice {
  my($zone, $to, $key, $port) = @_;

  my $st = $to->nodeByName($key);
  return undef if (!$st);
  my $st_ports = $st->{port};
  my $dev = $st_ports->[$port];
  if ($dev) {
#   print "$zone:$key:$port dev=$dev<br>";
    my($type, $k, $p) = split(/:/, $dev);
    return ($type,$k, $p);
  } else {
    return undef;
  }
}
 
sub findIPort {
  my($sw, $port) = @_;

  my $dp = $sw->{dport}; # internal links
  for ($x=1; $x <= $#$dp; $x++) {
     $d = $dp->[$x];
     if (index("-$d", "-$port-") >= 0) {
         return $x;
     }
  }
  return undef;
}

sub run {
  my($host, $extra) = @_;
  my(%RULES, %DETAILS);
  my($sw, $st, %ZONES);
  my $V = {};
  ($ZONES, $PORTS) = &get_zones();

  $to = TO->readExistingTopo($host);
  if (!$to) {
     return "Cannot read Topology $host";
  }
  my $to_info = $to->[0]{"host:$host"}{info};

  my $hosts = $to->hostList();
  my $host1 = $hosts->[0];
  my $h_hba = $host1->port();

  if ($h_hba && $#$h_hba == 0) {
     return "<b><font color=red>There is not hba attached to this host, cannot verify Python configuration</font></b>";
  }
     
  my $sws = $to->switchList();
  my $sts = $to->storageList();

  $RULE{'G-1'} = ["PASS"];
  my $g1;
  if ($#$sws < 0) {
      $g1 = "No switches found in the topology";
  }
  if ($#$sts < 0) {
      $g1 = ",No storage found in the topology";
  }
  if (!-f "/etc/fcswitch.conf") {
      $g1 = ",File /etc/fcswitch.conf not found!";
  }
  if ($g1) {
     $RULE{'G-1'} = ["FAIL", $g1];
  }

  my $sw_list;
  my %V;
  foreach $sw (@$sws) {
    $sw_list .= $sw->{info}{sw_ipAddr} . ",";
    # get the latest port_type directly from the switch
    %V = ();
    SWITCHAgent->getStatus(System->get_home() . "/bin", $sw->ip(), 30, \%V);        
    my $pi = $sw->{port};
    my $pinfo = $sw->{portInfo};
    for ($x=1; $x <= $#$pi; $x++) {
        my $pt =  $V{"port.$x.type"};
        if ($pt) {
           $pinfo->[$x]{sw_PortType} = $pt;
        }
    }
  }

  my $debug .= "[Z-2]\n";
  $DETAILS{'Z-2'} = "FCConfig::Python::zones_details";
  $RULE{'Z-2'}    = ["PASS"];
  $done = 0;
  $primary = 0;
  foreach $sw0 (keys %$ZONES) {
     my $sw = $ZONES->{$sw0};
     next if (index($sw_list, $sw->{ip}) < 0);
     if ($sw->{primary}) {
         $primary = 1;
         foreach my $sw20 (keys %$ZONES) {
             my $sw2 = $ZONES->{$sw20};
             if ($sw2->{ip} eq $sw->{primary}) {
                $debug .= "  Review $sw->{ip} and $sw2->{ip}\n";
                my $v = &compare($sw, $sw2);
                if ($v) {
                  $RULE{'Z-2'} = ["FAIL", $v];
                  $done = 1; last;
                }
             }
         }
         last if ($done);
     }
  }

  $RULE{'Z-3'} = ["PASS"];
  $debug .= "[Z-3]\n";
  foreach $sw0 (keys %$ZONES) {
     my $sw = $ZONES->{$sw0};
     $debug .= "  Review switch $sw->{ip}\n";
     foreach my $k (keys %$sw) {
       if (substr($k, 0, 5) eq "zone.") {
          my $ps = $sw->{$k};
          my @pss = split(/ /, substr($ps,0,-1));
          if ($#pss == -1) {
             $RULE{'Z-3'} = ["FAIL", "$k in switch $sw->{ip} has no port"];
          } elsif ($#pss == 0) {
             $RULE{'Z-3'} = ["PASS", "WARNING: $k in switch $sw->{ip} has only one port"];
          }
       }
     }
  }
    

  $RULE{'Z-1'} = ["PASS"];
  $debug .= "[Z-1]\n";
  foreach $sw (@$sws) {
     $debug .= "  Review switch $sw->{info}{sw_ipAddr}\n";
     if ($sw->{info}{ProductID} ne "Switch-8" && $sw->{info}{ProductID} ne "Switch-16") {
        $RULE{'Z-1'} = ["FAIL", $sw->{info}{sw_ipAddr} . " productID is invalid:" . 
                        $sw->{info}{ProductID}];
        last;
     }
  }
  if (!$primary) {
        $RULE{'Z-1'}[2] .= "No cascaded switches were found in $host topology.<br>";
  }

  $RULE{'Z-4'} = ["PASS"];
  $debug .= "[Z-4]\n";
  foreach $sw0 (keys %$ZONES) {
     my $sw = $ZONES->{$sw0};
     my %hard = ();
     $debug .= "  Review switch $sw->{ip}\n";
     foreach my $k (keys %$sw) {
       if (substr($k, 0, 7) eq "zone.hd") {
          my($t1, $n1, $rest) = split(/\./, substr($k,5));
          my @pss = split(/ /, substr($sw->{$k},0,-1));
          foreach $ps0 (@pss) {
            $hard{$ps0} = $n1 + 1;
          }
       }
     }
     my %hard2 = ();
     foreach my $k (keys %$sw) {
       if (substr($k, 0, 7) eq "zone.sl" || 
           substr($k, 0, 7) eq "zone.ns") {
          my $t1  = (substr($k, 0, 7) eq "zone.sl")? "sl":"ns";
          my $ps  = $sw->{$k};
          my @pss = split(/ /, substr($ps,0,-1));
          foreach $pp (@pss) {
             if (!$hard{$pp}) {
                $RULE{'Z-4'} = ["FAIL", 
                "Port $pp of $sw->{ip} is in a " . 
                 substr($k,5,2) . "_zone but not in a hard_zone"];
                $done = 1;last;
             } else {
                if ( $hard2{$hard{$pp}} &&  
                      $hard2{$hard{$pp}} ne $t1) {
                   $RULE{'Z-4'} = ["FAIL", 
                    "hard zone " . ($hard{$pp}-1) ." of $sw->{ip} is mixed"];
                   $done = 1;last;
                }
                $hard2{$hard{$pp}} = $t1;
             }
          }
          last if ($done);
       }
     }
  }
                 

  $RULE{'S-1'} = ["PASS"];
  $debug .= "[S-1]\n";
  foreach $sw (@$sws) {
    $debug .= "  Review switch $sw->{info}{sw_ipAddr}\n";
    if ($sw->{info}{sw_Flash} lt "30461") {
        $RULE{'S-1'} = ["FAIL", "switch $sw->{info}{sw_ipAddr} flash is old: " . 
               $sw->{info}{sw_Flash} ];
        last;
    }
  }

  $RULE{'C-4'} = ["PASS"];
  my %S;
  $debug .= "[C-4]\n";
  foreach $sw (@$sws) {
    if ($sw->{info}{sw_ipName} =~ /cascaded/) {
       $S{$sw->{info}{sw_ipAddr}} = $sw->{info}{sw_ipAddr_remote};
    }
  }
  foreach $ip (keys %S) {
      my $v = $S{$ip} ; 
      $debug .= "  Review switch $ip\n";
      if ($S{$v}) {
         $RULE{'C-4'} = ["FAIL", "Too many cascaded switches: $S{$v} -> $v -> $ip" ];
      }
  }

  $RULE{'C-1'} = ["PASS"];
  $debug .= "[C-1]\n";
  %MAP = ();
  $done = 0;
  foreach $st (@$sts) {
      my $ports = $st->{port};
      $debug .= "  Review storage ". $st->id() . "\n";
      my $p_count = 0;
      foreach my $p (@$ports) {
          my ($type, $key, $port) = split(/\:/, $p);
          if ($type eq "switch") {
            if ($MAP{$p}) {
                $RULE{'C-1'} = ["FAIL", $st->id(). " port $p_count [0|1] is attached to a Hub."];
                $done = 1; last;
            }
            $MAP{$p} = 1;
          }
          $p_count++;
      }
      last if ($done);
  }


  $RULE{'C-3'} = ["PASS"];
  $debug .= "[C-3]\n";
  $done = 0; $err = "";
  foreach $st (@$sts) {
      my $ports = $st->{port};
      $debug .= "  Review storage ". $st->id() . "\n";
      foreach my $p (@$ports) {
          next if (!$p);
          my ($type, $key, $port) = split(/\:/, $p);
          my $n1 = $to->nodeByName("$type:$key");
          if (!$n1) {
                $err .= "Cannot identify $p<br>";
          }
          if ($n1->{info}{sw_ipName} =~ /cascaded/) {
             if ($st->{info}{type} ne "t3") {
                $RULE{'C-3'} = ["FAIL", $st->id(). " attached to a cascaded switch"];
                $done = 1; last;
             }
          }
      }
      last if ($done);
  }

  $RULE{'C-3'}[2] = $err;
  $debug .= $err;
  $RULE{'C-6'} = ["PASS"];
  $RULE{'C-7'} = ["PASS"];
  $debug .= "[C-6, C-7]\n";

  $done = 0;
  foreach $sw0 (keys %$ZONES) {
     my $sw = $ZONES->{$sw0};
     next if (index($sw_list, $sw->{ip}) < 0);
     if ($sw->{primary}) {
         foreach my $sw20 (keys %$ZONES) {
             my $sw2 = $ZONES->{$sw20};
             if ($sw2->{ip} eq $sw->{primary}) {  # $sw2 -> $sw  cascade
                $debug .= "  Review switches: $sw2->{ip}, $sw->{ip}\n";
                foreach my $k (keys %$sw2) {
                  if (substr($k, 0, 5) eq "zone.") {
                     my $ps2 = $sw2->{$k};
                     my $ps0 = $sw->{$k};
                     if ($ps0) {  # found ports in both sw. for zone $k
                        if (&found_storage($to, "S", $sw0, $ps0) && 
                            &found_storage($to, "S", $sw20, $ps2)) {
                            $RULE{'C-6'} = ["FAIL", "found storage on $sw2->{ip} $k"];
                            $done =1; last;
                        }
                        if (&found_storage($to, "S", $sw0, $ps0) && 
                            &found_storage($to, "H", $sw0, $ps0)) {
                            $RULE{'C-7'} = ["FAIL", "found host on $sw->{ip} $k"];
                            $done =1; last;
                        }
                     }
                  }
                }
                last if ($done);
             }
         }
         last if ($done);
     }
  }

  $RULE{'H-1'} = ["PASS"];
  $debug .= "[H-1]\n";
  my $hos = $to->hostList();
  foreach my $host (@$hos) {
     $debug .= "  Review host $host->{info}{name} \n";
     if ($host->{info}{release} lt "5.8") {
        $RULE{'H-1'} = ['FAIL',"$host->{info}{name} is release $host->{info}{release}"];
        last;
     }
  }

                    
  $RULE{'D-2'} = ["PASS"];
  $debug .= "[D-2]\n";

  $done = 0;
  foreach $sw0 (keys %$ZONES) {
     my $sw = $ZONES->{$sw0};
     next if (index($sw_list, $sw->{ip}) < 0);
     if ($sw->{primary}) {
         foreach my $sw20 (keys %$ZONES) {
             my $sw2 = $ZONES->{$sw20};
             if ($sw2->{ip} eq $sw->{primary}) {  # $sw2 -> $sw  cascade
                $debug .= "  Review device(s) attached to $sw2->{ip}\n";
                foreach my $k (keys %$sw2) {
                  if (substr($k, 0, 5) eq "zone.") {
                     my $ps2 = $sw2->{$k};
                     my $ps0 = $sw->{$k};
                     my $type1 = "";
                     foreach $p1 (split(/ /, $ps2)) {
                        my ($t1, $k1, $to_p) = &getDevice($k, $to, $sw20, $p1);
                        next if ($t1 eq "host" || $t1 eq "switch" || !$t1);
                        if ($type1 && $type1 ne $t1) {
                           $RULE{'D-2'} = ["FAIL", "Found $type1 and $t1 in ($sw2->{ip}, $sw->{ip}) ".
                              substr($k,0,-8)];
                           $done = 1; last;
                        }
                        $type1 = $t1;
                     }
                     last if ($done);
                     foreach $p1 (split(/ /, $ps0)) {
                        my ($t1, $k1, $to_p) = &getDevice($k, $to, $sw0, $p1);
                        next if ($t1 eq "host" || $t1 eq "switch" || !$t1);
                        if ($type1 && $type1 ne $t1) {
                           $RULE{'D-2'} = ["FAIL", "Found $type1 and $t1 in ($sw2->{ip}, $sw->{ip}) ".
                               substr($k,0,-8)];
                           $done = 1; last;
                        }
                        $type1 = $t1;
                     }
                     last if ($done);
                   }
                }
             }
          }
          last if ($done);
      }
   }

  $RULE{'T3-1'} = ["PASS"];
  $debug .= "[T3-1]\n";
  
  $done = 0;
  foreach $st (@$sts) {
      next if ($st->{info}{type} ne "t3");
      $debug .= "  Review " . $st->id() ."\n";
      my $ports = $st->{port};
      foreach my $p (@$ports) {
          my ($type, $key, $port) = split(/\:/, $p);
          next if (!$type);
          my $n1 = $to->nodeByName("$type:$key");
          next if (!$n1);
          if ($n1->type() eq "switch" && $n1->{info}{sw_ipName} =~ /cascaded/) {
            my $sw_pi = $n1->{portInfo}[$port];
            if ($sw_pi->{sw_PortType} ne "TL_Port") {
                $RULE{'T3-1'} = ["FAIL", "T3 attached to switch $n1->{info}{sw_ipAddr} port $port is a $sw_pi->{sw_PortType} (should be TL_Port)"];
                $done = 1;last;
            }
            my $port2 = &findIPort($n1, $port);
            ($type, $key, $port) = split(/\:/, $n1->{port}[$port2]);
            my $n2 = $to->nodeByName("$type:$key");
            my $port3 = &findIPort($n2, $port);
            my $sw2_pi = $n2->{portInfo}[$port3];
            if ($sw2_pi->{sw_PortType} ne "F_Port") {
                $RULE{'T3-1'} = ["FAIL", "T3 goes thru switch $n2->{info}{sw_ipAddr} port $port3 is $sw2_pi->{sw_PortType} (should be F_Port) "];
                $done = 1;last;
            }
            
            
          }
      }
      last if ($done);
  }

  $RULE{'T3-5'} = ["PASS"];
  $debug .= "[T3-5]\n";
  my $err2;
  $done = 0;
  foreach $st (@$sts) {
      next if ($st->{info}{type} ne "t3");
      my $ports = $st->{portInfo};
      my $v = $ports->[0]{Revision};
      if ($v && $v lt "0116") {
           $err2 .=  $st->id() . " has version $v, should be 116<br>";
      }
  }
  if ($err2) {
     $RULE{'T3-5'} = ["FAIL",$err2];
  }


  $RULE{'T3-2'} = ["PASS"];
  $RULE{'T3-3'} = ["PASS"];
  $RULE{'A5-2'} = ["PASS"];
  $RULE{'A5-3'} = ["PASS"];
  $RULE{'A3-2'} = ["PASS"];
  $RULE{'A3-3'} = ["PASS"];

  $debug .= "[T3-2, T3-3]\n";
  $done = 0;
  foreach $sw0 (keys %$ZONES) {
     my $sw = $ZONES->{$sw0};
     $debug .= "  Review switch $sw->{ip}\n";
     foreach my $k (keys %$sw) {
       if (substr($k, 0, 5) eq "zone.") {
          my $ps = $sw->{$k};
          my @pss = split(/ /, substr($ps,0,-1));
          my $count= 0; my $icount = 0;
          my $A5count= 0;
          my $A3count= 0;
          foreach $p1 (@pss) {
             my ($t1, $k1, $to_p) = &getDevice($k, $to, $sw0, $p1);
             $count++ if ($t1 eq "t3");
             $icount++ if ($t1 eq "host");
             $A5count++ if ($t1 eq "a5k");
             $A3count++ if ($t1 eq "a3500fc");
          }
          if ($A3count > 4) {
                $RULE{'A3-2'} = ["FAIL", "found $A3count A3500fc in ".
                      $sw->{ip} .  substr($k,0,-7) ];
                $done = 1; last;
          }
          if ($A5count > 3) {
                $RULE{'A5-2'} = ["FAIL", "found $A5count A5200 in ".
                      $sw->{ip} .  substr($k,0,-7) ];
                $done = 1; last;
          }
          if ($count > 8) {
                $RULE{'T3-2'} = ["FAIL", "found $count t3s in $sw->{ip}".
                               substr($k,0,-7) ];
                $done = 1; last;
          }
          if ($icount > 2) {
              my $trule;
              if ($count) {
                 $trule = "T3-2";
              } else {
                 $trule = ($A5count) ? "A5-3":"A3-3";
              }
              $RULE{$trule} = ["FAIL", "found $icount initiators in $sw->{ip}".
                               substr($k,0,-7) ];
              $done = 1; last;
          }
        }
      }
      last if ($done);
  }

  $RULE{'D-3'} = ["PASS"];
  $debug .= "[D-3]\n";
  $done = 0;
  foreach $st (@$sts) {
      next if ($st->{info}{type} ne "a5k" && $st->{info}{type} ne "a3500fc");
      my $ports = $st->{port};
      $debug .= "  Review storage ". $st->id() . "\n";
      foreach my $p (@$ports) {
          my ($type, $key, $port) = split(/\:/, $p);
          next if (!$type);
          my $n1 = $to->nodeByName("$type:$key");
          next if ($type ne "switch" || !$n1);

          if ($n1->{portInfo}[$port]{sw_PortType} ne "SL_Port") {
                $RULE{'D-3'} = ["FAIL", $st->id() . " is attached to a ".
                  " $n1->{portInfo}[$port]{sw_PortType} port, should be SL_Port"];
              $done =1; last;
          }
      }
      last if ($done);
  }

  
  $RULE{'A5-1'} = ["PASS"];

  $debug .= "[A5-1]\n";
  $err = "";
  foreach $sw0 (keys %$ZONES) {
     my $sw = $ZONES->{$sw0};
     next if (index($sw_list, $sw->{ip}) < 0);
     if ($sw->{cascade}) {  # primary switch from a cascaded
         $sw1 = $to->nodeByName($sw0);
         next if (!$sw1);
         $sw_port = $sw1->{port};
         foreach $p (@$sw_port) {
            my ($t1, $k1, $to_p) = split(/\:/, $p);
            next if ($t1 ne "a5k");
            $debug .= "  Checking if $p is in split-loop mode\n";
            my $rep;
            if ($host eq System->hostname() ) {
               $rep = Report->readReport("$t1:$k1");
            } else {
               ($err0, $rep) = Report->readNetReport({host => $host,
                                   file => "$t1:$k1"});
            }
            if (!$rep) {
               $err .= "No report for $t1:$k1, cannot verify Split-Loop.";
               
            } else {
              my $val = $rep->value();
              if ($val->{'loop.A.info'} =~ /split/ || 
                  $val->{'loop.B.info'} =~ /split/) {
                  $RULE{'A5-1'} = ["FAIL", "a5k $k1 on $host is in split-loop mode"];
              }  
            }
         }
     }
  }
  $debug .= $err;
  $RULE{'A5-1'}[2] = $err;

  open(O, ">" . System->get_home() . "/DATA/config_verify.PY");
  print O $debug;
  close(O);


###################################################################################
# DUMP
###################################################################################

  my $out = "
<script>
  function win1(a) {
    var b = '/?GO=' + a;
    var O = window.open(b,'Fav','menubar=no,resizable=yes,scrollbars=yes,width=350,height=400');
      O.focus();
    }
</script>

   <table border=0 cellpadding=2 bgcolor=#CCCCFF width=100%><tr>
              <td><b>&nbsp HOST: <big>$host</big> &nbsp; $to_info->{model} &nbsp;
     <tr><td bgcolor=white>$extra</td>
   </table>
   <center>
   <table border=1 cellspacing=0 cellpadding=1 width=95%>
   <tr bgcolor=#F0F0F0>
       <td><b><center>Rule</td>
       <td><b>Description</td>
       <td><center><b>Details</td>
       <td><b><center>Test</td>";

  my ($total) = "PASS";
  my $last;
  for ($x=1; $x <= $#TEXT; $x++) {
      my $result;
      my $tx = $TEXT[$x];
      my $rno = $tx->[0];
      my $default = $tx->[2];
      my $desc = "";
      if ($default) {
         $result = $default;
      } else {
         if ($RULE{$rno}) {
            if ($RULE{$rno}[0] eq "FAIL") {
              $result = "<font color=red><b>FAIL";
              $total  = "FAIL";
            } elsif ($RULE{$rno}[0] eq "PASS")  {
              $result = "<font color=green><b>PASS";
            } else {
              $result = "<font color=blue><b>$RULE{$rno}[0]";
            }
            if ($RULE{$rno}[1] || $RULE{$rno}[2]) {
               $desc = "<br>&nbsp;<font color=red>";
               $desc .= $RULE{$rno}[1] . $RULE{$rno}[2];
            }
         } else {
            $result = "<b>??";
         }
      }
      my $det = "&nbsp;";
      if ($DETAILS{$rno}) {
         $det = "[ <a href=javascript:win1('$DETAILS{$rno}')>Details</a> ]" ;
      }
      if (substr($rno,0,1) ne $last) {
          $out .= "<tr><td colspan=4>&nbsp;</td>" if ($last);
          $last = substr($rno,0,1);
      }
      $out .= "<tr><td><center><b>$rno</td><td>$tx->[1]$desc</td>
               <td><center>$det</td>
               <td><center>$result</td>";
  }
  $col = ($total =~ /FAIL/)? "red":"green";

  $out .= "<tr>
           <td bgcolor=$col colspan=4><center><font color=white>
                 <b>Python Configuration on $host:&nbsp; $total</td>";

  $out .= "</table>";
  $out .= "<br>[ <a href=/?GO=FCConfig::Python::details&method=PY target=details>Report Log</a> ]";
#  $out .= "</center><pre>" . Dumper($ZONES) ."</pre>";
  return $out;

}

sub details {
  my($q) = @_;
  print "<body bgcolor=white>
 <table border=0 cellspadding=2 width=100% bgcolor=#666699>
  <tr><td><b><font color=white>Config Report Log: $q->{method}</table><pre>";
  open(O, System->get_home() . "/DATA/config_verify.$q->{method}");

  my @a = <O>; close(O);
  print join("",@a);
}

1;

