package FCRules::Rule4;
use TO;
use Data::Dumper;
use strict;

# WRITE
sub description {
  return "Target Device is one of multiple device (A5K) and is First device in loop and counter is detected";
}

sub category {
  return "NotUsed-CRC";
}

sub run {
  my($class, $th, $count, $key, $info, $reads, $writes,
     $fc_type) = @_;

  my($hba, $name, $port, $type) = split(/\|/, $key);

  return (undef,undef) if ($type ne "a"); # A5K only

  my $port_type = substr($port,0,1) ; # s=ses, f=front, r=rear
  my $port_no = substr($port,1) + 0;

 
  my $a5_no = $TO::NamesH->{"a:$name"};

  if ($port_no != 0 || $port_type eq "s") {
     print "Not first disk in enclosure $name<br>";
     return (undef,undef) ;
  }
  print "is an a5 , name=$name, no=$a5_no, port=$port_no\n";

  # first disk?

  my $a5 = $TO::Storages->[$a5_no];
  my $a5_info = $a5->{info};

  my $hba_path = $info->{"hba_a" . $hba};

  my $linkno =  FCRules->a5kIsFirst($hba_path, $a5, $port_type);

  return (undef,undef) if (!defined($linkno));

  my $hba_no = $hba;

  print "port $port on $name is first ,hbano=$hba_no<br>";
 
  my $links = $a5->{ses};  # ses are the controllers
  my $link  = $links->[$linkno];

  my($ltype, $hno, $pno) = split(/:/, $link);
  print "ltype=$ltype, name=$name, port=$pno<br>";

  if ($ltype eq "h" || $ltype eq "s") {      # connected to a host or dport
        my($level, $cnt, $desc, $mins) =
              $th->test("${fc_type}counters","rule4","$name-$port_no" , 1);
        $mins = 1 if (!$mins);
        print "level=$level<br>";
        if ( $level eq "E") { # error level threshold fired
           print "   <b><font color=red>Rule 4 fired</font></b>\n";
           if ($ltype eq "h") {        # host
              my $host = $TO::Hosts->[$hno];
              my $hbas = $host->{portInfo};
              $hba  = $hbas->[$pno];
              my $o = Dumper($hba);
              print "   Origin: Hba $pno on $host->{info}{name}</b><pre>\n " .
                  substr($o, 7) . "<br>";
              print "   Destination: A5k: $name, disk:$port \n";

              FCRules->log(4,
                      "$host->{info}{name}:$pno",
                      "a:$name:$linkno",
                      "Disk $port in $name", $reads, $writes);

           } elsif ($ltype eq "s") {  # switch
              my $sw = $TO::Switches->[$hno];
              my $sw_info = $sw->{info};
              my $sw_port = $pno;
              print "   <b>Origin Point</b>: Dport $sw_port on Switch $sw_info->{name}</b><pre>\n";
              print "   <b>Destination Point</b>: A5k: $name, disk:$port \n";

              FCRules->log(4,
                      "$sw_info->{name}:$sw_port",
                      "a:$name:$linkno",
                      "Disk $port in $name", $reads, $writes);
           }
        }
   
  } else {
        return ("A5K should be connected to a host or switch : ltype=$ltype",undef);
  }

  return (undef, "");
}


1;


