package FCRules::Rule6;
use TO;
use Data::Dumper;
use FCRules;
use strict;

# READ
sub description {
  return "dPort error in switch";
}
# last is r10 using interface A
# last is f10 using interface B
# scsi: [ID 107833 kern.warning] WARNING: /pci@6,2000/pci@2/SUNW,qlc@4 /fp@0,0/ssd@w220000203722c4bd,0 (ssd251):


sub category {
  return "CRC";
}

sub run {
  my($class, $To, $hm, $report, $th, $count, $key, $info, $reads, $writes,
     $fc_type) = @_;

  my($hba, $name, $port0, $type) = split(/\|/, $key);
  return(undef,undef) if ($type ne "switch");

  my($sw, $port, $sw_info, $port_info, $port_type, $CIMKey1, $CIMKey2, $key1, $key2);

  $port = substr($port0,5) + 0;

  $sw = $To->nodeByName("switch:$name");
  if (!$sw) {
    return ("FCRule6: Cannot find switch $name in Topology", undef);
  }
  $sw_info = $sw->{info};

  $port_info = $sw->{portInfo};
  
  my $pt = $port_info->[$port]{sw_PortType};
  return(undef,undef) if ($pt ne "TL_Port" && $pt ne "SL_Port");

#  $port_type = ($port_info->[$port]{RegisterName} =~ /-ip\d+\_/) ? "i" : "d";
#  return (undef,undef) if ($port_type ne "d"); 

  my $links = $sw->{port};
  my $link  = $links->[$port]; # where is the port going

  my($ltype, $storage_name, $pno) = split(/:/, $link);

  if ($ltype eq "host") { 
     return ("Error: dport is pointing to a host", undef);
  }

  my $st = $To->nodeByName("$ltype:$storage_name");
  if (!$st) {
     return ("Nothing attached to $name:$port0");
  }

  my $st_info = $st->{info};
  my $stI = $st->{portInfo};
  my $target_name= $st_info->{name};

  my($level, $cnt, $desc, $mins) = 
              $th->test("${fc_type}counters","rule6","$name.$port" , $count);
  $mins = 1 if (!$mins);

  if ( $level eq "E") { # error level threshold fired
    Debug->print2("Rule 6 fired");

   my($ctrl, $port_desc);

   my $link0 = "|$storage_name|$pno|$ltype";
   ($CIMKey1, $key1, $ctrl, $port_desc) = FCRules->findPortKeys($link0, $info, $st);

    $desc = "$fc_type-ERROR ($cnt in $mins mins): Origin: $port_desc on " . $st->id() . 
          ". Destination: Port $port on " . $sw->id()  . ":";

    my($code);
    $code = ($ltype eq "a5k") ? "Ra5k":"R6";
    my $pc  = Util->getInfo("Probable_Causes", $code);
    my $ra  = Util->getInfo("Recommended_Actions", $code) .
" Register-Origin : $stI->[$pno]{RegisterName}
 Register-Destin.: $port_info->[$port]{RegisterName}";

    $CIMKey2 = $sw->key() . ".port.$port";
    $key2    = $sw->name() . ":$port";

    $hm->linkEvent($report, $ltype, $CIMKey1, $key1, "switch", $CIMKey2 , $key2,
                    $desc, $reads, $writes, undef, $pc,$ra);

  }

  return (undef, "");
}


1;


