package Health;
#  $Name:  $ 
#  $Id: Health.pm,v 1.36 2001/08/23 00:24:20 ccadieux Exp $

use Report;
use Carp;
use CIM::Instance;
use strict;
use Message;
use State;

sub new {
  my($class) = @_;

  my($x) = {};

  bless($x, "Health");
  return $x;
}


###########################################################
#  TRANSITION STUFF
#  Will keep track of the value of an attribute and return
#  ON if a transition occured, OFF if the value changed
#  and ON_AGAIN if it's been $repeat hours and it's still bad.
#
#
#  Ex: t300:diag230.status = 'CC'

#  $key   (t300:diag230) : how to save this information in the cache
#  $code  (status      ) : name of this attribute
#  $value ('CC'        ) : current value of the attribute
#  $tran_value ('CC'   ) : transition value , can be a ARRAYREF
#  $repeat('24h'|'60m' ) : how long to wait before repeating the same info.

#  Returns: IN      : In the transition.
#           IN_AGAIN: Still in for $repeat time.
#           LOST    : Lost comunication, nothing to report
#           OUT     : Out of the transition.
#           ""      : All fine, noithing to report.
#
###########################################################


sub getTransition {
  my($hm,  $arg) = @_;
  my($t, $now);
  my($key) = "transition-". $arg->{key}     || croak("getTransition requires a key ");
  my($code) = $arg->{code}   || croak("getTransition requires a code");
  my($value) = $arg->{value} ;
  my($tran_value) = $arg->{transition_value} || croak("getTransition requires a transition value");
  my($repeat) = $arg->{repeat};
  my($pdm) = $hm->{pdm};

  if ($repeat && $repeat !~ /\d+h/ && $repeat !~ /\d+m/) {
    croak("getTransition repeat value must be XXh or XXm");
  }
  my($now) = int(time/60);
  my($h) = $pdm->getCacheHandle($key);

  if (&set_equals($value, $tran_value) ) {  # comm lost.
     if (!$h->{$code} || $h->{$code}{value} ne $value)  { # transition
        $h->{$code}{value} = $value;
        $h->{$code}{time} = $now;
        return "IN";

     } elsif ($h->{$code}{value} eq $value) { # still IN
        if ($repeat) {
           $t = ($repeat =~ /\d+h/)? $repeat * 60: $repeat+ 0;
           if ($now - $h->{$code}{time} > $t) {
              $h->{$code}{time} = $now;
              return "IN_AGAIN";
           }
        }
     }
     return "LOST";

  } else {  # communication present 
     if (!$h->{$code}) {
        $h->{$code}{value} = $value;
        $h->{$code}{time} = $now;
        return "";

     } elsif (&set_equals($h->{$code}{value}, $tran_value) ) { # OUT
        $h->{$code}{value} = $value;
        $h->{$code}{time} = $now;
        return "OUT";
     }
  }
}

# true if val is in set
sub set_equals {
  my($val, $set) = @_;
  my($i);

  if (ref($set) eq "ARRAY") {
    foreach $i (@$set) {
      return 1 if ($val eq $i);
    }
    return 0;
  } else {
    return ($val eq $set);
  }
}

sub saveAlert {
  my($hm, $ev, $level, $id, $report, $cause, $desc) = @_;
  my($pdm) = $hm->{pdm};

  my($type) = $ev->propertyByName("EventType")->value;
  my($cap) = $ev->propertyByName("Caption")->value;
  $type .= ".$cap" if ($cap);

  my($alert) = CIM::Instance->new('NWS_AlertIndication', [
                  [ AlertId      => $id  ],
                  [ Description  => $desc],
                  [ AlertType    => $type],
                  [ Severity     => $level],
                 [ ProbableCause => $cause],
                  ]);
  my($pertains) =  CIM::Instance->new('NWS_AlertPertainsToEvent', [
                  [ Event        => $ev ],
                  [ Alert        => $alert ],
                  ]);

  my($ed) = Message->new({ id     => $report->id,
                            instances => [$alert, $pertains],
                            type      => Message::TYPE_ALERT ,
                            });
  $pdm->saveMessage($ed);
}


sub locationChangeEvent {
  my($hm, $report, $rep, $orep, $keyval) = @_;

  my($ev, $key, $sd, $pertains, $ed);
  my($pdm) = $hm->{pdm};
  my($cat) = $report->category();
  my($name) = $report->id('display');

  my($old) = "Contract:"   . $orep->get("location.contractNo")  . ", " . 
             "HostId:"     . $orep->get('location.hostId')      . ", " .
             "CustomerNo:" . $orep->get("location.customerNo")  . ", " .
             "Site:" .       $orep->get("location.siteName")    . ", " . 
                             $orep->get("location.siteAddress") . ", " .
                             $orep->get("location.siteCity")    . ", " .
                             $orep->get("location.siteState")   . ", " .
                             $orep->get("location.siteZip")     . ", " .
                             $orep->get("location.siteContact") . "," .
                             $orep->get("location.siteEmail") ;

  my($new) = "Contract:"   . $rep->get("location.contractNo")   . ", " . 
             "HostId:"     . $rep->get('location.hostId')       . ", " .
             "CustomerNo:" . $rep->get("location.customerNo")   . ", " . 
             "Site:" .       $rep->get("location.siteName")    . ", " . 
                             $rep->get("location.siteAddress") . ", " .
                             $rep->get("location.siteCity")    . ", " .
                             $rep->get("location.siteState")   . ", " .
                             $rep->get("location.siteZip")     . ", " .
                             $rep->get("location.siteContact") . "," .
                             $rep->get("location.siteEmail") ;

  if ($old ne $new) {
    $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => lc($cat) . ".LocationChange"  ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "Location of $cat $name was changed" ],
                  [ Data        => "Location Changed to: $new" ],
                         ]);
     my($key) = CIM::Key->new( ['NWS_System',
                   'Name'       => $keyval,
                    CreationClassName => 'NWS_System']);

     $sd = Events->sourceDetector({ event => $ev });

     $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $key ],
                    ]);

     my($loc) = NWS->location($rep);

     my($locp) = CIM::Instance->new('NWS_AgentLocation', [
                  [ Agent       => $sd->[0]  ],
                  [ Element     => $loc ],
                    ]);


     $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd, $pertains, $loc, $locp ],
                          });
     $pdm->saveMessage($ed);
  }
     
}

sub alarmEvent {
  my($hm, $data, $report, $keyval, $desc) = @_;

  my($ev, $key, $sd, $pertains, $ed);
  my($pdm) = $hm->{pdm};
  my($cat) = $report->category();
  my($id)  = $pdm->getEventSequence ;
  my($rep) = $report->content();

  $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => lc($cat) . ".AlarmEvent"  ],
                  [ EventId     => $id ],
                  [ Data        => $data],
                  [ Caption     => "AlarmEvent" ],
                  [ Actionable  => 1  ],
                  [ SourceIP    => $rep->{'id.ipno'}],
                  [ Description => $desc ],
                         ]);
   my($key) = CIM::Key->new( ['NWS_System',
                   'Name'       => $keyval,
                    CreationClassName => 'NWS_System']);
   $sd = Events->sourceDetector({ event => $ev });

   $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $key ],
                    ]);

   $ed = Message->new( { id        => $report->id,
                          instances => [$ev, @$sd, $pertains ],
                        });
   $pdm->saveMessage($ed);

   $hm->saveAlert($ev,  Message::SEVERITY_WARNING, $id, $report, 
       $report->id("logFile") . "\n $data" , $desc);
}



sub logEvent {
  my($hm, $log, $report, $keyval, $desc, $save) = @_;

  my($ev, $key, $sd, $pertains, $ed);
  my($pdm) = $hm->{pdm};
  my($cat) = $report->category();
  my($id)  =  $pdm->getEventSequence ;
  my($id2) =  $report->id('display');

  my($rep) = $report->content();
  my($name)= $rep->get('id.name') if ($rep);

  if ($save) {
     open(O, ">" . System->get_home() . "/DATA/dump/${cat}LogFile");
     print O $log;
     close(O);
  }
  $desc =  "$cat.MessageLog on $id2:" if (!$desc);
  $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => lc($cat) . ".AlarmEvent"  ],
                  [ EventId     => $id ],
                  [ Data        => $log ],
                  [ Caption     => "MessageLog" ],
                  [ Actionable  => 1],
                  [ SourceIP    => $rep->{'id.ipno'}],
                  [ Description => $desc ] 
                         ]);
   my($key) = CIM::Key->new( ['NWS_System',
                   'Name'       => $keyval,
                    CreationClassName => 'NWS_System']);
   $sd = Events->sourceDetector({ event => $ev });

   $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $key ],
                    ]);

   $ed = Message->new( { id        => $report->id,
                          instances => [$ev, @$sd, $pertains ],
                        });
   $pdm->saveMessage($ed);

   $hm->saveAlert($ev,  Message::SEVERITY_WARNING, $id, $report, 
       $report->id("logFile") . "\n $log", $desc );

}


# pc = probable cause
# ra = recomm actions
sub linkEvent {
  my($hm, $report, $type1, $CIMKey1, $key1, $type2, $CIMKey2, $key2,
      $desc, $reads, $writes, $hba, $pc, $ra) = @_;

  my($ev, $key, $sd, $pertains, $ed, $ehba);
  my($pdm)   = $hm->{pdm};
  my($id)    =  $pdm->getEventSequence ;

  my($e_hba, $k1, $host_hba, $f1, $f2, $f3, $hostid, $link1, $link2);
  my(@hba_list, $ix, $node1, $loc);

  if ($type1 eq "host" || $type2 eq "host") {
    
     if ($type1 eq "host") {
       ($f1, $f2, $f3) = split(/:/, $key1);
       $ix = index($CIMKey1, ".");
       $hostid = substr($CIMKey1, 0, $ix);
       $node1 = $CIMKey1;
       $loc   = substr($CIMKey1, $ix+1);
     } else {
       ($f1, $f2, $f3) = split(/:/, $key2);
       $ix = index($CIMKey2, ".");
       $hostid = substr($CIMKey2, 0, $ix);
       $node1 = $CIMKey2;
       $loc   = substr($CIMKey2, $ix+1);
     }

     $e_hba = CIM::Instance->new('NWS_HbaPort', [
                  [ Tag         =>  $node1  ],
                  [ PortWWN     =>  $hba->{PortWWN} ],
                  [ NodeWWN     =>  $hba->{NodeWWN} ],
                  [ RegisterName=>  $hba->{RegisterName}],
                  [ LGroup      =>  $hba->{LGroup}],
                  [ HbaNumber   =>  $f3],
                  [ Path        =>  $hba->{path}],
                         ]);

     $k1 = CIM::Key->new( ['NWS_Host',
                   Name       => $hostid, 
                   CreationClassName => 'NWS_Host']);

     $host_hba = CIM::Instance->new('NWS_HostHba', [
                  [ Host    => $k1  ],
                  [ Hba     => $e_hba],
                  [LocationWithinContainer => $loc],
                    ]);

     @hba_list = ($e_hba, $host_hba);

     if ($type1 eq "host") {
       $link1 = $e_hba;
       $link2 = CIM::Key->new( ['CIM_Card',
                   Tag       => $CIMKey2,
                   CreationClassName => 'CIM_Card']);
     } else {
       $link2 = $e_hba;
       $link1 = CIM::Key->new( ['CIM_Card',
                   Tag       => $CIMKey1,
                   CreationClassName => 'CIM_Card']);
     }

  } else {
     $link1 = CIM::Key->new( ['CIM_Card',
                   Tag       => $CIMKey1,
                   CreationClassName => 'CIM_Card']);
     
     $link2 = CIM::Key->new( ['CIM_Card',
                   Tag       => $CIMKey2,
                   CreationClassName => 'CIM_Card']);
  }
  
  $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => "san.LinkEvent"  ],
                  [ EventId     => $id ],
                  [ Caption     => "${type1}To$type2" ],
                  [ Actionable  => 1],
                  [ SourceIP    => System->get_ipno() ],
                  [ Description => $desc ],
                  [ Data        => "Probable Cause: $pc\nRecommended Action: $ra"]
                         ]);

   $sd = Events->sourceDetector({ event => $ev });

   my($link) = CIM::Instance->new('NWS_Link', [
                  [ Link1       => $link1 ],
                  [ Link2       => $link2 ],
                    ]);

   $pertains = CIM::Instance->new('NWS_EventPertainsToLink', [
                  [ Event       => $ev  ],
                  [ Link        => $link ],
                    ]);

   $ed = Message->new({ id        => $report->id,
                        instances =>
                        [$ev, @$sd, @hba_list, $link, $pertains ],
                      });

   $pdm->saveMessage($ed);

   State->saveLinkState($type1, $key1, $type2, $key2, 
         Message::SEVERITY_WARNING, $desc, $reads, $writes, $pc, $ra);

   $hm->saveAlert($ev,  Message::SEVERITY_WARNING, $id, $report, "", $desc );

}
  
  
# 2=outoband, 1=inband 3=both

sub connectionEvent {
  my($hm, $keyval, $report, $args) = @_;
  my($key, $ev, $sd, $pertains, $ed, $id, $text, $rc);
  my($cat) = $report->category();
  my($rep) = $report->content();
  my($name)= $rep->get('id.name') if ($rep);

  if ($args->{method} == 3) {  # BOTH
    $text = "OutOfBand";

    my($tran) = $hm->getTransition({      key => $report->deviceName(),
                                         code => 'status',
                                        value => $report->status(),
                             transition_value => Report::STATUS_CANNOT_CONNECT,
                                       repeat => '5h',
                                 });
    $rc = $hm->connection($keyval, $tran, $report, $text, $args);

    if (defined($rep->{'info.IB_status'})) {  # T3 only so far.
       my $host = $rep->{'info.IB_host'}; 

       $text = "InBand(" . Util->shortHostname($host) . ")";

       my($tran) = $hm->getTransition( {  key => $report->deviceName(),
                                         code => 'statusIB',
                                        value => $rep->{'info.IB_status'},
                             transition_value => Report::STATUS_CANNOT_CONNECT,
                                       repeat => '5h',
                                       });
       $hm->connection($keyval, $tran, $report, $text, $args);
    }
    return $rc;

  } else {
    my $ho = System->hostname();
    $text = ($args->{method} == 1) ? "InBand($ho)": "OutOfBand($ho)";

    my($tran) = $hm->getTransition({      key => $report->deviceName(),
                                         code => 'status',
                                        value => $report->status(),
                             transition_value => Report::STATUS_CANNOT_CONNECT,
                                       repeat => '5h',
                                 });
    return $hm->connection($keyval, $tran, $report, $text, $args);
  }

}

sub connection {
  my($hm, $keyval, $tran, $report, $text, $args) = @_;
  my($pdm) = $hm->{pdm};
  my($key, $id, $desc);
  my($ev, $sd, $pertains, $ed);
  my($cat) = $report->category();
  my($rep) = $report->content();
  my($name)= $rep->get('id.name') if ($rep);
  my($id2)  = $report->id('display');

  $key = CIM::Key->new( ['NWS_System',
                   'Name'             => $keyval,
                    CreationClassName => 'NWS_System']);

  return 1 if ($tran eq "LOST");

  if ($tran =~ /IN/) { #lost  or lost-repeat
     return 1 if (!$keyval);  # never had good connection
     $id =  $pdm->getEventSequence;
     my $reboot = "( last reboot was " . System->lastReboot() . ")" 
                 if ($text =~ /InBand/);
     $desc = "Lost communication ($text) with $id2 $reboot";
     $ev = CIM::Instance->new('NWS_CommunicationLostEvent', [
                  [ EventType   => lc($cat) . ".CommunicationLostEvent"  ],
                  [ EventId     => $id  ],
                  [ Caption     => $cat ],
                  [ Description => $desc],
                  [ Method => $args->{method} ],
                  [ Actionable  => 1],
                  [ SourceIP     => $rep->{'id.ipno'}],
                  [ Timeout     => 20],
                         ]);
     $sd = Events->sourceDetector({ event => $ev });

     $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $key ],
                    ]);

     State->saveState($cat,  $keyval, "", $name, 
         Message::SEVERITY_DOWN, "Lost Communication ($text) with $id2");

     $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd, $pertains ],
                          });
     $pdm->saveMessage($ed);

     $hm->saveAlert($ev,  Message::SEVERITY_DOWN, $id, $report, "", $desc);
     return 1;
  }
  if ($tran eq "OUT") { # can connect
     $desc = "Regained communication($text) with $id2";
     $id =  $pdm->getEventSequence;
     $ev = CIM::Instance->new('NWS_CommunicationEstablishedEvent', [
                  [ EventType   => lc($cat) . ".CommunicationEstablishedEvent"  ],
                  [ EventId     => $id ],
                  [ Description => $desc ],
                  [ Caption     => $cat ],
                  [ Actionable  => 1],
                  [ SourceIP     => $rep->{'id.ipno'}],
                  [ Method      => 1],
                         ]);
     $sd = Events->sourceDetector({ event => $ev });


     $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $key ],
                    ]);

     State->saveState($cat,  $keyval, "", $name, 
                  Message::SEVERITY_GREEN, "Regained Communication ($text)");

     $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd, $pertains ],
                            });

     $pdm->saveMessage($ed);

     $hm->saveAlert($ev,  Message::SEVERITY_WARNING, $id, $report,"",$desc);
  }
  return 0;  # keep going
}

#
# returns availability-state of the device (old,new, description) :
#    returns (1/0, 1/0, "Old=..., New=...");

sub status {
   croak("Health::status is abstract, must implement in device Health");
}

sub stateEvent {
  my($hm, $component, $report,  $rep, $orep, $cim1, $nserial, $keyval, $volKey) = @_;
  my($key, $ev, $sd, $pertains, $ed , $id);
  my($ovalue, $nvalue, $old, $new) = $hm->status($rep, $orep, $component);
  my($name)= $rep->get('id.name');
  my($data) = "";
  my($ix) = index($component, ".");
  my($id2) = $report->id('display');

  my($comp0) = ($ix)? substr($component, 0, $ix): $component;
   if ($ovalue ne $nvalue ||
       ($rep == $orep && !$new) ) { # availability change.
         my($cat) = $report->category();
         my($catcap) = uc($cat);
         my($pdm) = $hm->{pdm};
         my($newval); 
         if ($new eq "X") {
            $newval = "Unknown";
         } else {
            $newval = ($new)?"Available":"Not-Available";
         }
         if ($rep == $orep) {
            $old = 1;
            $ovalue = "unknown";
         }
         $id =  $pdm->getEventSequence;
         my $xx = (index($component, $nserial) < 0) ? "($nserial) ":" ";
         my($desc) = "'$component'${xx}in $catcap $id2 is now $newval (status-state changed from '$ovalue' to '$nvalue'):";

         if (!$nserial) {
             $hm->alarmEvent("", $report, $keyval, $desc);
             if (!$new) { #  bad comp
                State->saveState($cat,  $keyval, $component, $name, 
                 Message::SEVERITY_ERROR, $desc);
             } else {
                State->saveState($cat,  $keyval, $component, $name, 
                 Message::SEVERITY_GREEN, $desc);
             }
             return;
         }
         $old = 0 if ($old eq "X");
         my $yellow;
         if ($new eq "X") {
            $new = $old; 
            $yellow =1;
         }
         $ev = CIM::Instance->new('NWS_StateChangeEvent', [
             [ EventType    => lc($cat) . ".StateChangeEvent"   ],
             [ EventId      => $id ],
             [ PriorState   => $old],
             [ CurrentState => $new],
             [ PriorValue   => $ovalue],
             [ CurrentValue => $nvalue],
             [ Actionable   => 1     ],
             [ Caption      => $comp0],
             [ Description  => $desc ],
             [ Data         => $data],
             [ SourceIP     => $rep->{'id.ipno'}],
                    ]);

         $sd = Events->sourceDetector({ event => $ev });

         $key = CIM::Key->new( [$cim1,
                  Tag               => $nserial,
                  CreationClassName => $cim1]);

         $pertains = CIM::Instance->new('NWS_EventPertainsToPhysicalElement', [
             [ Event       => $ev  ],
             [ Element     => $key ],
               ]);

         $ed = Message->new( { id        => $report->id,
                       instances => [$ev, @$sd, $pertains ],
                       });

         $pdm->saveMessage($ed);
         my $severity;
         if (!$new) { #  bad comp
           $severity = Message::SEVERITY_ERROR;
         } elsif ($yellow) {
           $severity = Message::SEVERITY_WARNING;
         } else {
           $severity = Message::SEVERITY_GREEN;
         }
         State->saveState($cat,  $keyval, $component, $name, $severity, $desc);
         $hm->saveAlert($ev,  $severity, $id, $report, "", $desc);
   }
}

sub removeCompEvent {
  my($hm, $component, $pf, $report,  $cim1, $oserial, $keyval) = @_;
  
  my($key, $ev, $sd, $pertains, $ed, $contain);
  my($pdm) = $hm->{pdm};
  my($cat) = $report->category();

  my($ix) = index($component, ".");
  my($comp0) = ($ix)? substr($component, 0, $ix): $component;

  my($id)  =  $pdm->getEventSequence;
  my($rep) = $report->content();
  my($name)= $rep->get('id.name') if ($rep);
  my($comp1) = $component;
  $comp1  = "" if (index($oserial, $component) >= 0);
  my($id2) = $report->id('display');

  my($desc) = "$comp1($oserial) was removed from $id2";
  $ev = CIM::Instance->new('NWS_Event', [
             [ EventType   => lc($cat) . ".TopologyEvent"  ],
             [ Caption     => "$comp0-remove"],
             [ Description => $desc ],
             [ Actionable   => 1     ],
             [ SourceIP     => $rep->{'id.ipno'}],
             [ EventId     => $id ]]);

   $sd = Events->sourceDetector({ event => $ev });

   $key = CIM::Key->new( [$cim1,   # key of the disk
                  Tag               => $oserial,
                  CreationClassName => $cim1]);

   $contain = CIM::Instance->new( 'CIM_Container', [
                  [GroupComponent => $pf],            # frame is a package
                  [PartComponent  => $key],           # package is an element
                  [LocationWithinContainer => $component],
                   ]);

   $pertains = CIM::Instance->new('NWS_EventPertainsToContainer', [
                  [Event          => $ev   ],
                  [Association    => $contain ],
                  [DiscoveryType => "D"   ],   # delete
               ]);

   State->saveState($cat,  $keyval, $component, $name, 
              Message::SEVERITY_ERROR, $desc);

   $ed = Message->new( { id        => $report->id,
                       instances => [$ev, @$sd, $contain, $pertains ],
                       severity  => Message::SEVERITY_ERROR});

   $pdm->saveMessage($ed);
   $hm->saveAlert($ev,  Message::SEVERITY_ERROR, $id, $report, "",$desc);
}

sub insertCompEvent {
  my($hm, $component, $pf, $report, $part, $nserial, $keyval) = @_;
  my($key, $ev, $sd, $pertains, $ed, $contain, $pertains2);

  my($pdm) = $hm->{pdm};
  my($cat) = $report->category();
  my($ix) = index($component, ".");
  my($comp0) = ($ix)? substr($component, 0, $ix): $component;

  my($rep) = $report->content();
  my($name)= $rep->get('id.name') if ($rep);

  my($id) =  $pdm->getEventSequence;
  my($id2) = $report->id('display');

  my($comp1) = $component;
  $comp1  = "" if (index($nserial, $component) >= 0);

  my($desc) = "$comp1($nserial) was added to $id2";

  $ev = CIM::Instance->new('NWS_Event', [  
             [ EventType   => lc($cat) . ".TopologyEvent"  ],
             [ Caption     => "$comp0-add"],
             [ Description => $desc ],
             [ Actionable   => 1     ],
             [ SourceIP     => $rep->{'id.ipno'}],
             [ EventId     => $id ]]);

  $sd = Events->sourceDetector({ event => $ev });

  $pertains = CIM::Instance->new('NWS_EventPertainsToPhysicalElement', [
             [ Event       => $ev  ],
             [ Element     => $part->[1]],
               ]);

  $contain = CIM::Instance->new('CIM_Container', [
              [GroupComponent           => $pf],         #PhysicalFrame
              [PartComponent            => $part->[1]],
              [LocationWithinContainer  => $component],
               ]);

  $pertains2 = CIM::Instance->new('NWS_EventPertainsToContainer', [
              [Event                    => $ev],
              [Association             => $contain  ],
              [DiscoveryType            => "A"       ],
               ]);

  State->saveState($cat,  $keyval, $component, $name, 
           Message::SEVERITY_NORMAL, $desc);

  $ed = Message->new( { id        => $report->id,
                       instances => [$ev, @$sd, @$part, $pertains , $contain, $pertains2],
                       severity  => Message::SEVERITY_ERROR});

  $pdm->saveMessage($ed);
  $hm->saveAlert($ev,  Message::SEVERITY_WARNING, $id, $report, "", $desc);
}

1;
