package A3500FCHealth;
#  or A1000
#  $Name:  $ 
#  $Id: A3500FCHealth.pm,v 1.10 2001/06/22 21:04:51 ccadieux Exp $


use base 'Health';
use NWS::A3500FC;
use Ilist;
use Message;
use Catalog;
use Report;
use Events;
use strict;
use Debug;
sub revision {'$Revision: 1.10 $'}

sub new {
  my($hm, $pdm) = @_;

  my($self) = {pdm => $pdm};
  bless ($self, 'A3500FCHealth');

# Specify each callback
#                            HM  Callback,  Filters
  $pdm->reportRequest($self, 'all_logic' , {category => Report::CAT_A3500FC});
  $pdm->reportRequest($self, 'all_logic' , {category => Report::CAT_A1000});
  return $self;
}

#####################################################

sub all_logic {
  my($hm, $report) = @_;
  my($pdm) = $hm->{pdm};
  my($ev, $ed, $pertains, $x ,$y, $contain, $sd);
  my($orep, $key);
  my($rep)     = $report->content;
  my($oreport) = $pdm->getLastReport($report->fileKey);
  my($access)  = $report->accessName;
  my($category) = $report->category();
  my($ucategory) = lc($category);
  my($id)        = $report->id('display');
  TO->setIncomplete() if (!$oreport);

  my $cinfo    = Catalog->deviceRead('a3500fc',Catalog::Name, $access);
  my $key      = $cinfo->[Catalog::Key];
  my($log)     = $report->log();
  if (!$key) {
     Debug->err(ERROR => "This $category ($access) cannot be identified");
     return;
  }
  CIM->version("1.1");
  $DB::single = 1;

  $hm->logEvent($log, $report, $key) if ($log);
  return if ($hm->connectionEvent($key, $report, {method => 1}));

  my($audit); 
  my($freq) = System->get_renv()->{audit_freq} || 7;
  if (!$oreport || ($audit = $pdm->isXdays("Audit$key", $freq)) eq "YES" ) {  # new t300 or audit time.
     Catalog->newCatalog();
     my($etype) = $audit ? "$ucategory.AuditEvent" : "$ucategory.DiscoveryEvent";
     my($title) = $audit ? "Auditing":"Discovered";
     my($dt)    = "A" if (!$audit);
     $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => $etype  ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "$title a new $ucategory called $id"],
                         ]);

     $sd = Events->sourceDetector({ event => $ev, host => 1, rep => $rep });
      
     my($p) = NWS::A3500FC->newSystem($rep,$category);

     $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $p->[0] ],
                  [ DiscoveryType => $dt],
                    ]);

     my($monitors) = CIM::Instance->new('NWS_AgentMonitors', [
                  [ Agent       => $sd->[0]  ],
                  [ Element     => $p->[0] ],
                    ]);

     $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd,  @$p, $monitors, $pertains ],
                            severity  => Message::SEVERITY_NORMAL });

     $pdm->saveMessage($ed);
     if ($audit) {  # dump stats.

        $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => "$ucategory.Statistics"  ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "Statistics about $ucategory with ID:$key:"],
                  [ Data        => Util->nnl($rep->toString('device_info.'))],
                         ]);

        $sd = Events->sourceDetector({ event => $ev});

        my($key1) = CIM::Key->new( ['NWS_System',
                   'Name'             =>  $rep->get('id.ctrl_id'),
                    CreationClassName => 'NWS_System']);

        $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $key1 ],
                    ]);

        $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd, $pertains ],
                            severity  => Message::SEVERITY_NORMAL });

        $pdm->saveMessage($ed);

     }
  }
  $orep = ($oreport)? $oreport->content() : $rep; # same report if new disco.

  $hm->locationChangeEvent($report, $rep, $orep, $key);

  my($physicalFrame) = CIM::Key->new( ['CIM_PhysicalFrame',   # key of the frame
                        Tag               => $key,
                        CreationClassName => 'CIM_PhysicalFrame' ]);

  if ($rep->get("device_info.healthck") ne 
      $orep->get("device_info.healthck")) {
     my $desc0 = "HealthCheck on '$key' returns: " . 
                $rep->get("device_info.healthck") ;
     $hm->alarmEvent( "", $report, $key, $desc0);
  }

  my($CTRL) = $rep->get('controller.count');
# CONTROLLER (1)
# check if disks were replaced or their state has changed.

  my($comp);
  foreach $x ('a','b') {
     my($oserial, $nserial);
     if ($rep->get("controller.$x.device")) {
       $oserial =  $orep->get("device_info.$x.Vendor_ID") . "." . 
                   $orep->get("controller.$x.serial");
       $nserial = $rep->get("device_info.$x.Vendor_ID") . "." .
                  $rep->get("controller.$x.serial");
       $oserial = "" if ($oserial eq ".");
       $nserial = "" if ($nserial eq ".");

       $comp = "controller.$x";
       if ($oserial eq $nserial) {
          $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_Card', $nserial, $key);
       } else {
          if ($oserial) {
            $hm->removeCompEvent($comp,$physicalFrame,$report,'CIM_Card',$oserial, $key);
          }
          if ($nserial) {
            my($part) = NWS::A3500FC->newController($rep, $comp, $category);
            $hm->insertCompEvent($comp,$physicalFrame, $report, $part, $nserial, $key);
          }
       }
     }
  }

# DISKS (9)
  for ($y=0; $y < $rep->get('disk.count'); $y++) {
      $comp = "disk.$y";
      my($nserial) = $rep->get("disk.$y.vendor") . "." . $rep->get("disk.$y.serial");
      my($oserial) = $orep->get("disk.$y.vendor"). "." . $orep->get("disk.$y.serial");
      $oserial = "" if ($oserial eq ".");
      $nserial = "" if ($nserial eq ".");
      if ($oserial eq $nserial) {
           $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_PhysicalPackage', $nserial, $key);
      } else {
           if ($oserial) {
             $hm->removeCompEvent($comp,$physicalFrame,$report,'CIM_PhysicalPackage',$oserial, $key);
           }
           if ($nserial) {
             my($part) = NWS::A3500FC->newDiskDrive($rep, $comp, $category);
             $hm->insertCompEvent($comp,$physicalFrame, $report, $part, $nserial, $key);
           }
     }
  }
}

sub status {
   my($hm, $rep, $orep, $comp) = @_;
   my($pdm) = $hm->{pdm};
   my($map) = $pdm->getDeviceStateMap("a3500fc.availability");
   my($state,$ostate, $old, $new);

   if ($comp =~ /controller/) {
      $state  = $rep->getState("$comp.state");
      $ostate = $orep->getState("$comp.state");
      $old    = $map->get("controller.state.$ostate");
      $new    = $map->get("controller.state.$state");
   } else {
      $state  = $rep->getState("$comp.status");
      $ostate = $orep->getState("$comp.status");
      $old    = $map->get("disk.status.$ostate");
      $new    = $map->get("disk.status.$state");
   }


   return ($ostate, $state, $old, $new);
}

# will return the state of the mentioned component and 0/1 for availability
# static method
# comp = 
#=controller.a
#=controller.b
#=disk.0
#=disk.1
#=disk.2
#=disk.3
#=disk.4
#=disk.5
#=disk.6
#=disk.7
#=disk.8
#=disk.9


sub compStatus {
   my($hm, $rep, $comp) = @_;

   my($map) = PDM->getDeviceStateMap("a3500fc.availability");
   my($state, $new);

   if ($comp =~ /controller/) {
      $state  = $rep->get("$comp.state");
      $new    = $map->get("controller.state.$state");
   } else {
      $state  = $rep->get("$comp.status");
      $new    = $map->get("disk.status.$state");
   }

   $new = ($new eq "1")? 1:0;

   return ($state, $new);

}


1;
