package A5KHealth;
#  $Name:  $ 
#  $Id: A5KHealth.pm,v 1.22 2001/06/22 21:04:51 ccadieux Exp $


use base 'Health';
use NWS::A5K;
use Ilist;
use Message;
use Catalog;
use Report;
use Events;
use strict;
use Debug;
sub revision {'$Revision: 1.22 $'}


sub new {
  my($hm, $pdm) = @_;

  my($self) = {pdm => $pdm};
  bless ($self, 'A5KHealth');

# Specify each callback
#                            HM  Callback,  Filters
  $pdm->reportRequest($self, 'all_logic' , {category => Report::CAT_A5K});
  return $self;
}

#####################################################


sub all_logic {
  my($hm, $report) = @_;
  my($pdm) = $hm->{pdm};
  my($ev, $ed, $pertains, $x ,$y, $contain, $sd);
  my($orep, $comp);
  $DB::single = 1;
  my($rep)     = $report->content;
  my($oreport) = $pdm->getLastReport($report->fileKey);
  my($name)    =  $report->accessName;  # name of the A5K, by luxadm
  my $wwn      = Catalog->deviceRead('a5k',Catalog::Name, $name, Catalog::Key);
  TO->setIncomplete() if (!$oreport);


  my($log)     = $report->log();
  my($id)      = $report->id("display");
  CIM->version("1.1");
  if (!$wwn) {
     Debug->err(ERROR => "This A5000 ($name) cannot be identified");
     return;
  }

  return if ( $hm->connectionEvent($wwn, $report, {method => 1} )); # return if cannot_connect

  my($audit); 
  my($freq) = System->get_renv()->{audit_freq} || 7;
  if (!$oreport || ($audit = $pdm->isXdays("Audit$wwn", $freq)) eq "YES" ) {  # new t300 or audit time.
     A5KAgent->disks_display("*", $rep);  # add the disk details.
     Catalog->newCatalog();

     my($etype) = $audit ? "a5k.AuditEvent" : "a5k.DiscoveryEvent";
     my($title) = $audit ? "Auditing":"Discovered";
     my($dt)    = "A" if (!$audit);
     $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => $etype ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "$title a new A5K called $id"],
                         ]);

     $sd = Events->sourceDetector({ event => $ev , host => 1, rep => $rep});
      
     my($p) = NWS::A5K->newSystem($rep);

     $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $p->[0] ],
                  [ DiscoveryType => $dt],
                    ]);

     my($monitors) = CIM::Instance->new('NWS_AgentMonitors', [
                  [ Agent       => $sd->[0]  ],
                  [ Element     => $p->[0] ],
                    ]);

     $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd, @$p, $monitors, $pertains ],
                            severity  => Message::SEVERITY_NORMAL });

     $pdm->saveMessage($ed);
  }
  $orep = ($oreport)? $oreport->content() : $rep; # same report if new disco.

  $hm->locationChangeEvent($report, $rep, $orep, $wwn);
  my($physicalFrame) = CIM::Key->new( ['CIM_PhysicalFrame', # frame key 
                       Tag               => $wwn,
                       CreationClassName => 'CIM_PhysicalFrame' ]);

  my($j);
  foreach my $slot ('disk_front','disk_rear') {
    for ($j=0; $j < 30; $j++) {
      last if (!$rep->get("$slot.$j.state"));
      my($comp) = "$slot.$j";
      my($oserial) = Util->rtrim($orep->get("$comp.wwn")) ;
      my($nserial) = Util->rtrim($rep->get("$comp.wwn")) ;
      if ($oserial eq $nserial) {
         $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_PhysicalPackage', $nserial, $wwn);
      } else {
         if ($oserial) {
           if ($rep->{"$comp.status"} =~ /Bypassed/) {
              $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_PhysicalPackage', $nserial, $wwn);
             
           } else {
             $hm->removeCompEvent($comp,$physicalFrame, $report, 'CIM_PhysicalPackage',$oserial, $wwn);
           }
         }
         if ($nserial && $nserial !~ /^0+$/) {
           A5KAgent->disks_display($comp, $rep);  # add the disk details.
          
           my($disk) = NWS::A5K->newDiskDrive($rep, $comp);
           $hm->insertCompEvent($comp,$physicalFrame, $report,  $disk, $nserial, $wwn);
         }
      }
    }
  }
  for ($j = 0; $j <= 2 ; $j++) {
    $comp = "power.$j";
    $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_PhysicalPackage', $comp, $wwn);
  }
  my($s1, $s2);
  foreach $j ('A','B') {
    $comp = "interface_board.$j";
    $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_Card', $comp, $wwn);
    for my $j1 ('0','1') {
      $s2 = $orep->get("$comp.gbic$j1.status");
      $s1 = $rep->get("$comp.gbic$j1.status");
      if ($s1 ne $s2) {
         my($desc0) = "$comp.gbic$j1 status changed from $s2 to $s1 on $id";
         $hm->alarmEvent( "", $report, $wwn, $desc0);
         if ($s1 ne "O.K.") {
             State->saveState("A5K", $wwn, "$comp.gbic$j1", $name, 
               Message::SEVERITY_WARNING, $desc0);
         } else {
             State->saveState("A5K", $wwn, "$comp.gbic$j1", $name,
               Message::SEVERITY_GREEN, $desc0);
         }
      }
    }
  }

  for ($j=0; $j < 2; $j++) {
      $s2 = $orep->get("fan.$j.status");
      $s1 = $rep->get("fan.$j.status");
      if ($s1 ne $s2) {
         my($desc0) = "Fan $j status changed from $s2 to $s1 on $id";
         $hm->alarmEvent( "", $report, $wwn,  $desc0);
         if ($s1 ne "OK") {
             State->saveState("A5K", $wwn, "fan.$j", $name,
               Message::SEVERITY_WARNING, $desc0);
         } else {
             State->saveState("A5K", $wwn, "fan.$j", $name,
               Message::SEVERITY_GREEN, $desc0);
         }
      }
  }

  foreach $j ('backplane.front', 'backplane.rear') {
     $s2 = $orep->get("$j.status");
     $s1 = $rep->get("$j.status");
     if ($s1 ne $s2) {
         my($desc0) =  "$j status changed from $s2 to $s1 on $id";
         $hm->alarmEvent( "", $report, $wwn, $desc0);
         if ($s1 ne "OK") {
             State->saveState("A5K", $wwn, $j, $name,
               Message::SEVERITY_WARNING, $desc0);
         } else {
             State->saveState("A5K", $wwn, $j, $name,
               Message::SEVERITY_GREEN, $desc0);
         }
     }
  }
         

}


sub status {
   my($hm, $rep, $orep, $comp) = @_;
   my($pdm) = $hm->{pdm};
   my($map) = $pdm->getDeviceStateMap("a5.availability");
   my($state, $ostate, $status, $ostatus, $old, $new);

   if ($comp =~ /disk/) { # disk have state and status
     $state   = $rep->getState("$comp.state");
     $ostate  = $orep->getState("$comp.state");
     $status  = $rep->getState("$comp.status");
     $ostatus = $orep->getState("$comp.status");

     $old = $map->get("disk.status.$ostatus") . $map->get("disk.state.$ostate");
     $new = $map->get("disk.status.$status") . $map->get("disk.state.$state");

     $old = ($old eq "11")? 1:0;
     $new = ($new eq "11")? 1:0;
     return ("$ostatus-$ostate", "$status-$state", $old, $new);

   } else {
     $status  = $rep->getState("$comp.status");
     $ostatus = $orep->getState("$comp.status");

     $old = $map->get("status.$ostatus");
     $new = $map->get("status.$status");

     return ($ostatus, $status, $old, $new);
   }
}

#
# static method
# returns status-state and 0/1
#   disk_front.0
#   disk_front.1
#   disk_front.2
#   disk_front.3
#   disk_front.4
#   disk_front.5
#   disk_front.6
#   disk_front.7
#   disk_front.8
#   disk_front.9
#   disk_front.10
#   disk_rear.0
#   disk_rear.1
#   disk_rear.2
#   disk_rear.3
#   disk_rear.4
#   disk_rear.5
#   disk_rear.6
#   disk_rear.7
#   disk_rear.8
#   disk_rear.9
#   disk_rear.10
#   power.0
#   power.1
#   power.2
#   interface_board.A
#   interface_board.B



sub compStatus {
   my($hm, $rep, $orep, $comp) = @_;

   my($map) = PDM->getDeviceStateMap("a5.availability");
   my($state, $status, $new);

   if ($comp =~ /disk/) { # disk have state and status
     $state   = $rep->get("$comp.state");
     $status  = $rep->get("$comp.status");

     $new = $map->get("disk.status.$status") . $map->get("disk.state.$state");

     $new = ($new eq "11")? 1:0;
     return ("$status-$state", $new);

   } else {
     $status  = $rep->get("$comp.status");
     $new = $map->get("status.$status");

     return ($status, $new);
   }
}
   


1;
