package BrocadeHealth;
#  $Name:  $ 
#  $Id: BrocadeHealth.pm,v 1.6 2001/08/08 23:25:19 ccadieux Exp $


use base 'Health';
use NWS::Brocade;
use Ilist;
use Message;
use Report;
use Catalog;
use Events;
use strict;
use Debug;

sub revision {'$Revision: 1.6 $'}

sub new {
  my($hm, $pdm) = @_;

  my($self) = {pdm => $pdm};
  bless ($self, 'BrocadeHealth');

# Specify each callback
#                            HM  Callback,  Filters
  $pdm->reportRequest($self, 'all_logic' , {category => Report::CAT_BROCADE});
  return $self;
}

#####################################################

sub all_logic {
  my($hm, $report) = @_;
  my($pdm) = $hm->{pdm};
  my($ev, $ed, $pertains, $x ,$y, $contain, $sd);
  my($orep, $key);
  my($rep)     = $report->content;
  my($oreport) = $pdm->getLastReport($report->fileKey);
  my($ip)      = $report->accessName;
  my($name)    = $report->name;
  my $wwn      = Catalog->deviceRead('brocade', Catalog::IP, $ip, Catalog::Key);
  my($id)      = $report->id('display');
  my($log)     = $report->log();
  if (!$wwn) {
     Debug->err(ERROR => "This Switch \"$name\" ($ip) cannot be identified");
     return;
  }
  TO->setIncomplete() if (!$oreport);

  CIM->version("1.1");
  $DB::single = 1;

  return if ($hm->connectionEvent($wwn, $report, {method => 2} ));
  my($audit); 
  my($freq) = System->get_renv()->{audit_freq} || 7;
  if (!$oreport || ($audit = $pdm->isXdays("Audit$wwn", $freq)) eq "YES" ) {  # new t300 or audit time.
     Catalog->newCatalog();
     my($etype) = $audit ? "brocade.AuditEvent" : "brocade.DiscoveryEvent";
     my($title) = $audit ? "Auditing":"Discovered";
     my($dt)    = "A" if (!$audit);
     $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => $etype ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "$title a new Brocade Switch called $id"],
                         ]);

     $sd = Events->sourceDetector({ event => $ev , host => 1, rep => $rep});
      
     my($p) = NWS::Brocade->newSystem($rep);

     $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $p->[0] ],
                  [ DiscoveryType => $dt],
                    ]);

     my($monitors) = CIM::Instance->new('NWS_AgentMonitors', [
                  [ Agent       => $sd->[0]  ],
                  [ Element     => $p->[0] ],
                    ]);

     $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd,  @$p, $monitors, $pertains ],
                            severity  => Message::SEVERITY_NORMAL });

     $pdm->saveMessage($ed);
     if ($audit) {  # dump stats.

        $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => 'brocade.Statistics'  ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "Statistics about Brocade Switch $id :"],
                  [ Data        => Util->nnl($rep->toStringP('port.\d+.stats'))],
                         ]);

        $sd = Events->sourceDetector({ event => $ev });

        my($system_key) = CIM::Key->new( ['NWS_System', 
                    'Name'            =>$rep->get('id.wwn'), 
                    CreationClassName => 'NWS_System']);

        $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $system_key ],
                    ]);

        $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd, $pertains ],
                            severity  => Message::SEVERITY_NORMAL});

        $pdm->saveMessage($ed);

     }
  }
  return if (!$oreport);


  $orep = $oreport->content;
  $hm->locationChangeEvent($report, $rep, $orep, $wwn);

  my($physicalFrame) = CIM::Key->new( ['CIM_PhysicalFrame',   # key of the frame
                        Tag               => $rep->get("id.wwn") ,
                        CreationClassName => 'CIM_PhysicalFrame' ]);

  my($PORTS) = $rep->get('port.count');

# check if port state has changed

  my($comp);
  for ($x=1; $x <= $PORTS; $x++) {
     $comp = "port.$x"; # . sprintf("%3.3d", $x);
     $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_Card', $comp, $wwn);
  }
#
# check sensors for changes
#
  my($type, $status, $ostatus, $val, $oval);
  foreach my $el ('Fan','Power_Supply','Temp') {
    for ($x=1; $x <= 10; $x++) {
      $type    = $rep->get("sensor.${el}_$x.Type");
      last if (!$type);
      $status  = substr($rep->get("sensor.${el}_$x.Status"),0,-3);
      $ostatus = substr($orep->get("sensor.${el}_$x.Status"), 0, -3);
      $val     = $rep->get("sensor.${el}_$x.Value");
      $oval    = $orep->get("sensor.${el}_$x.Value");
      if ($status ne $ostatus) {
        my($desc0) = "Status of ${el}_$x changed from '$ostatus to '$status, value changed from $oval to $val on $id";
        $hm->alarmEvent( "", $report, $wwn, $desc0);
      }
    }
  }

#
#  generate events for the important error counters 
#
  my($port);
  my($data);
  for ($x=1; $x <= $PORTS; $x++) {
     $port = "port.$x"; 
     my($rep1) = $rep->subset("$port.error");
     foreach my $val (keys %$rep1) {
         my($old) = $orep->get("$port.error.$val");
         my($new) = $rep->get("$port.error.$val");
         if ($new > $old) {
           my($dif) = $new - $old;
           my($tx)  = "Error Counter $val increased by $dif on port $x";
           $data .= "W $tx\n";
         }
     }
  }
  if ($data) {
      chop($data);
      $hm->logEvent($data, $report,  $wwn);
  }

  for ($x=1; $x <= $PORTS; $x++) {
     $port = "port.$x"; 
     my($old) = $orep->get("$port.state.PhyState");
     my($new) = $rep->get("$port.state.PhyState");
     if ($old ne $new) {
        $hm->logEvent("PhysicalState of port $x changed from '$old to '$new", 
                $report,  $wwn, 0);
     }
  }
      
}

  
sub status {
   my($hm, $rep, $orep, $comp) = @_;
   my($pdm) = $hm->{pdm};
   my($map) = $pdm->getDeviceStateMap("brocade.availability");
   Debug->print3("brocade: $comp");

   my($status)  = $rep->getState("$comp.state.OpStatus");
   my($ostatus) = $orep->getState("$comp.state.OpStatus");
   $status      = substr($status,0, -3);
   $ostatus     = substr($ostatus,0,-3);

   my($old)     = $map->get("state.OpStatus.$ostatus");
   my($new)     = $map->get("state.OpStatus.$status");

   return ("$ostatus", "$status",$old, $new );

}

# statiuc method
# return status-state and 0/1
# port.001
# port.002
# ...
# port.016

sub compStatus {
   my($hm, $rep, $orep, $comp) = @_;

   my($map) = PDM->getDeviceStateMap("brocade.availability");

   my($state) = $rep->get("$comp.operational");
   my($status) = $rep->get("$comp.mode");

   my($new) = $map->get("mode.$status") . $map->get("operational.$state");

   $new = ($new eq "11")? 1:0;
   return ( "$status-$state",$new );

}




1;

