package HostHealth;
#  $Name:  $ 
#  $Id: HostHealth.pm,v 1.13 2001/06/15 19:54:46 ccadieux Exp $

 
#             CREATION
####################################################
 
use Health;
use PDM;
use Carp;
use Data::Dumper;
use Report;
use strict;
use CIM::Instance;
use Message;
use Debug;
use TO;
  
use base 'Health';

sub revision {'$Revision: 1.13 $'}
 
sub new {
  my($hm, $pdm) = @_;

  my($self) = {pdm => $pdm};
  bless ($self, 'HostHealth');

# Specify each callback
#                            HM  Callback,  Filters
  $pdm->reportRequest($self, 'all_logic' , {FINAL => 1});
  $pdm->reportRequest($self, 'run_logic' , {category => Report::CAT_HOST} );
  $pdm->reportRequest($self, 'run_mpxio' , {category => Report::CAT_MPXIO} );
  return $self;
}                     

sub run_mpxio {
  my($hm, $report) = @_;
  my($run, $orep, $ompxio);
  my($pdm)     = $hm->{pdm};
  CIM->version("1.1");
  $DB::single = 1;
  my $rep     = $report->content();
  my $mpxio   = $rep->{mpxio};
  my $oreport = $pdm->getLastReport($report->fileKey);
  if ($oreport) {
     $orep    = $oreport->content() ;
     $ompxio  = $orep->{mpxio};
  }
  my $to = TO->readExistingTopo();
  return if (!$to);
 
 
# ONLINE OFFLINE STANDBY
  foreach my $k (keys %$mpxio) {
     my($state, $desc) = split(/\t/, $mpxio->{$k});
     my $v = "MPXIO: $state on $desc";

     my($ostate, $odesc) = split(/\t/, $ompxio->{$k});
     if (!$oreport && ($state eq "OFFLINE")) {
       &saveMpx($to,"t3",$k, Message::SEVERITY_ERROR, $v);

     } elsif ($oreport && ($state ne $ostate)) {
        if ($state eq "OFFLINE") {
           &saveMpx($to,"t3",$k, Message::SEVERITY_ERROR, $v);
        } elsif ($state eq "ONLINE") {
           &saveMpx($to,"t3",$k, Message::SEVERITY_GREEN, $v);
        }
     }
  }

}

sub saveMpx {
  my($to, $cat, $k, $sev, $desc) = @_;
  my($x,$y);
  my $L = $to->storageList();
  my $done = 0;
  for ($x=0; $x <= $#$L; $x++) {
     my $st = $L->[$x];
     next if ($st->{info}{type} ne $cat);
     my $k1 = $st->{info}{name};
     my $pi = $st->{portInfo};
     my $po = $st->{port};
     next if (!$pi);
     for ($y=0; $y <= $#$pi; $y++) {
       if ($pi->[$y]{PortWWN} eq $k) {
          my($t2, $k2) = split(/\:/, $po->[$y]);
          State->saveLinkState($cat, "$k1:" . ($y+1), $t2, $po->[$y],
            $sev, $desc, undef, undef, undef, undef);
          $done = 1 ;last;
       }
     }
     last if ($done);
  }
        
  State->saveState($cat,  $k, "mpxio", undef, $sev, $desc);
}



sub run_logic {
  my($hm, $report ) = @_;

  my($pdm)     = $hm->{pdm};
  CIM->version("1.1");
  $DB::single = 1;
  my($rep)     = $report->content();
  my($packages) = $rep->{showrev};

  my($oreport) = $pdm->getLastReport($report->fileKey);

  my($orep, $desc, $data, $run, $disco, $opackages, $obackup);

#
# BACKUP
#
  if (!$oreport) {
     $run = 1;
  } else {
     $orep  = $oreport->content();
     if ($orep->{backup_config} ne $rep->{backup_config}) {
        $run = 1;
     }
  }
  if ($run) {
       my $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => "host.backup"  ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "NS Agent backup"],
                  [ Data        => $rep->{backup_config}]
                         ]);

       my $sd = Events->sourceDetector({ event => $ev , host => 1, rep => $rep});
 
       my $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $sd->[3]]
                    ]);

       my $ed = Message->new( {  id        => $report->id,
                            instances => [$ev, @$sd , $pertains],
                            severity  => Message::SEVERITY_NORMAL });

       $pdm->saveMessage($ed);
  }

#
#  PACKAGES
#
  $run = 0;
  if (!$oreport) {
     $desc  = "New Patch and Package Information generated";
     $disco = "D";
     foreach my $k (sort keys %$packages) {
       next if (substr($k,0,5) ne "patch" && substr($k,0,5) ne "packa");
       $data .= "ADD=$k|$packages->{$k}\n";
     }
     $run = 1; 
  } else {
     $desc  = "Patch and/or Package Information has changed:";
     $orep  = $oreport->content();
     $opackages =  $orep->{showrev};
     $disco = "A";
     foreach my $k (keys %$packages) {
        next if (substr($k,0,5) ne "patch" && substr($k,0,5) ne "packa");
        if (!$opackages->{$k}) {
           $data .= "ADD=$k|$packages->{$k}\n";
           $run = 1;
        } elsif ($packages->{$k} ne $opackages->{$k}) {
           $data .= "UPDATE=$k|$packages->{$k}\n";
           $run = 1;
        }
     }
     foreach my $k (keys %$opackages) {
        next if (substr($k,0,5) ne "patch" && substr($k,0,5) ne "packa");
        if (!$packages->{$k}) {
           $data .= "DELETE=$k|$opackages->{$k}\n";
           $run = 1;
        }
     }
   }
   if ($run) {
       my $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => "host.patchInfo"  ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => $desc],
                  [ Data        => $data]
                         ]);

       my $sd = Events->sourceDetector({ event => $ev , host => 1, rep => $rep});
 
       my $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $sd->[3]],
                  [ DiscoveryType => $disco],
                    ]);

       my $ed = Message->new( {  id        => $report->id,
                            instances => [$ev, @$sd , $pertains],
                            severity  => Message::SEVERITY_NORMAL });

       $pdm->saveMessage($ed);
   }
}

sub all_logic {
  my($hm ) = @_;
  my($pdm) = $hm->{pdm};
  my($ev, $ed, $x, $pertains , $sd);
  my($HB) = 'heartbeat';
  CIM->version("1.1");
  $DB::single = 1;

#
# RUNS after all health ran.
#
 if ($pdm->getEventCount > 0) {
     $pdm->resetTimer($HB);
 } else {
     my($hb_delay) = System->get_renv()->{heartbeat} || 24; # hours
     $hb_delay *= 60;
     if ($pdm->getTimer($HB) > $hb_delay) { #it's been a day with no transmission.
        $pdm->resetTimer($HB);
        $ev = CIM::Instance->new('NWS_HeartbeatEvent', [
                  [ EventType   => 'agent.HeartbeatEvent' ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Interval    => 24 * 60],
                         ]);
        $sd = Events->sourceDetector({ event => $ev });
        $pertains = CIM::Instance->new('NWS_EventPertainsToSoftwareElement', [
                  [ Event       => $ev  ],
                  [ Element     => $sd->[0] ],
                    ]);
        $ed = Message->new( { id   => {},  # no associated report
                            instances => [$ev, @$sd, $pertains ],
                            severity  => Message::SEVERITY_NORMAL });
        $pdm->saveMessage($ed);
     }
 }

# GET ALL SYSTEM ERRORS AND SEND AS EN EVENT.
 my($sys_errs) = Debug->get_system_errors;

 if ($sys_errs) {
    $ev = CIM::Instance->new('NWS_Event', [
              [ EventType   => 'agent.AlarmEvent' ],
              [ EventId     => $pdm->getEventSequence  ],
              [ Data        => $sys_errs   ],
              [ Description => "rasagent system errors:" ],
                     ]);
    $sd = Events->sourceDetector({ event => $ev });

    $pertains = CIM::Instance->new('NWS_EventPertainsToSoftwareElement', [
              [ Event           => $ev  ],
              [ Element         => $sd->[0] ],
                ]);

    $ed = Message->new( { id   => {},
                        instances => [$ev, @$sd, $pertains ],
                        severity  => Message::SEVERITY_WARNING});
    $pdm->saveMessage($ed);
  }
} 

1;
