package SANHealth;
#  $Name:  $ 
#  $Id: SANHealth.pm,v 1.10 2001/08/24 00:01:10 ccadieux Exp $


use base 'Health';
use NWS::SAN;
use Ilist;
use Message;
use Report;
use Events;
use FCRules;
use strict;
use Debug;
use TO;

sub revision {'$Revision: 1.10 $'}

sub new {
  my($hm, $pdm) = @_;

  my($self) = {pdm => $pdm};
  bless ($self, 'SANHealth');

# Specify each callback
#                            HM  Callback,  Filters
  $pdm->reportRequest($self, 'all_logic' , {category => Report::CAT_SAN});
  return $self;
}

#####################################################

sub all_logic {
  my($hm, $report) = @_;
  my($pdm) = $hm->{pdm};
  my($ev, $ed, $pertains, $x ,$y, $contain, $sd);
  my($orep, $key, $x);
  my($rep)     = $report->content;
  
  $DB::single = 1;
  my($oreport) = $pdm->getLastReport($report->fileKey);
  my($name)    =  $report->accessName;  # name of the A5K, by luxadm

  return if (!$oreport);
  $orep = $oreport->content;

  CIM->version("1.1");

  my($date) = FCRules->today();

  my(@rules) = FCRules->load_rules();
  my($th) = PDM::Thresholds->new;
  $th->init;
  my($new, $old, $found, @o1,  @n1, $o_crc , $n_crc , $r, $ix);
  my(@a, $x0);

  $found = 0;
  my($To) = TO->readTopo();

  my($reads)       = $rep->{reads};
  my($writes)      = $rep->{writes};
#  my($fc_type)     = $rep->{counter_type};  # CRC or ITW

  foreach my $r (keys %$rep) {  
    next if (substr($r, 0, 3) ne "FC_");
    $new = $rep->{$r}{data};
    $old = $orep->{$r}{data};
    next if (!$old);  # this category is not in the old one, skip
    my($info) = { hba => $rep->{$r}{hba}, enc => $rep->{$r}{enc}};
   
    foreach my $k (sort keys %$new) {
       my($hba, $wwn, $comp, $type) = split(/\|/, $k);
       @n1 = split(/\t/, $new->{$k});
       @o1 = split(/\t/, $old->{$k});

       $o_crc = $o1[3]; 
       $n_crc = $n1[3];
  
       if ($n_crc > $o_crc) {
            next if ($type eq "switch" && ($o1[6] == $n1[6]) && ($o1[7] == $n1[7]));
            $found++;
            Debug->print2("CRC went up by " . ($n_crc - $o_crc) . " in $k");
            foreach my $rule (@rules) {
    
               next if ($rule->category() ne "CRC") ;
               my ($error, $event);
               eval {
                 ($error, $event) = $rule->run($To,$hm, $report, $th, $n_crc - $o_crc, 
                                        $k, $info, $reads, $writes, "CRC");
               };
               Debug->err(FC => $@) if ($@);
               Debug->print2($event) if ($event);
            }
       }

       my $o_itw = $o1[5]; 
       my $n_itw = $n1[5];
  
       if ($n_itw > $o_itw) {
            $found++;
            Debug->print2("ITW went up by " . ($n_itw - $o_itw) . " in $k");
            foreach my $rule (@rules) {
               my ($error, $event);
               eval {
                 ($error, $event) = $rule->run($To,$hm, $report, $th, 
                                        $n_itw - $o_itw, $k, $info, 
                                        $reads, $writes, "ITW");
               };
               Debug->err(FC => $@) if ($@);
               Debug->print2($error . " " . $event) if ($event || $error);
            }
       }
    }
  }
  if (!$found) { 
     Debug->print2("<b>Nothing found (no changes in CRC/ITW counters)</b>");
  }

  $th->serialize;
}
1;

