package SwitchHealth;
#  $Name:  $ 
#  $Id: SwitchHealth.pm,v 1.25 2001/07/29 22:00:03 ccadieux Exp $


use base 'Health';
use NWS::Switch;
use Ilist;
use Message;
use Report;
use Catalog;
use Events;
use strict;
use Debug;

sub revision {'$Revision: 1.25 $'}

sub new {
  my($hm, $pdm) = @_;

  my($self) = {pdm => $pdm};
  bless ($self, 'SwitchHealth');

# Specify each callback
#                            HM  Callback,  Filters
  $pdm->reportRequest($self, 'all_logic' , {category => Report::CAT_SWITCH});
  return $self;
}

#####################################################

sub all_logic {
  my($hm, $report) = @_;
  my($pdm) = $hm->{pdm};
  my($ev, $ed, $pertains, $x ,$y, $contain, $sd);
  my($orep, $key);
  my($rep)     = $report->content;
  my($oreport) = $pdm->getLastReport($report->fileKey);
  my($ip)      = $report->accessName;
  my($name)    = $report->name;
  my $wwn      = Catalog->deviceRead('switch', Catalog::IP, $ip, Catalog::Key);
  my($id)      = $report->id('display');
  my($log)     = $report->log();
  if (!$wwn) {
     Debug->err(ERROR => "This Switch \"$name\" ($ip) cannot be identified");
     return;
  }
  TO->setIncomplete() if (!$oreport);

  CIM->version("1.1");
  $DB::single = 1;

  my $method = ($rep->{'id.primary'}) ? 1:2;

  return if ($hm->connectionEvent($wwn, $report, {method => $method} ));
  my($audit); 
  my($freq) = System->get_renv()->{audit_freq} || 7;
  if (!$oreport || ($audit = $pdm->isXdays("Audit$wwn", $freq)) eq "YES" ) {  # new t300 or audit time.
     Catalog->newCatalog();
     my($etype) = $audit ? "switch.AuditEvent" : "switch.DiscoveryEvent";
     my($title) = $audit ? "Auditing":"Discovered";
     my($dt)    = "A" if (!$audit);
     $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => $etype ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "$title a new Switch called $id"],
                         ]);

     $sd = Events->sourceDetector({ event => $ev , host => 1, rep => $rep});
      
     my($p) = NWS::Switch->newSystem($rep);

     $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $p->[0] ],
                  [ DiscoveryType => $dt],
                    ]);

     my($monitors) = CIM::Instance->new('NWS_AgentMonitors', [
                  [ Agent       => $sd->[0]  ],
                  [ Element     => $p->[0] ],
                    ]);

     $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd,  @$p, $monitors, $pertains ],
                            severity  => Message::SEVERITY_NORMAL });

     $pdm->saveMessage($ed);
     if ($audit) {  # dump stats.

        $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => 'switch.Statistics'  ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "Statistics about Switch $id :"],
                  [ Data        => Util->nnl($rep->toStringP('port.\d+.stats'))],
                         ]);

        $sd = Events->sourceDetector({ event => $ev });

        my($system_key) = CIM::Key->new( ['NWS_System', 
                    'Name'            =>$rep->get('id.wwn'), 
                    CreationClassName => 'NWS_System']);

        $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $system_key ],
                    ]);

        $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd, $pertains ],
                            severity  => Message::SEVERITY_NORMAL});

        $pdm->saveMessage($ed);

     }
  }

  $orep = $oreport->content() if ($oreport);

  foreach my $el ('fan.1', 'fan.2', 'power', 'temp') {
      my $s1 = $rep->get("chassis.$el.status");
      my $s2 = $orep? $orep->get("chassis.$el.status") : "";

      if (($s2 && ($s1 ne $s2)) || (!$s2 && ($s1 ne "OK"))) {
         my $desc0;
         if (!$s2) {
            $desc0 = "chassis.$el status is '$s1' on $id";
         } else {
            $desc0 = "chassis.$el status changed from '$s2' to '$s1' on $id";
         }

         $hm->alarmEvent( "", $report, $wwn, $desc0);

         if ($s1 ne "OK") {
             State->saveState("SWITCH", $wwn, "chassis.$el", $ip,
               Message::SEVERITY_ERROR, $desc0);
         } else {
             State->saveState("A5K", $wwn, "chassis.$el", $ip,
               Message::SEVERITY_GREEN, $desc0);
         }
      }
  }

  return if (!$oreport);

  $hm->locationChangeEvent($report, $rep, $orep, $wwn);

  my($physicalFrame) = CIM::Key->new( ['CIM_PhysicalFrame',   # key of the frame
                        Tag               => $rep->get("id.wwn") ,
                        CreationClassName => 'CIM_PhysicalFrame' ]);

  my($PORTS) = $rep->get('version.ports');

# check if port state has changed

  my($comp);
  for ($x=1; $x <= $PORTS; $x++) {
     $comp = "port.$x"; # . sprintf("%3.3d", $x);

     $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_Card', $comp, $wwn);
  }
#
#  generate events for the important counters 
#
  my($port);
  my($data);
  for ($x=1; $x <= $PORTS; $x++) {
     $port = "port.$x"; # . sprintf("%3.3d", $x);
     my($rep1) = $rep->subset("$port.error");
     foreach my $val (keys %$rep1) {
         my($old) = $orep->get("$port.error.$val");
         my($new) = $rep->get("$port.error.$val");
         if ($new > $old) {
           my($level, $cnt, $desc, $mins) = $pdm->threshold()->test('switch',$val, $x, $new-$old);
           my($tx)  = "Port-$x: Received $cnt '$desc' in $mins mins (value=$new)";

           if ($level eq "W" || $level eq "E") {
             $data .= "$tx\n";
           }
         }
     }
  }
  if ($data) {
      chop($data);
      $hm->logEvent($data, $report,  $wwn, "Change in Port Statistics on switch $id:");
  }
  my($ozone, $nzone) ;
  for ($x=1; $x <= $orep->get("zone.total"); $x++) {
    $ozone .= "zone$x: " . $orep->get("zone.$x.members") . ", ";
  }
  for ($x=1; $x <= $rep->get("zone.total"); $x++) {
    $nzone .= "zone$x: " . $rep->get("zone.$x.members") . ", ";
  }
  if ($ozone) {
    chop($ozone); chop($ozone) ;
  }
  if ($nzone) {
    chop($nzone); chop($nzone) ;
  }
  if ($ozone ne $nzone) {
       my($desc) = "Switch $id was rezoned: [$nzone] ";
       $hm->alarmEvent( "", $report, $wwn, $desc);

       my($eid, $ev, $sd);
       $eid  =  $pdm->getEventSequence;
       $ev = CIM::Instance->new('NWS_Event', [
             [ EventType   => "switch.TopologyEvent"  ],
             [ Caption     => "zone-new"],
             [ Description => $desc ],
             [ EventId     => $eid ]]);

       $sd = Events->sourceDetector({ event => $ev });

       my($switch) = CIM::Key->new( ['NWS_System', 
                    'Name'            =>$rep->get('id.wwn'), 
                    CreationClassName => 'NWS_System']);

       my($zones) = NWS::Switch->zones($rep, $switch);

       $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $switch],
                    ]);

       $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd, @$zones, $pertains ],
                            severity  => Message::SEVERITY_NORMAL});

       $pdm->saveMessage($ed);

  }
}

  
sub status {
   my($hm, $rep, $orep, $comp) = @_;
   my($pdm) = $hm->{pdm};
   my($map) = $pdm->getDeviceStateMap("switch.availability");

#   my($state)  = $rep->get("$comp.operational");
#   my($ostate) = $orep->get("$comp.operational");

   my($ostatus) = $orep->getState("$comp.mode");

   if (!$rep->{"$comp.mode"}) {
      $rep->{"$comp.mode"} = $orep->{"$comp.mode"};
      $rep->{"$comp.undefined_count"}++;
      $rep->{"$comp.undefined_start"} = time 
              if ($rep->{"$comp.undefined_count"} == 1);
   } else {
      delete $rep->{"$comp.undefined_count"};
   }
   my($status)  = $rep->getState("$comp.mode");

   my($old) = $map->get("mode.$ostatus"); # . $map->get("operational.$ostate");
   my($new) = $map->get("mode.$status");  # . $map->get("operational.$state");

   return ($ostatus, $status, $old, $new );

}

# statiuc method
# return status-state and 0/1
# port.001
# port.002
# ...
# port.016

sub compStatus {
   my($hm, $rep, $orep, $comp) = @_;

   my($map) = PDM->getDeviceStateMap("switch.availability");

   my($status) = $rep->get("$comp.mode");

   my($new) = $map->get("mode.$status");

   $status  = "UNKNOWN" if (!$status);

   return ( $status, $new );

}




1;

