package NWS::Switch;
#  $Name:  $ 
#  $Id: Switch.pm,v 1.8 2001/06/22 21:04:52 ccadieux Exp $

use Report;
use CIM::Instance;
use Data::Dumper;
use strict;
use Ilist;
use PDM;
use NWS;
sub revision {'$Revision: 1.8 $'}

sub newSystem {
  my ($class, $rep) = @_;
  my(@pil, $i, $dd, $a, $j, $k, $k1, $v);
  my($PORTS);
  my($wwn) = $rep->get('id.wwn');
  my($p1) = CIM::Instance->new('NWS_System',[
                    [Name 				=> $wwn               ],
                    [SystemName         => $rep->get('id.name')],
                    [OtherIdentifyingInfo  => "wwn=$wwn"      ],
                    [IP 				=> $rep->get('version.IP')],
                    [FruCount 			=> $rep->get('version.ports')+1],
                    [Flash              => $rep->get('version.FLASH')],
                    [HW                 => $rep->get('version.HW')],
                    [MacAddress         => $rep->get('version.MAC')],
                    [PROM               => $rep->get('version.PROM')],
                    [Caption            => "Switch"],
                ]);

  my($p2) = CIM::Instance->new('CIM_PhysicalFrame', [
                  [Tag 					=> $rep->get("id.wwn")],
                  [Name					=> $rep->get("id.wwn")],
                  [Manufacturer 		=> "SUN" ],
                  [Model 				=> "Ancor". $rep->get('version.ports')],
                  ]);

  my($a2) = CIM::Instance->new('CIM_ComputerSystemPackage', [
                  [Antecedent            => $p1],
                  [Dependent             => $p2],
                  ]);

  my($l1) = NWS->location($rep);
  
  my($al1) = CIM::Instance->new('NWS_PhysicalFrameLocation', [
                  [Element 				=> $p2],
                  [PhysicalLocation 	=> $l1],
                  ]);
               
  push (@pil, $p1, $p2, $a2, $l1, $al1);

  $PORTS =  $rep->get('version.ports');


  for ($i = 1; $i <= $PORTS; $i++) {
    $dd = $class->newPort($rep,"port.$i");
    $a = CIM::Instance->new('CIM_Container', [
                   [GroupComponent  => $p2],
                   [PartComponent => $dd->[1]],
                   [LocationWithinContainer => "port$i"],
                   ]);
    push (@pil, @$dd);
    push (@pil, $a);
  }
  
  my($zones) = $class->zones($rep, $p1);
  push(@pil, @$zones);

  return Ilist->new(\@pil);
}


sub zones {
  my($class, $rep, $switch) = @_;
  my($i, $a, @pil, $zn);

  foreach my $zt ('hd','ns','sl') {
     for ($zn = 0; $zn <= 100; $zn++) {
       my $v = $rep->get("zone.$zt.$zn.members");
       last if (length($v) <= 1);

       my($z1) = CIM::Instance->new('NWS_Zone', [
               [ZoneId    => $rep->get("id.wwn")  . ".zone.$zt.$zn"] ,
               [ZoneType  => $zt] ,
              ]);
    
       $a = CIM::Instance->new('NWS_SwitchZone', [
                    [ Switch  => $switch ] , 
                    [ Zone    => $z1 ] , 
               ]);

       push(@pil, $z1, $a);

       my(@m) = split(/\s+/, substr($v,0,-1));

       foreach my $member (@m) {
           my($serial_no) = $rep->get("id.wwn") .  ".port.$member";

           my($port) = CIM::Key->new( ['NWS_SwitchPort',
                   SystemName   => "Switch" . $rep->get("version.ports"), 
                   DeviceID     => $serial_no ,
                   CreationClassName => 'NWS_SwitchPort']);

           my($a2) = CIM::Instance->new('NWS_ZonePort', [
                    [ Zone    => $z1 ] , 
                    [ Port    => $port ] , 
               ]);
           push(@pil, $a2);
       }
     }
  }
  return \@pil;
}


sub newPort {
  my($class, $rep, $cd, $no) = @_;
  my(@il, $p1, $c1, $a1);
  
  my($status) = $rep->get("${cd}.mode") . "-" . 
                $rep->get("${cd}.operational");
  my($serial_no) = $rep->get("id.wwn") .  "." . $cd;
  my($map) = PDM->getDeviceStateMap("switch.availability");
  my($state) =  $map->get("mode." . $rep->get("$cd.mode") ) . 
                $map->get("operational." . $rep->get("$cd.operational"));
  $state = ($state eq "11")? 1:0;

  $p1 = CIM::Instance->new('NWS_SwitchPort',  [
                    [Admin        => $rep->get("${cd}.admin")],
                    [Mode         => $rep->get("${cd}.mode")],
                    [Operational  => $rep->get("${cd}.operational")],

             [SystemName   => "Switch" . $rep->get("version.ports") ],
             [DeviceID     => $serial_no ],
             [Name         => $serial_no ],
             [StatusInfo   => $status],
             [Status       => $state ],
             [LossOfSignalCounter => $rep->get("$cd.error.loss_of_signal_cnt") ],
             [LossOfSyncCounter   => $rep->get("$cd.error.sync_loss") ],
             [CRCErrors           => $rep->get("$cd.error.CRC Errs") ],
             [InvalidTransmissionWords => $rep->get("$cd.error.InvalidTxWds") ],
             [FramesTooShort    => $rep->get("$cd.error.short_frame_err_cnt") ],
             [FramesTooLong     => $rep->get("$cd.error.long_frame_err_cnt") ],
                 
                    ]);

  push(@il,$p1);

  $c1 = CIM::Instance->new('CIM_Card', [
                [Removable 			=> "Yes"],
                [Replaceable 		=> $rep->get("${cd}.fruCtlrIsExpendable")],
                [Tag 				=> $serial_no],
                [Name  				=> $serial_no],
                [SerialNumber 		=> $serial_no],
                    ]);

  push(@il, $c1);
  $a1 = CIM::Instance->new('CIM_Realizes', [
                [Antecedent => $c1],
                [Dependent => $p1],
                    ]);
                    
  push(@il,$a1);

  return Ilist->new(\@il);

}

1;
