package PDM::ConfigFile;
#  $Name:  $ 
#  $Id: ConfigFile.pm,v 1.10 2001/04/21 00:10:00 ccadieux Exp $

use System;
use Debug;
sub revision {'$Revision: 1.10 $'}

#  INTERFACE:

#  $c = PDM::ConfigFile->new({ hostid => $hostid});
#  $c->getSlaveConfig : used by http to return a custom config to a slave.
#  $c->read           : used by http and rasagent to get their configfile.
#  $c->write          : used by http to update it's configFile.
#
sub new {
  my($class, $this) = @_;
  $this = {} if (!$this);
  bless($this, 'PDM::ConfigFile');
  return $this;
}

#
# returns a slave's config-file : STRING
# used also by master http
# force_local is for testing where a SLAVE and a MASTER on the same machine.
#
sub getSlaveConfig {
  my($this, $hostid, $force_local) = @_;
  my($config, $renv, $devices, $hosts, $notifs);
  $renv = System->get_renv();
  my($master) = $renv->{hostname};

  ($renv, $devices, $hosts,$notifs) = $this->read($hostid, $force_local);
 
# filter the right host
  foreach $h (@$hosts) {
    if ($h->{hostid} eq $hostid) {
      $name = $h->{hostname};
      foreach $k (keys %$h) {
         if (index(",logfile,t300logfile,contract,", ",$k,") >= 0) {
            $renv->{$k} = $h->{$k}     if ($h->{$k});
         } else {
            $renv->{$k} = $h->{$k};
         }
      }
    }
  }
  return undef if (!$name);

  foreach $x (keys %$renv) {
     $config .= "$x=$renv->{$x}\n";
  }

  foreach $d (@$devices) {
    $d->{datahost} = $d->{host} if (!$d->{datahost});

#   if ($d->{host} eq $name || ($d->{datahost} eq $name) ) {
    $d->{host} = $master if (!$d->{host});

    $config .= "\n[$d->{_name}]\n";
    foreach $x (keys %$d) {
      next if ($x eq "_name");
      $config .= "$x=$d->{$x}\n";
    }
#   }
  }

# no need for notif, slave don't use them

  return $config;
}
 
#
#  returns the  proper configfile
#  used by master-http and all rasagents
#
sub read {
  my($cfg, $hostid, $force_local) = @_;
  my($renv, $devices, @F, $hosts) ;

  $hostid = $cfg->{hostid} if (!$hostid);
  ($err, $config) = $cfg->_request_config($hostid, $force_local);
  if ($err) {
    Debug->print1("Cannot read Configuration file rasagent.conf: $err") ;

  } else {
    @F = split(/\n/, $config);
    return &_parse_config(\@F);
  }
}

#
#   GET CONFIG FILE
#     If you are a slave , read the pushed configfile or get a new one from the master
#     else, get the complete local one
#
sub _request_config {
  my($class, $hostid, $force_local) = @_;
  my($local) = System->get_config();
  my($localC) = System->get_home() . "/System/config";
  my($push) = System->get_config() . ".push";
  my($config);

  my($slave) = Util->findMaster();
  if ($slave && !$force_local) {  
      if (open(O, $push)) {
         while ($l = <O>) {
            $config .= $l;
         }
         close(O);
         return ("", $config);
      } else {  
         Debug->err(TEXT => "Cannot read $push $!");
         return ("NO Config found", undef);
      }
  } else {  # on a master
    if (open(O, $localC)) { # read system config
       while ($l = <O>) {
          $config .= "STATIC.$l";
       }
       close(O);
    }
    if (open(O, $local)) {  # read current config
       while ($l = <O>) {
          $config .= $l;
       }
       close(O);
       return ("", $config);
    } else {
       my($err) = $!;
       Debug->err(CANNOT_READ => "rasagent.conf: $err");
       return ("Cannot read rasagent.conf: $err", undef);
    }
  }
}


sub _parse_config {
  my($F) = @_;
  my($x, $section, $k);
  my(%renv, @devices, %last, @hosts, $host);
  my(@notifs, $notif);
  $section = -1;
  $host = -1;
  $notif = -1;
  $in = "";
  for ($x=0; $x <= $#$F; $x++) {
      $l = $F->[$x];
      next if (substr($l,0,1) eq "#" || $l =~ /^ *$/);
      if ($l =~ /\[(.*)\]/) {
        $n = $1;
        if (substr($n,0,1) eq "h") {
           $host++;
           $hosts[$host]{_name} = $n;
           $in = "h";
        } elsif (substr($n,0,1) eq "n") {
           $notif++;
           $notifs[$notif]{_name} = $n;
           $in = "n";
        } else {
           $section++;
           $devices[$section]{_name} = $n;
           $in = "s";
        }
      } elsif ($l =~ /^([\w\.]+)= *(.*)$/) {
        if ($in eq "") {
          $renv{$1} = $2;

        } elsif ($in eq "n") {
          $notifs[$notif]{$1} = $2;

        } elsif ($in eq "h") {
          $hosts[$host]{$1} = $2;

        } else {
          if ($devices[$section]{$1}) {
             $last{"$section $1"}++;
             $k = $1 . $last{"$section $1"};
             $devices[$section]{$k} = $2;
          } else {
             $devices[$section]{$1} = $2;
             $last{"$section $1"} = 1;
          }
        }
      }
  }
  foreach $k (keys %renv) {
     if ($k =~ /STATIC\.(.*)/) {
        $renv{$1} = $renv{$k};
        delete $renv{$k};
     }
  } 
  return (\%renv, \@devices, \@hosts, \@notifs);
}

sub write {
 my($class, $renv, $devices, $hosts, $notifs) = @_;
 my($file);

 $file = System->get_config();
 $file .= ".push" if (Util->findMaster());

 if (!open(O,">$file")) {
   return undef;
 } else {
   foreach $x (sort keys %$renv) {
     print O "$x=$renv->{$x}\n";
   }
   $cnt = 1;
   foreach $x (@$hosts) {
     next if ($x->{_name} eq "");
     print O "\n[host$cnt]\n";
     $cnt++;
     foreach $y (sort keys %$x) {
        next if ($y eq "_name");
        print O "$y=$x->{$y}\n";
     }
   }
   $cnt = 1;
   foreach $x (@$devices) {
     next if ($x->{_name} eq "");
     print O "\n[device$cnt]\n";
     $cnt++;
     foreach $y (sort keys %$x) {
        next if ($y eq "_name");
        print O "$y=$x->{$y}\n";
     }
   }

   $cnt = 1;
   foreach $x (@$notifs) {
     next if ($x->{_name} eq "");
     print O "\n[notification$cnt]\n";
     $cnt++;
     foreach $y (sort keys %$x) {
        next if ($y eq "_name");
        print O "$y=$x->{$y}\n";
     }
   }
   close(O);
   return 1;
 }
}

1;


#
# last lines of the module...
#
# 1;
#


__END__

=head1 NAME

PDM:ConfigFile - ConfigFile acess class


=head2 SYNOPSIS

 use PDM::ConfigFile

 
  $c = PDM::ConfigFile->new({ hostid => $hostid});
  $c->getSlaveConfig : used by http to return a custom config to a slave.
  my($renv, $devices, $hosts, $notifs) = $c->read();
  $c->write(($renv, $devices, $hosts, $notifs);



=head2 DESCRIPTION

This module read and write a configfile into a set of hashes and arrays.


=head2 CONSTRUCTOR

=over 4

=item new( { hostid => $hostid } )

C<OPTIONS> are passed in a HashPointer.
The possible options are:

B<hostid> - HostId information

=back 4

=head2 METHODS

=over 4

=item read()

Read a configFile


=item write($renv, $devices, $hosts, $notifs)

Write back a config File

=back 4

=head2 AUTHOR

 Christian Cadieux



=head2 COPYRIGHT

Copyright SUN Microsystems


=cut
