package PDM::Thresholds;
#  $Name:  $ 
#  $Id: Thresholds.pm,v 1.10 2001/03/23 00:41:36 ccadieux Exp $

sub revision {'$Revision: 1.10 $'}
use Carp;
use System;
use Debug;

#############################################################
#       SCROLLING WINDOW DATA   
#
# TH : a5000.SCSFRT=cnt,period,quiet, W/E, text
# period,quiet: can be hours(h) or minutes(m).
# created by the PDM
# $th = PDM::Thresholds->new;
# must implement serialize
# Interface:
#  ($level, $cnt, $desc, $mins) = $th->test      Used by Health Monitors
#  if ($level eq "E" || $level eq "W") ...
#     print "Received $cnt $desc messages in $mins mins";
#
#  $th->init      Executed at PDM creation
#  $th->serialize Executed when PDM serialize.
#
# a5000.SFOFFL=  10, 24h, 4h, W, socal/ifp Offline
# a5000.SFOFFL=  10, 10m, 4m, W, socal/ifp Offline
#############################################################

sub new {
  my($class, $arg) = @_;
  my($self) = {};

  bless($self, 'PDM::Thresholds');
  return $self;
}

$CACHE_NAME = "/DATA/thresholds.cache";
%SC_DATA = ();   # data: time, count
%SC_TH = ();     # threshold: cnt, time

sub init {
  my($th) = @_;

  my($home) = System->get_home();
  if (open(O, "$home/$CACHE_NAME")) {
     while ($l = <O>) {
        chop($l);
        @a = split(/\t,\t/, $l);
        $SC_DATA{$a[0]}{$a[1]}{$a[2]} = [$a[3],$a[4]];
     }
  }
  if (open(O, "$home/System/SW_Thresholds")) {
     my(@lines) = <O>; close(O);
     my($x);
     for ($x=0; $x <= $#lines; $x++) {
        $l = $lines[$x];
        chomp($l);
        next if ($l =~ /^ *$/  or substr($l,0,1) eq "#") ;
        @a = split(/ *[=\.\,\:] */, $l, 7);

        my($comment) = "";
        while (substr($lines[$x+1],0,1) eq "'") {
            $comment .= substr($lines[$x+1],1);
            $x++;
        }  
        $SC_TH{$a[0]}{$a[1]} = [$a[2],$a[3],$a[4],$a[5],$a[6], $comment];
     }
  } else {
     Debug->err(NO_SW  => "$!");
     return 0;
  }
  $th->{_sc_data} = \%SC_DATA;
  $th->{_sc_th}   = \%SC_TH;
  return 1;
}

#
#  exist('hck.switch','TXW')  # healthcheck.switch TXW

sub exist {
  my($thr, $cat, $type) = @_;

  my($val,$th) = $thr->_read($cat, $type, 0);
  return $th;
}


# $cnt : represents the numbers of new errors or of new log-entries.
# returns undef when no threshold information exist in the database (SW_T..).
#
#  test('a5k','FCOFF', '12', cnt=2)


sub test {
  my($thr, $cat, $type ,$idx, $cnt) = @_;
  my($val, $th, $hrs, $mins, $min1, $th1, $th2);
  ($val,$th) = $thr->_read($cat, $type, $idx);

  return $thr->_test($th, $val, $cat, $type, $idx, $cnt);
}

#
#  patternTest($lineWithParttern, 'host', 1);
#
sub patternTest {
  my($thr,$text, $cat, $cnt) = @_;
  my($val, $th, $hrs, $mins, $min1, $th1, $th2, $pat);

  foreach $type (keys %{$SC_TH{$cat}} ) {
     $val = $SC_DATA{$cat}{$type}{1};
     $th  = $SC_TH{$cat}{$type};
     $pat = $th->[4];
     if (substr($pat,0,1) eq "/") {
       $pat = substr($pat,1,-2);
     } else {
       next;
     }
     next if ($text !~ /$pat/);

     @ret =$thr->_test($th, $val, $cat, $type, 1,1);
     return @ret if ($#ret >= 0);
  }
}

sub _test {
  my($thr, $th, $val, $cat, $type, $idx, $cnt) = @_;

  my($now) = int(time/60);   # minutes

  if (!$val) { # brand new threshold
     $thr->_write($cat,$type,$idx, [$now,0]);
     ($val,$th) = $thr->_read($cat, $type, $idx);
  }

  $min1 = ($now - $val->[0]); #minutes elapsed

  if ($th->[1] =~ /\d+m/) {
    $th1  = $th->[1] + 0;
  } else { # hours
    $th1  = $th->[1] * 60;
  }
  if ($th->[2] =~ /\d+m/) {
    $th2  = $th->[2] + 0;
  } else { # hours
    $th2  = $th->[2] * 60;
  }
  if ($min1 < 0) {
     return (); # quiet time
  } elsif ($th->[0] == 1 || $min1 < $th1) {  #is it in the range
     $val->[1] += $cnt;
     if ($val->[1] >= $th->[0]) { # counter 
        $mins = $now - $val->[0];
        $ret = $val->[1];
        $val->[0] = $now + $th2;
        $val->[1] = 0;
        my($desc) = $th->[4] || $type;
#                                             th=10  in 24hours
        return ($th->[3], $ret, $desc, $mins, $th->[0], $th->[1]);
     }
  } else {   # reset
     $val->[0] = $now;
     $val->[1] = 0;
  }
  return ();
}


sub _read {
  my($thr, $cat, $type, $idx) = @_;
  my($type2) = $type;
  $type2 =~ s/ /_/g;
  return ($SC_DATA{$cat}{$type}{$idx}, $SC_TH{$cat}{$type2});
}

sub _write {
  my($thr, $cat, $type, $idx, $data) = @_;

  $SC_DATA{$cat}{$type}{$idx} = $data;
}

sub serialize {
  my($thr) = @_;

  open(O, ">" . System->get_home() . $CACHE_NAME);
  $del = "\t,\t";
  foreach $c (sort keys %SC_DATA) {
     $p = $SC_DATA{$c};
     foreach $t (sort keys %$p) {
       $p2 = $p->{$t};
       foreach $i (sort keys %$p2) {
          $val1=$p2->{$i}->[0];
          $val2=$p2->{$i}->[1];
          print O "$c$del$t$del$i$del$val1$del$val2\n";
       }
     }
  }
  close(O);

}

1;

