package ReportContent;
#  $Name:  $ 
#  $Id: ReportContent.pm,v 1.4 2001/06/22 21:04:46 ccadieux Exp $

use Debug;

#
#  $rep = Report->new(...)->content;
#  $rep = Report->fromFile(..)->content;

#  $rep->read('disk1.u1d1.fruState');



sub new {
  my($class, $arg) = @_;

  bless($arg, "ReportContent");

  return $arg;
}

#  $rep->verify(1);  # will complain when accessing a undefined attribute
#  $rep->verify(0);  # will not.

$VERIFY;

sub verify {
  my($arg, $flag) = @_;

  $VERIFY = $flag;
}

sub toString {
  my($rep, $pat) = @_;
  my($o);
  foreach $x (sort keys %$rep) {
     next if ($pat && substr($x,0,length($pat)) ne $pat);
     $o .= "$x = $rep->{$x}\n";
  }
  return $o;
}

sub toStringP {
  my($rep, $pat) = @_;
  my($o);
  foreach $x (sort keys %$rep) {
     if ($x =~ /^$pat/) {
       $o .= "$x = $rep->{$x}\n";
     }
  }
  return $o;
}

sub set {
   my($rep, $name, $value) = @_;
   if ($VERIFY) {
     if (exists($rep->{$name})) {
        $rep->{$name} = $value;
     } else {
        Debug->print0("ReportContent: Attribute '$name' does not exist!");
        return undef;
     }
      
   } else {
     $rep->{$name} = $value;
   }
   return 1;
}

sub getState {
   my($arg, $val) = @_;
   return $arg->{$val} || "[undefined]";
}

sub get {
   my($arg, $val) = @_;

   if ($VERIFY) {
     if (exists($arg->{$val})) {
        Debug->print2("ReportContent: Attribute '$val' empty!") if (!$arg->{$val});
        return $arg->{$val};
     } else {
        Debug->print2("ReportContent: Attribute '$val' does not exist!");
        return undef;
     }
      
   } else {
     return $arg->{$val};
   }
}

# $r->mergeWith($r1, $r2 ...);
#  $config_report->mergeWith($stats_report);

sub merge {
   my($arg, @merge) = @_;

   my($x, $cnt);
   for ($x = 0; $x <= $#merge; $x++) {
       my($r) = $merge[$x];
       foreach $key (keys %$r) {
          $arg->{$key} = $r->{$key};
          $cnt++;
       }
   }  
   return $cnt;
}


# $rep1 = $rep->subset("disk");  # will return all entries that start with 'disk'
sub subset {
   my($arg, $val) = @_;
   my($x, %s, $l);
   $val .= "." if (substr($val, -1) ne ".");
   $l = length($val);

   foreach $x (keys %$arg) {
      if (substr($x,0,$l) eq $val) {
         $s{substr($x, $l)} = $arg->{$x};
      }
   }
   my($sp) = \%s;
   bless($sp, "ReportContent");
   return $sp;
}

1;


__END__

=head1 NAME

ReportContent.pm - ReportContent is a component of a 'Report', contains
 the Config report itself.


=head3 SYNOPSIS

 use Report;
 use ReportContent;

 $repContent = Report->new({ ... })->content;
 $repContent->get('disk.u1d1.State');
 $disk = $repContent->subset('disk.u1d1');
 $disk->get('u1d1.State');



=head3 DESCRIPTION

This module allow access to a reportContent. Sub-reports can be made from an existing
reportContent.



=head3 CONSTRUCTOR

=over 4

=item new( <HASHREF> )


=back 4


=head3 METHODS

=over 4

=item verify([0/1]);

with verify(1), using 'get' will complain if the key does not exist.

=item get(<key>);

returns the value of 'key' in the report.
 
 Exampl: print $report->get('State');

=item subset();

returns a reportContent with a subset of the original reportContent.

  Example: $diskContent = $content->subset('disk.u1d1');

=item mergeWith();

Join the current report with other reports passed as args.

  Example: $config_obj->mergeWith($stats_report);



=back 4

=head3 SEE ALSO




=head3 AUTHOR

 Christian Cadieux (ccadieux@central.sun)



=head3 COPYRIGHT

Copyright (c) 2000 Sun Microsystems

=cut
