package System;
#  $Name:  $ 
#  $Id: System.pm,v 1.21 2001/06/28 23:08:48 ccadieux Exp $

use Debug;
use strict 'vars';
use vars qw($AUTOLOAD %AUTO); 
use System;
use Util;
use PDM;
use CIM;
use Date::Format;
use Time::Zone;


use vars qw( $PATH %COMM %COUNTERS);
$PATH = "/bin:/usr/bin:/usr/sbin";

#
#  System->counter("discovery", "ZERO");
#  System->counter("discovery", 1);
#  System->counter("discovery");
#
sub counter {
  my($class, $tag, $inc) = @_;
  
  if ($inc eq "ZERO") {
     $COUNTERS{$tag} = 0;

  } elsif ($inc) {
     $COUNTERS{$tag} += $inc;
  }
  return $COUNTERS{$tag};
}

sub srs_time {
  my($class) = @_;

  my($o) =  tz_local_offset();

  $o = sprintf("%+03d%02d", int($o / 3600), abs(int($o % 3600)));
  my($s) = Date::Format::time2str('%Y%m%d.%H%M%S', time) . substr($o,0,3);
  return $s;
}


sub appletHeader {
  my($class, $app, $offx, $off)  = @_;

  my $o =<<EOF;
 <script language=JavaScript>
  var a_width;
  var a_height;
  if (navigator.appName.indexOf("Netscape") != -1) {
      a_width = window.innerWidth - $offx;
      a_height = window.innerHeight- $off;
  } else {
      a_width = window.document.body.offsetWidth - $offx;
      a_height = window.document.body.offsetHeight - $off;
  }
  var s = '<applet $app height=' +a_height+ ' width=' +a_width + '>';
  window.document.writeln(s);
  window.document.writeln('');
  </script>
EOF

  return $o;
}

#
# runs only once then returns the cache value
#
sub run_command {
  my($com) = @_;
  my(@a, $dir, $val);

  return $COMM{$com} if $COMM{$com};

  my($ix)   = index($com, " ");
  my($com0) = ($ix >= 0)? substr($com, 0,$ix) : $com;

  if (substr($com,0,1) eq "/") {
     if (-x "$com0") {
        $val = `$dir/$com`;
        chop($val);
        $COMM{$com} = $val;
        return $val;
     } else {
        return undef;
     }
  }
    
  @a = split(/:/, $PATH);
  foreach $dir (@a) {
     if (-x "$dir/$com0") {
        $val = `$dir/$com`;
        chop($val);
        $COMM{$com} = $val;
        return $val;
     }
  }
  
  return undef;
}

sub grep {
  my($class, $file, $pat) = @_;
  my($l, $ret);

  open(O, $file);
  while ($l = <O>) {
    chop($l);
    if ($l =~ /$pat/) {
      $ret .= "| " if ($ret);
      $ret .= $l;
    }
  }
  close(O);
  return $ret;
}
  

sub get_service_port {
  my($class, $service) = @_;
  my($sport);
  my($x) = `/usr/bin/grep $service /etc/services`;
  if ($x) {
    $x =~ /$service\s+(\d+)/;
    $sport = $1;
  }
  return $sport;
}

sub get_config {
  my($class) = @_;

  if (!defined($AUTO{config})) {
    return System->get_home . "/DATA/rasagent.conf";
  } else {
    return $AUTO{config};
  }
}


sub find_command {
  my($class, $com) = @_;
  my(@a) = split(/:/, $PATH);
  foreach my $dir (@a) {
      return "$dir/$com" if (-x "$dir/$com");
  }
  Debug->err(ERROR => "Cannot find command $com");
  return "echo ";
}
         
sub host_type {
  my($class, $hostname) = @_;
  my($type);
  my($ans) = run_command(System->get_home() . 
                "/bin/snmpget $hostname public 1.1.0 2>/dev/null");
  if (!$ans) {
     return "";
  } else {
    my(@a) = split(/\s*=\s*/, $ans);
    $a[1] =~ s/Sun SNMP Agent,//;
    $type = Util->ltrim($a[1]);

    if (substr($type,0,1) eq "\"") {
       $type = substr($type,1,-1);
    }
    return $type;
  }

}

sub bitMode {
  my($class) = @_;
  my($bit);
  if (-x "/usr/bin/isainfo") {
     $bit = run_command('isainfo -b') || 'UNKNOWN';
     return $bit;
  } else {
     return "32";
  }
}
     

#    .       system boot  Apr 12 09:42

sub lastReboot {
  my($class) = @_;

  my $x = run_command('who -b');
  $x =~ /system boot (.*)/;
  $x = Util->trim($1);
  my ($m, $d, $h) = split(/\s/, $x);

  $m = $Util::MTH{$m};
  my $boot = sprintf("%2.2d-%2.2d %s:00", $m, $d, $h);

  my $today = Util->today("YMDH");
  my $today1 = substr($today, 5);

  if ($boot > $today1) {
     my $year = substr($today,0,4) - 1;
     return sprintf("%4.4d-", $year) . $boot
  } else {
     return substr($today,0,5) . $boot;
  }
  
}


sub os_version {
  my($class) = @_;
  my($os_version);

  return run_command('uname -r') || 'UNKNOWN';
}
  
sub hostid {
  my($class) = @_;
  return  run_command('hostid') || 'UNKNOWN';
}


sub hostinfo {
  my($class) = @_;
  return  run_command('uname -a') || 'UNKNOWN';
}

sub shortHostname {
  my($class) = @_;
  my($h) = run_command('uname -n') || 'UNKNOWN';
  my $i = index($h, ".");
  if ($i > 0) {
     return substr($h,0,$i);
  } else {
     return $h;
  }
}

sub hostname {
  my($class) = @_;
  my($h) = run_command('uname -n') || 'UNKNOWN';

  my $i = index($h, ".");

  if ($i > 0) {
     return substr($h,0,$i) . lc(substr($h,$i));
  } else {
     my $domain = $class->domain();
     if ($domain) {
        return $h . "." . lc($domain);
     } else {
        return $h;
     }
  }

}

sub os {
  my($class) = @_;

  return run_command('uname -s') || 'UNKNOWN';
}

sub model {
  my($class) = @_;

  return run_command('uname -i') || 'UNKNOWN';
}

sub memory {
  my($class) = @_;
  my($x, $mem);
  $x = run_command('prtconf -v');
  $x =~ /Memory size: (\d+)\s/;
  $mem = $1 || "UNKNOWN";
  return $mem;
}

sub swap {
  my($class) = @_;
  my($swap, $x);
  $x = run_command(System->get_home() . "/bin/swap.sh");
  $x =~ /SWAP: (\d+) total/;
  $swap = $1 || "UNKNOWN";
  return $swap;
}

sub cpus {
  my($class) = @_;
  my(@x) = split(/\n/, run_command('psrinfo'));
  return $#x + 1;
}



sub hostInfo {
  my($class) = @_;
  my($h) = PDM->getCacheHandle("HostInfo");

  if (PDM->isXdays("hostInfo", 1) || !$h->{hostid}) {
    $h->{hostid}   = $class->hostid();
    $h->{hostinfo} = $class->hostinfo();
    $h->{hostname} = $class->hostname();
    $h->{os}       = $class->os();
    $h->{os_version} = $class->os_version();
    $h->{model}    = $class->model();
    $h->{swap}     = $class->swap();
    $h->{memory}   = $class->memory();
    $h->{domain}   = $class->domain();
    $h->{cpus}     = $class->cpus();
    $h->{ip}       = Util->name2ip($h->{hostname} );
    $h->{tz}       = CIM->tz();
  }
  return $h;
}
  
sub find_executable {
  my($class, $bin) = @_;
  my($dir);

  my($home) = System->get_home();

  my(@a) = split(/:/, "/usr/sbin:/usr/bin:/bin:$home/bin");
  foreach $dir (@a) {
     if (-x "$dir/$bin") {
        return "$dir/$bin";
     }
  }
  return undef;
}


sub find_luxadm {
  my($class) = @_;
  my($dir);

  my($home) = System->get_home();

  my(@a) = split(/:/, "/usr/sbin:/usr/bin:/bin:$home/bin");
  foreach $dir (@a) {
     if (-x "$dir/luxadm") {
        return "$dir/luxadm";
     }
  }
}

sub find_disk_inquiry {
  my($class) = @_;
  my(@a, $dir);

  my($home) = System->get_home();

  @a = split(/:/, "$home/bin:/opt/STORtools:/usr/bin:/bin");
  foreach $dir (@a) {
     if (-x "$dir/luxadm") {
        return "$dir/luxadm";
     }
  }
}

sub domain {
  my($class) = @_;
  my $do;
  require Net::Domain;
  return Net::Domain::hostdomain();

  if ($do) {
     return $do;
  } else {
     $do = run_command('domainname') || 'UNKNOWN';
     my $ix = index($do, ".");
     return substr($do,$ix+1);
  }
  
}

sub host_domain {
  my($class) = @_;
  require Net::Domain;
  return Net::Domain::hostfqdn();
}

# inet addr:10.10.10.10  Bcast:10.10.255.255  Mask:255.255.0.0

sub local_ip {
  my($class) = @_;
  my($if, @a, $x, $i, $ip);

  if ($class->os() =~ /Linux/) {
    @a = `/sbin/ifconfig -a`;
    for ($x = 0; $x <= $#a; $x++) {
       if ($a[$x] =~ /inet addr\:(.*) Bcast\:/) {
          return $1;
       }
    }
    return "Unknown";
  }
  @a = `/usr/sbin/ifconfig -a`;
     
  for ($x = 1; $x <= $#a; $x += 2) {
     $i = $a[$x];
     $i =~ /inet ([\d\.]+)/;
     $ip = $1;
     last if ($ip ne "127.0.0.1");
  }
  return $ip;
}


sub get_hba {
  my($class) = @_ ;
  my($hba, $sbus, $pci) ;

#  chomp( $sbus = `/usr/sbin/prtconf -D | grep "SUNW,socal"` );
#  chomp( $pci = `/usr/sbin/prtconf -pv | grep "vendor-id:  00001077"` );

  if ($pci) {
     $hba = "P";
  } elsif ($sbus)  {
     $hba = "S";
  } else {
     $hba = "S";
  }
  return $hba;


}

sub readModules {
  my($class, $dir, $skip, $prefix) = @_;
  my($d0, @mods, $d);

  if (opendir(O, System->get_home . "/lib/$dir")) {
    while ($d0 = readdir(O)) {
       next if (substr($d0,-3) ne ".pm");
       $d = substr($d0, 0, -3);
       next if ($skip && $d eq $skip);
       next if ($prefix && substr($d,0,length($prefix)) ne $prefix);
       push(@mods, $d);
    }
    closedir(O);
    return @mods;
  } else {
    return ();
  }
}




#########################
# ENVIRONMENT
# save variables
#########################


sub AUTOLOAD {
  my($class, $val) = @_;
  my($key);

  if ($AUTOLOAD =~ /::set_(.*)/ ) {
       $key = $1;
       $AUTO{$key} = $val;
       return $val;
  } elsif ($AUTOLOAD =~ /::get_(.*)/ ) {
       $key = $1;
       if (!defined($AUTO{$key})) {
           print "set_$key not defined in System->get\n" if ($Debug::LEVEL > 2);
       }
       return $AUTO{$key};
  } else {
       return undef;
  }
}

1;
